import{Ii as t,Ji as D}from"./chunk-GDJMZZ4C.js";import{Rd as r,sa as s}from"./chunk-4HFZZ7HS.js";import{e}from"./chunk-SZAW6ZBW.js";var o,c=e(()=>{o=(()=>{class n{static{this.connectionStatusArchivedID="{333FB448-F180-448E-BE46-4773AD0F1679}"}static{this.connectionStatusBrokenID="{3B07FDA8-FF2A-428B-8D96-24D77365A47E}"}static{this.connectionStatusDefaultID="{B299F468-BC24-43cb-9417-4FEDFC813E7C}"}static{this.connectionStatusDisabledID="{380B6ED9-94F9-4e1e-B4F6-72714B9D44CD}"}static{this.connectionStatusExpiredID="{7B926104-9C7E-4c91-BDAC-DFCCC36B8D70}"}static{this.connectionStatusLockedID="{DE4018E0-38E4-416b-AB4E-DEA6B754C711}"}static{this.connectionStatusPendingApprovalID="{6240196F-F8B0-42BA-A256-C5B5B130CE3B}"}static{this.connectionStatusRootEnforceDefaultID="{5E97362E-44C9-4082-B85A-353AC7C3A982}}"}static{this.connectionStatusWarningID="{123A44CB-7EDC-4ecb-926B-031793668148}"}static{this.logStatusDefaultID="{3B6C0824-BB78-4d01-A2C0-00549E9D034A}"}static{this.logStatusNoChargeID="{57228922-D7C5-44b4-9975-53FAA747D3E2}"}static{this.logStatusPaidID="{810D29EC-97BE-45bf-8252-88B8C5DC9B27}"}static{this.logStatusStarredID="{F913F5B7-4022-4802-A726-A9798EE6F8DD}"}static{this.logStatusToBillID="{86A84E83-8B66-4688-BA06-AC079185AEBE}"}static{this.logStatusToVerifyID="{45B108AD-30D5-4516-A3CC-914D5FC148D4}"}toQueryString(){return""}constructor(i=null){this.clone=()=>new n(this)}}return n})()});var d,B=e(()=>{D();c();r();d=(()=>{class n{constructor(){this.guidToEnumMap=new Map([[o.connectionStatusDefaultID,t.Default],[o.connectionStatusExpiredID,t.Expired],[o.connectionStatusBrokenID,t.Broken],[o.connectionStatusLockedID,t.Locked],[o.connectionStatusDisabledID,t.Disabled],[o.connectionStatusWarningID,t.Warning],[o.connectionStatusArchivedID,t.Archived],[o.connectionStatusPendingApprovalID,t.PendingApproval]])}mapGuidToEnum(i){return this.guidToEnumMap.get(i)??t.Default}mapEnumToGuid(i){return this.enumToGuidMap||(this.enumToGuidMap=new Map(Array.from(this.guidToEnumMap.entries()).map(([a,u])=>[u,a]))),this.enumToGuidMap.get(i)??""}setDefaultStatus(i){return this.mapEnumToGuid(i.value)}getDefaultStatus(i){return this.mapGuidToEnum(i)}static{this.\u0275fac=function(a){return new(a||n)}}static{this.\u0275prov=s({token:n,factory:n.\u0275fac,providedIn:"root"})}}return n})()});export{o as a,c as b,d as c,B as d};
