import{a as n,b as e}from"./chunk-CXJOJS5Y.js";import"./chunk-ILBO5ZGN.js";import"./chunk-MWQIPWDG.js";import"./chunk-UTF7W4HP.js";import"./chunk-OJRDBZWF.js";import"./chunk-6G52AAKU.js";import"./chunk-QQZCALHS.js";import"./chunk-EXP5MDOK.js";import"./chunk-M55KBHC5.js";import"./chunk-JE2X3YFA.js";import"./chunk-HW6LKGMV.js";import"./chunk-GDJMZZ4C.js";import"./chunk-V3J5YJ3L.js";import"./chunk-UIVS6623.js";import"./chunk-2XIOYYFS.js";import"./chunk-HYPRVELK.js";import"./chunk-YTY7HHO3.js";import"./chunk-UOI75ONS.js";import"./chunk-4HFZZ7HS.js";import{e as t}from"./chunk-SZAW6ZBW.js";var m,p=t(()=>{e();m=[{path:":repositoryId/:connectionId",canActivate:[n],loadComponent:()=>import("./chunk-PV3TU2QG.js").then(o=>o.WebClientComponent),children:[{path:":sessionId",loadComponent:()=>import("./chunk-6K2NLBGL.js").then(o=>o.WebClientPlaceholderComponent)},{path:"ard",loadComponent:()=>import("./chunk-WXIZ3W47.js").then(o=>o.WebClientArdComponent)},{path:"rdp",loadComponent:()=>import("./chunk-LDPJVOFX.js").then(o=>o.WebClientRdpComponent)},{path:"vnc",loadComponent:()=>import("./chunk-6TXX64MG.js").then(o=>o.WebClientVncComponent)},{path:"ssh",loadComponent:()=>import("./chunk-B22JOO5D.js").then(o=>o.WebClientSshShellComponent)},{path:"telnet",loadComponent:()=>import("./chunk-552NYOJO.js").then(o=>o.WebClientTelnetComponent)},{path:"powershell",loadComponent:()=>import("./chunk-WE6DRF7Z.js").then(o=>o.WebClientPowerShellComponent)}]}]});p();export{m as WEB_CLIENT_ROUTES};
