import{yd as r,zd as o}from"./chunk-GDJMZZ4C.js";import{e as n}from"./chunk-SZAW6ZBW.js";var s,u=n(()=>{o();s=class e extends r{toQueryString(){let t=[];return t.push(super.toQueryString()),t.join("&")}constructor(t=null){super(t),this.clone=()=>new e(this)}}});export{s as a,u as b};
