import{i as e,j as o}from"./chunk-2JML2BCE.js";import{e as n}from"./chunk-SZAW6ZBW.js";var l=n(()=>{o();e.setLanguage(["nl","nl-NL"],{Markdown:"Markdown",WYSIWYG:"WYSIWYG",Write:"Opslaan",Preview:"Voorbeeld",Headings:"Koppen",Paragraph:"Alinea",Bold:"Vet",Italic:"Cursief",Strike:"Doorhalen",Code:"Inline code",Line:"Regel",Blockquote:"Citaatblok","Unordered list":"Opsomming","Ordered list":"Genummerde opsomming",Task:"Taak",Indent:"Niveau verhogen",Outdent:"Niveau verlagen","Insert link":"Link invoegen","Insert CodeBlock":"Codeblok toevoegen","Insert table":"Tabel invoegen","Insert image":"Afbeelding invoegen",Heading:"Kop","Image URL":"Afbeelding URL","Select image file":"Selecteer een afbeelding","Choose a file":"Kies een bestand","No file":"Geen bestand",Description:"Omschrijving",OK:"OK",More:"Meer",Cancel:"Annuleren",File:"Bestand",URL:"URL","Link text":"Link tekst","Add row to up":"Voeg rij toe aan omhoog","Add row to down":"Rij naar beneden toevoegen","Add column to left":"Voeg kolom aan de linkerkant toe","Add column to right":"Voeg een kolom aan de rechterkant toe","Remove row":"Rij verwijderen","Remove column":"Kolom verwijderen","Align column to left":"Links uitlijnen","Align column to center":"Centreren","Align column to right":"Rechts uitlijnen","Remove table":"Verwijder tabel","Would you like to paste as table?":"Wil je dit als tabel plakken?","Text color":"Tekstkleur","Auto scroll enabled":"Autoscroll ingeschakeld","Auto scroll disabled":"Autoscroll uitgeschakeld","Choose language":"Kies een taal"})});l();
