import{a as s,b as g}from"./chunk-6Q7YU3GA.js";import{q as a,v as h}from"./chunk-HW6LKGMV.js";import{e as n,f as l,g as c,h as f}from"./chunk-V3J5YJ3L.js";import{Rd as v,sa as o,xa as i}from"./chunk-4HFZZ7HS.js";import{e as p}from"./chunk-SZAW6ZBW.js";var O,m=p(()=>{l();v();h();g();f();O=(()=>{class r{constructor(t,e,d){this.router=t,this.longOperationIndicatorService=e,this.errorService=d}navigate(t){this.longOperationIndicatorService.showIndicator("standard"),this.router.navigateByUrl(t).then(()=>this.longOperationIndicatorService.hideIndicator("standard"),()=>{this.errorService.pushError(new n({errorType:"client",exceptionMessage:"navigation error",fatal:!1,message:`could not navigate to ${t}`})),this.longOperationIndicatorService.hideIndicator("standard")})}navigateToRoot(){return void this.router.navigateByUrl("/")}static{this.\u0275fac=function(e){return new(e||r)(i(a),i(s),i(c))}}static{this.\u0275prov=o({token:r,factory:r.\u0275fac,providedIn:"root"})}}return r})()});export{O as a,m as b};
