import{Rd as a,g as r,ma as o,sa as s}from"./chunk-4HFZZ7HS.js";import{e as c}from"./chunk-SZAW6ZBW.js";var d,u=c(()=>{o();a();d=(()=>{class e{constructor(){this.editRequestSource=new r,this.createRequestSource=new r,this.editRequest=this.editRequestSource.asObservable(),this.createRequest=this.createRequestSource.asObservable()}edit(t){this.editRequestSource.next(t)}create(t){this.createRequestSource.next(t)}static{this.\u0275fac=function(i){return new(i||e)}}static{this.\u0275prov=s({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});export{d as a,u as b};
