import{Rd as l,d as r,g as s,ma as i,sa as c}from"./chunk-4HFZZ7HS.js";import{e as o}from"./chunk-SZAW6ZBW.js";var b,a=o(()=>{i();l();b=(()=>{class e{constructor(){this.selectTreeElement=new r,this.selectTreeElementSource=new s,this.selectTreeElement=this.selectTreeElementSource.asObservable()}select(t){return t.key=t.key||"global_tree_selector",this.selectTreeElementSource.next(t),this}static{this.\u0275fac=function(n){return new(n||e)}}static{this.\u0275prov=c({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});export{b as a,a as b};
