import{a as c}from"./chunk-YTY7HHO3.js";import{Rd as f,Tb as s}from"./chunk-4HFZZ7HS.js";import{e as p,h as a}from"./chunk-SZAW6ZBW.js";var n,m,M,d=p(()=>{n=a(c());f();m=n.default,M=(()=>{class i{transform(e,t,o=!1){let r="";return(!t||t?.trim().length===0)&&(t="dddd, MMMM Do YYYY, h:mm:ss a"),e&&e!==""&&(o?r=m(e).format(t):r=m.utc(e).local().format(t)),r}static{this.\u0275fac=function(t){return new(t||i)}}static{this.\u0275pipe=s({name:"date",type:i,pure:!1})}}return i})()});export{M as a,d as b};
