import{i as e,j as i}from"./chunk-2JML2BCE.js";import{e as o}from"./chunk-SZAW6ZBW.js";var a=o(()=>{i();e.setLanguage(["pl","pl-PL"],{Markdown:"Markdown",WYSIWYG:"WYSIWYG",Write:"Napisz",Preview:"Podgl\u0105d",Headings:"Nag\u0142\xF3wki",Paragraph:"Akapit",Bold:"Pogrubienie",Italic:"Kursywa",Strike:"Przekre\u015Blenie",Code:"Fragment kodu",Line:"Linia",Blockquote:"Cytat","Unordered list":"Lista nieuporz\u0105dkowana","Ordered list":"Lista uporz\u0105dkowana",Task:"Zadanie",Indent:"Utw\xF3rz wci\u0119cie",Outdent:"Usu\u0144 wci\u0119cie","Insert link":"Umie\u015B\u0107 odno\u015Bnik","Insert CodeBlock":"Umie\u015B\u0107 blok kodu","Insert table":"Umie\u015B\u0107 tabel\u0119","Insert image":"Umie\u015B\u0107 obraz",Heading:"Nag\u0142\xF3wek","Image URL":"Adres URL obrazu","Select image file":"Wybierz plik obrazu","Choose a file":"Wybierz plik","No file":"Brak plik",Description:"Opis",OK:"OK",More:"Wi\u0119cej",Cancel:"Anuluj",File:"Plik",URL:"URL","Link text":"Tekst odno\u015Bnika","Add row to up":"Dodaj wiersz do g\xF3ry","Add row to down":"Dodaj wiersz w d\xF3\u0142","Add column to left":"Dodaj kolumn\u0119 po lewej stronie","Add column to right":"Dodaj kolumn\u0119 po prawej stronie","Remove row":"Usu\u0144 rz\u0105d","Remove column":"Usu\u0144 kolumn\u0119","Align column to left":"Wyr\xF3wnaj do lewej","Align column to center":"Wy\u015Brodkuj","Align column to right":"Wyr\xF3wnaj do prawej","Remove table":"Usu\u0144 tabel\u0119","Would you like to paste as table?":"Czy chcesz wklei\u0107 tekst jako tabel\u0119?","Text color":"Kolor tekstu","Auto scroll enabled":"W\u0142\u0105czono automatyczne przewijanie","Auto scroll disabled":"Wy\u0142\u0105czono automatyczne przewijanie","Choose language":"Wybierz j\u0119zyk"})});a();
