import{a as p,b as g}from"./chunk-M55KBHC5.js";import{P as m,Q as I}from"./chunk-JE2X3YFA.js";import{Wj as c,Xj as j}from"./chunk-GDJMZZ4C.js";import{g as n,h as d}from"./chunk-V3J5YJ3L.js";import{b as o,c as S}from"./chunk-YTY7HHO3.js";import{Rd as h,na as v,sa as a,x as s,xa as i}from"./chunk-4HFZZ7HS.js";import{e as u}from"./chunk-SZAW6ZBW.js";var w,T=u(()=>{g();v();h();S();d();I();j();w=(()=>{class t extends p{constructor(e,r,f,l){super(e,r,f),this.assetService=l}getTimezones(e=!1){return this.getCache(e,this.getTimezonesInternal())}getTimezonesInternal(){return this.assetService.GetTimezones().pipe(s(e=>e.data.map(r=>({label:r.DisplayName,value:r.Id}))))}static{this.\u0275fac=function(r){return new(r||t)(i(o),i(n),i(m),i(c))}}static{this.\u0275prov=a({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});export{w as a,T as b};
