import{Rd as d,g as i,ma as c,sa as r}from"./chunk-4HFZZ7HS.js";import{e as a}from"./chunk-SZAW6ZBW.js";var l,p=a(()=>{c();d();l=(()=>{class s{constructor(){this.dialogRequestSubject=new i,this.dialogRequest=this.dialogRequestSubject.asObservable()}showDialog(e){this.dialogRequestSubject.next(e)}askForComment(e,t,o="credentialViewed"){this.showDialog({commentType:o,required:{ticketNumber:this.getPromptCommentOptions(e,o)?.ticketNumberRequired,comment:this.getPromptCommentOptions(e,o).commentRequired},name:e?.name,connectionId:e?.id,accept:n=>t?t(n):()=>{}})}getPromptCommentOptions(e,t){switch(t){case"credentialViewed":return e?.resolvedEventsLogsOptions?.credentialsViewedLogsOptions;case"open":return e?.resolvedEventsLogsOptions?.openLogsOptions;case"close":return e?.resolvedEventsLogsOptions?.closeLogsOptions}}static{this.\u0275fac=function(t){return new(t||s)}}static{this.\u0275prov=r({token:s,factory:s.\u0275fac,providedIn:"root"})}}return s})()});export{l as a,p as b};
