import{a as M,b as A,c as F}from"./chunk-OZYXV5FT.js";import{Ha as a,La as x}from"./chunk-2XIOYYFS.js";import{K as P,t as Z}from"./chunk-UOI75ONS.js";import{Ab as m,Bc as T,Gc as b,Hc as d,Kb as _,Kd as r,Ld as f,Nd as c,Qb as v,Rd as I,Uc as w,Wa as C,Wb as p,Wc as k,Xa as n,jc as l,mc as y,md as h,rc as u,wc as g,xc as D}from"./chunk-4HFZZ7HS.js";import{e as E}from"./chunk-SZAW6ZBW.js";function S(t,o){if(t&1&&T(0,3),t&2){let s=o.$implicit,i=d(2);l("ngTemplateOutlet",i.messageTemplate().template)("ngTemplateOutletContext",h(2,Q,s,s))}}function Y(t,o){t&1&&p(0,S,1,5,"ng-template",1)}function q(t,o){if(t&1&&T(0,3),t&2){let s=o.$implicit,i=d(2);l("ngTemplateOutlet",i.headlessTemplate().template)("ngTemplateOutletContext",h(2,Q,s,s))}}function N(t,o){t&1&&p(0,q,1,5,"ng-template",2)}var Q,U,V=E(()=>{I();P();F();x();I();F();x();Q=(t,o)=>({$implicit:t,message:o});U=(()=>{class t{constructor(){this.key=n(),this.position=n("top-right"),this.life=n(3e3,{transform:f}),this.baseZIndex=n(0,{transform:f}),this.autoZIndex=n(!0,{transform:r}),this.preventDuplicates=n(!1,{transform:r}),this.preventOpenDuplicates=n(!1,{transform:r}),this.style=n(),this.styleClass=n(),this.breakpoints=n(),this.showTransformOptions=n("translateY(100%)"),this.hideTransformOptions=n("translateY(-100%)"),this.showTransitionOptions=n("300ms ease-out"),this.hideTransitionOptions=n("250ms ease-in"),this.onClose=C(),this.templates=_(a),this.messageTemplate=c(()=>this.templates().find(s=>s.name==="message")),this.headlessTemplate=c(()=>this.templates().find(s=>s.name==="headless"))}static{this.\u0275fac=function(i){return new(i||t)}}static{this.\u0275cmp=v({type:t,selectors:[["dvl-toast"]],contentQueries:function(i,e,O){i&1&&w(O,e.templates,a,4),i&2&&k()},inputs:{key:[1,"key"],position:[1,"position"],life:[1,"life"],baseZIndex:[1,"baseZIndex"],autoZIndex:[1,"autoZIndex"],preventDuplicates:[1,"preventDuplicates"],preventOpenDuplicates:[1,"preventOpenDuplicates"],style:[1,"style"],styleClass:[1,"styleClass"],breakpoints:[1,"breakpoints"],showTransformOptions:[1,"showTransformOptions"],hideTransformOptions:[1,"hideTransformOptions"],showTransitionOptions:[1,"showTransitionOptions"],hideTransitionOptions:[1,"hideTransitionOptions"]},outputs:{onClose:"onClose"},decls:3,vars:17,consts:[[3,"onClose","key","position","life","baseZIndex","autoZIndex","preventDuplicates","preventOpenDuplicates","styleClass","breakpoints","showTransformOptions","hideTransformOptions","showTransitionOptions","hideTransitionOptions"],["pTemplate","message"],["pTemplate","headless"],[3,"ngTemplateOutlet","ngTemplateOutletContext"]],template:function(i,e){i&1&&(g(0,"p-toast",0),b("onClose",function($){return e.onClose.emit($)}),p(1,Y,1,0,null,1)(2,N,1,0,null,2),D()),i&2&&(y(e.style()),l("key",e.key())("position",e.position())("life",e.life())("baseZIndex",e.baseZIndex())("autoZIndex",e.autoZIndex())("preventDuplicates",e.preventDuplicates())("preventOpenDuplicates",e.preventOpenDuplicates())("styleClass",e.styleClass())("breakpoints",e.breakpoints())("showTransformOptions",e.showTransformOptions())("hideTransformOptions",e.hideTransformOptions())("showTransitionOptions",e.showTransitionOptions())("hideTransitionOptions",e.hideTransitionOptions()),m(),u(e.messageTemplate()?1:-1),m(),u(e.headlessTemplate()?2:-1))},dependencies:[A,M,a,Z],encapsulation:2})}}return t})()});export{U as a,V as b};
