import{c as f,d as U}from"./chunk-LDWXTTVD.js";import{P as p,Q as l}from"./chunk-JE2X3YFA.js";import{J as u,Rd as a,T as c,na as m,sa as i,x as o,xa as s}from"./chunk-4HFZZ7HS.js";import{e as n}from"./chunk-SZAW6ZBW.js";var d,h=n(()=>{m();a();l();d=(()=>{class r{constructor(e){this.currentUserService=e}getCurrentUser(){return this.currentUserService.currentUserChanged.pipe(u(e=>!!e),c(1),o(e=>({id:this.currentUserService.currentUserId,name:e.userName,isAdministrator:this.currentUserService.isAdministrator})))}static{this.\u0275fac=function(t){return new(t||r)(s(p))}}static{this.\u0275prov=i({token:r,factory:r.\u0275fac,providedIn:"root"})}}return r})()});var y,I=n(()=>{a();h();y=(()=>{class r{constructor(e){this.dpsCurrentUserService=e}resolve(e,t){return this.dpsCurrentUserService.getCurrentUser()}static{this.\u0275fac=function(t){return new(t||r)(s(d))}}static{this.\u0275prov=i({token:r,factory:r.\u0275fac,providedIn:"root"})}}return r})()});var x,j=n(()=>{m();a();U();x=(()=>{class r{constructor(e){this.userService=e}resolve(e){return this.userService.getUserSummaries().pipe(c(1),o(()=>!0))}static{this.\u0275fac=function(t){return new(t||r)(s(f))}}static{this.\u0275prov=i({token:r,factory:r.\u0275fac,providedIn:"root"})}}return r})()});export{y as a,I as b,x as c,j as d};
