import{a as l,b as J}from"./chunk-SUSYLW52.js";import{a as S,b as I,c as P,d as O}from"./chunk-G2VUEUEO.js";import{c as h,d as R}from"./chunk-LGC4ZOWD.js";import{a as n,c as w}from"./chunk-VHKJVCTZ.js";import"./chunk-SN34CLEC.js";import"./chunk-ZENOWJOG.js";import{a as f,b as g}from"./chunk-CXJOJS5Y.js";import"./chunk-ILBO5ZGN.js";import"./chunk-7YUL6BIM.js";import"./chunk-A3Y77RII.js";import"./chunk-5DATZV72.js";import"./chunk-QST7UV2H.js";import"./chunk-QUK73RJT.js";import"./chunk-ZSHAELBT.js";import"./chunk-MWQIPWDG.js";import"./chunk-UTF7W4HP.js";import"./chunk-OJRDBZWF.js";import"./chunk-6G52AAKU.js";import"./chunk-QQZCALHS.js";import"./chunk-EXP5MDOK.js";import"./chunk-M55KBHC5.js";import"./chunk-JE2X3YFA.js";import"./chunk-HW6LKGMV.js";import{Yb as s,Zb as A}from"./chunk-GDJMZZ4C.js";import"./chunk-V3J5YJ3L.js";import"./chunk-UIVS6623.js";import"./chunk-2XIOYYFS.js";import"./chunk-HYPRVELK.js";import"./chunk-YTY7HHO3.js";import"./chunk-UOI75ONS.js";import{Rd as y,ha as d,ka as v,na as D,sa as u,x as p,xa as a}from"./chunk-4HFZZ7HS.js";import{e as m}from"./chunk-SZAW6ZBW.js";var c,b=m(()=>{D();y();R();I();c=(()=>{class t{constructor(e,i){this.networkDiscoveryService=e,this.discoveryJobService=i}resolve(e,i){let C=e.paramMap.get("id"),r;return this.discoveryJobService.getJob(C).pipe(v(o=>r=o),d(()=>this.networkDiscoveryService.get(r.scanConfigurationID)),p(o=>({scanJob:r,scanConfiguration:o})))}static{this.\u0275fac=function(i){return new(i||t)(a(h),a(S))}}static{this.\u0275prov=u({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});var Y,j=m(()=>{g();J();b();O();w();A();Y=[{path:"",canActivate:[f,l],children:[{path:"scan-list",loadComponent:()=>import("./chunk-AMNOBOYD.js").then(t=>t.ScansDashboardComponent),canActivate:[n],data:{permissions:[s.PamProvidersScanView],isPamProviderDependant:!0}},{path:"scan-result/:id",loadComponent:()=>import("./chunk-R7GLSJ3F.js").then(t=>t.ScanResultListPageComponent),resolve:{scanResultElements:c},providers:[P,c],canActivate:[n],data:{permissions:[s.PamProvidersScanResultsView],isPamProviderDependant:!0}},{path:"",redirectTo:"scan-list",pathMatch:"full"}]}]});j();export{Y as ACCOUNT_DISCOVERY_ROUTES};
