import{P as o,Q as f}from"./chunk-JE2X3YFA.js";import{q as c,v as u}from"./chunk-HW6LKGMV.js";import{Rd as a,sa as s,xa as i}from"./chunk-4HFZZ7HS.js";import{e as n}from"./chunk-SZAW6ZBW.js";var d,m=n(()=>{a();f();u();d=(()=>{class t{constructor(e,r){this.currentUserService=e,this.router=r}canActivate(e,r){return!this.currentUserService.isAdministrator&&!this.currentUserService.accessSnapshot.hasAccessPam?(this.router.navigate(["/connections"]),!1):!0}static{this.\u0275fac=function(r){return new(r||t)(i(o),i(c))}}static{this.\u0275prov=s({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});export{d as a,m as b};
