import{a as q,b as A,c as B}from"./chunk-K4GYXCFH.js";import{Ha as f,La as k}from"./chunk-2XIOYYFS.js";import{D as P,K as R,p as N,t as j}from"./chunk-UOI75ONS.js";import{$c as x,Ab as l,Ad as W,Bc as u,Hc as i,Ic as E,Jc as S,Kd as h,Qb as M,Qc as O,Rd as Q,Sc as I,Tc as w,Wb as c,Xc as $,Zc as v,jc as r,kd as g,ld as y,mc as d,md as T,rc as _,sc as F,uc as D,vc as b,wc as s,xc as o}from"./chunk-4HFZZ7HS.js";import{e as z}from"./chunk-SZAW6ZBW.js";function X(e,a){if(e&1&&u(0,7),e&2){let t=i().$implicit,n=i(2);r("ngTemplateOutlet",n.itemTemplate.template)("ngTemplateOutletContext",T(2,V,t,t))}}function Y(e,a){if(e&1&&(s(0,"div"),v(1),o()),e&2){let t=i(2).$implicit,n=i(2);d(y(3,H,n.summaryFontWeight||"600")),l(),x(" ",t.summary," ")}}function Z(e,a){if(e&1&&(s(0,"div"),v(1),o()),e&2){let t=i(2).$implicit,n=i(2);d(y(3,J,n.detailFontWeight||"normal")),l(),x(" ",t.detail," ")}}function ee(e,a){if(e&1&&(s(0,"div",8),c(1,Y,2,5,"div",9)(2,Z,2,5,"div",9),o()),e&2){let t=i().$implicit;l(),_(t.summary?1:-1),l(),_(t.detail?2:-1)}}function te(e,a){if(e&1&&(s(0,"div",10),u(1,7),o()),e&2){let t=i().$implicit,n=i(2);d(g(4,K)),l(),r("ngTemplateOutlet",n.endTemplate.template)("ngTemplateOutletContext",T(5,V,t,t))}}function ne(e,a){if(e&1&&(s(0,"div",2)(1,"p-message",3),c(2,X,1,5,"ng-container",4)(3,ee,3,2,"ng-template",5,0,W),o(),c(5,te,2,8,"div",6),o()),e&2){let t,n,m=a.$implicit,C=$(4),p=i(2);l(),r("severity",(t=m.severity)!==null&&t!==void 0?t:p.severity)("escape",p.escape)("closable",(n=m.closable)!==null&&n!==void 0?n:p.closable)("styleClass",p.messageStyleClass),l(),r("ngIf",p.itemTemplate)("ngIfElse",C),l(3),_(p.endTemplate?5:-1)}}function ie(e,a){if(e&1&&(s(0,"div",1),D(1,ne,6,7,"div",2,F),o()),e&2){let t=i();l(),b(t.value)}}function le(e,a){if(e&1&&u(0,14),e&2){let t=i(3);r("ngTemplateOutlet",t.itemTemplate.template)}}function ae(e,a){e&1&&S(0)}function oe(e,a){if(e&1&&(s(0,"div"),u(1,14),o()),e&2){let t=i(3);d(g(3,U)),l(),r("ngTemplateOutlet",t.endTemplate.template)}}function se(e,a){if(e&1&&(s(0,"div",12)(1,"div",13),c(2,le,1,1,"ng-container",14)(3,ae,1,0),o(),c(4,oe,2,4,"div",9),o()),e&2){let t=i(2);d(g(4,L)),l(2),_(t.itemTemplate?2:3),l(2),_(t.endTemplate?4:-1)}}function me(e,a){if(e&1&&(s(0,"div",1)(1,"p-message",11),c(2,se,5,5,"ng-template",5),o()()),e&2){let t=i();l(),r("severity",t.severity)("escape",t.escape)("closable",t.closable)("styleClass",t.messageStyleClass)}}var G,V,H,J,K,L,U,Me,pe=z(()=>{Q();R();B();k();Q();R();B();k();G=["*"],V=(e,a)=>({$implicit:e,message:a}),H=e=>({"font-weight":e,color:"var(--text-400)"}),J=e=>({"font-weight":e,color:"var(--text-300)"}),K=()=>({"padding-right":"1.25rem"}),L=()=>({width:"100%"}),U=()=>({"margin-left":"1rem"});Me=(()=>{class e{constructor(){this.value=null,this.severity="info",this.escape=!0,this.closable=!1}get itemTemplate(){if(this.templates)return this.templates.find(t=>t.name==="content")??this.templates.find(t=>t.name==="message")??this.templates.find(t=>!t.name)}get endTemplate(){if(this.templates)return this.templates.find(t=>t.name==="end"||t.name==="actions")}hasContent(){return!!this.itemTemplate||!!this.endTemplate}static{this.\u0275fac=function(n){return new(n||e)}}static{this.\u0275cmp=M({type:e,selectors:[["p-messages"],["dvl-messages"]],contentQueries:function(n,m,C){if(n&1&&O(C,f,4),n&2){let p;I(p=w())&&(m.templates=p)}},inputs:{value:"value",severity:"severity",escape:[2,"escape","escape",h],closable:[2,"closable","closable",h],messageStyleClass:"messageStyleClass",summaryFontWeight:"summaryFontWeight",detailFontWeight:"detailFontWeight"},ngContentSelectors:G,decls:2,vars:2,consts:[["defaultTpl",""],[1,"flex","flex-col","w-full"],[1,"flex","items-center","w-full","relative"],[1,"flex-1",3,"severity","escape","closable","styleClass"],[3,"ngTemplateOutlet","ngTemplateOutletContext",4,"ngIf","ngIfElse"],["pTemplate","content"],[1,"absolute","right-0","top-0","bottom-0","flex","items-center",3,"style"],[3,"ngTemplateOutlet","ngTemplateOutletContext"],[1,"flex","flex-col","gap-1.5"],[3,"style"],[1,"absolute","right-0","top-0","bottom-0","flex","items-center"],[3,"severity","escape","closable","styleClass"],[1,"flex","items-center"],[1,"flex-1"],[3,"ngTemplateOutlet"]],template:function(n,m){n&1&&(E(),c(0,ie,3,0,"div",1)(1,me,3,4,"div",1)),n&2&&(_(m.value&&m.value.length>0?0:-1),l(),_(m.value===null||m.value===void 0?1:-1))},dependencies:[P,N,j,A,q,f],encapsulation:2})}}return e})()});export{Me as a,pe as b};
