import{f as a,j as p}from"./chunk-HYPRVELK.js";import{Rd as s,Ta as o,sa as r,xa as n}from"./chunk-4HFZZ7HS.js";import{e as c}from"./chunk-SZAW6ZBW.js";var v,h=c(()=>{s();s();p();v=(()=>{class t{constructor(e){this.httpClient=e,this.viewSessions=new o,this.route="api/gateway"}terminateSession(e,i){return this.httpClient.post(`${this.route}/${i}/session/${e}/terminate`,{})}static{this.\u0275fac=function(i){return new(i||t)(n(a))}}static{this.\u0275prov=r({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});export{v as a,h as b};
