import{a as u,b as k}from"./chunk-I6KHI5I6.js";import{a as g,b as w}from"./chunk-GKRJ4DYR.js";import"./chunk-CD2FEHIR.js";import"./chunk-A7OV5QSE.js";import"./chunk-TWOXH2JP.js";import{a as S,b as P}from"./chunk-2VAKOPKD.js";import"./chunk-24LNMKH3.js";import"./chunk-5VIETITT.js";import"./chunk-SWDR6UFF.js";import"./chunk-MRFU7ZH5.js";import"./chunk-EXSN6QWV.js";import"./chunk-JU5LF7AN.js";import"./chunk-VYRWAFSM.js";import"./chunk-LFKM2JDJ.js";import"./chunk-JE2X3YFA.js";import"./chunk-HW6LKGMV.js";import"./chunk-XRGTMJIH.js";import{Yb as e,Zb as y}from"./chunk-GDJMZZ4C.js";import"./chunk-V3J5YJ3L.js";import"./chunk-UIVS6623.js";import"./chunk-2XIOYYFS.js";import"./chunk-HYPRVELK.js";import{b as h,c as f}from"./chunk-YTY7HHO3.js";import"./chunk-UOI75ONS.js";import{Ab as a,Fb as o,Qb as l,Rd as b,Ub as c,jc as s,wc as v,xc as d,yc as m}from"./chunk-4HFZZ7HS.js";import{e as p}from"./chunk-SZAW6ZBW.js";var F,U=p(()=>{P();w();k();y();b();f();F=(()=>{class r extends S{constructor(t){super(),this.translateService=t,this.titleHeader=[{title:"lblAdministration",link:"/administration"},{title:"lblServerSettings"}],this.sections=[]}ngOnInit(){super.ngOnInit(),this.createSectionsTemplate();let t=this.sectionsTemplate.filter(i=>i.items.some(n=>n.condition()));t.forEach(i=>i.items=i.items.filter(n=>n.condition())),this.sections=t}getComponentTitle(){return"lblServerSettings"}createSectionsTemplate(){this.sectionsTemplate=[{label:this.translateService.translate("lblGeneral"),items:[{label:this.translateService.translate("lblGeneral"),icon:"dvl-icon-general-settings",link:"/administration/server-settings/general-settings",condition:()=>this.currentUserService.hasPermission(e.ServerSettingsGeneralView)},{label:this.translateService.translate("lblAuthentication"),icon:"dvl-icon-logo-authenticator",link:"/administration/server-settings/authentication",condition:()=>this.currentUserService.hasPermission(e.ServerSettingsAuthenticationView)},{label:this.translateService.translate("lblEmail"),icon:"dvl-icon-email",link:"/administration/server-settings/email",condition:()=>this.currentUserService.hasPermission(e.ServerSettingsEmailView)},{label:this.translateService.translate("lblLogging"),icon:"dvl-icon-logs",link:"/administration/server-settings/logging",condition:()=>this.currentUserService.hasPermission(e.ServerSettingsLogsView)},{label:this.translateService.translate("lblFeatures"),icon:"dvl-icon-features",link:"/administration/server-settings/features",condition:()=>this.currentUserService.hasPermission(e.ServerSettingsFeaturesView)},{label:this.translateService.translate("lblApproval"),icon:"dvl-icon-user-check",link:"/administration/server-settings/approval",condition:()=>this.currentUserService.hasPermission(e.ServerSettingsApprovalsView)},{label:this.translateService.translate("lblAdvanced"),icon:"dvl-icon-advanced-settings",link:"/administration/server-settings/advanced",condition:()=>this.currentUserService.hasPermission(e.ServerSettingsAdvancedView)}]},{label:this.translateService.translate("lblSecurity"),items:[{label:this.translateService.translate("lblTwoFactor"),icon:"dvl-icon-two-factor",link:"/administration/server-settings/two-factor",condition:()=>this.currentUserService.hasPermission(e.ServerSettingsMfaView)},{label:this.translateService.translate("lblSecurity"),icon:"dvl-icon-security",link:"/administration/server-settings/security",condition:()=>this.currentUserService.hasPermission(e.ServerSettingsSecurityView)},{label:this.translateService.translate("lblGeoIPSecurity"),icon:"dvl-icon-geo-security",link:"/administration/server-settings/geo-ip",condition:()=>this.currentUserService.hasPermission(e.ServerSettingsGeoIPView)},{label:this.translateService.translate("lblSecurityPolicies"),icon:"dvl-icon-security-policy",link:"/administration/server-settings/security-policies",condition:()=>this.currentUserService.hasPermission(e.SecurityPoliciesView)},{label:this.translateService.translate("lblUserBehaviorAnalytics"),icon:"di dvl-icon-user-behavior-analytics",link:"/administration/server-settings/user-behavior-analytics",condition:()=>this.currentUserService.hasPermission(e.UserBehaviorAnalyticsConfigurationView)}]}]}static{this.\u0275fac=function(i){return new(i||r)(o(h))}}static{this.\u0275cmp=l({type:r,selectors:[["ng-component"]],features:[c],decls:2,vars:2,consts:[[3,"titlePage"],[3,"sections"]],template:function(i,n){i&1&&(v(0,"dvl-page-wrapper-v2",0),m(1,"dvl-menu-dashboard",1),d()),i&2&&(s("titlePage",n.titleHeader),a(),s("sections",n.sections))},dependencies:[g,u],encapsulation:2})}}return r})()});U();export{F as ConfigurationDashboardComponent};
