import{q as o,v as h}from"./chunk-HW6LKGMV.js";import{Rd as u,h as n,ma as s,sa as i,xa as a}from"./chunk-4HFZZ7HS.js";import{e as c}from"./chunk-SZAW6ZBW.js";var f,l=c(()=>{s();u();h();f=(()=>{class e{constructor(t){this.router=t,this.navigationTreeRequest=new n(null),this.currentPamAccountSubject=new n(null),this.navigationTree=this.navigationTreeRequest.asObservable(),this.currentPamAccount=this.currentPamAccountSubject.asObservable()}navigateTree(t){this.navigationTreeRequest.next(t)}navigateToPamEntry(t,r=null){this.router.navigateByUrl(`connections/${t}/${r??t}`)}static{this.\u0275fac=function(r){return new(r||e)(a(o))}}static{this.\u0275prov=i({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});export{f as a,l as b};
