import{a as f,b as D}from"./chunk-I6KHI5I6.js";import{a as u,b as C}from"./chunk-GKRJ4DYR.js";import"./chunk-CD2FEHIR.js";import"./chunk-A7OV5QSE.js";import"./chunk-TWOXH2JP.js";import{a as b,b as g}from"./chunk-2VAKOPKD.js";import"./chunk-24LNMKH3.js";import"./chunk-5VIETITT.js";import"./chunk-SWDR6UFF.js";import"./chunk-MRFU7ZH5.js";import"./chunk-EXSN6QWV.js";import"./chunk-JU5LF7AN.js";import"./chunk-VYRWAFSM.js";import"./chunk-LFKM2JDJ.js";import"./chunk-JE2X3YFA.js";import"./chunk-HW6LKGMV.js";import"./chunk-XRGTMJIH.js";import"./chunk-GDJMZZ4C.js";import"./chunk-V3J5YJ3L.js";import"./chunk-UIVS6623.js";import"./chunk-2XIOYYFS.js";import"./chunk-HYPRVELK.js";import{b as d,c as S}from"./chunk-YTY7HHO3.js";import"./chunk-UOI75ONS.js";import{Ab as s,Fb as r,Qb as a,Rd as h,Ub as l,jc as o,wc as c,xc as p,yc as m}from"./chunk-4HFZZ7HS.js";import{e as v}from"./chunk-SZAW6ZBW.js";var P,T=v(()=>{g();C();D();h();S();P=(()=>{class n extends b{constructor(e){super(),this.translateService=e,this.titleHeader=[{title:"lblAdministration",link:"/administration"},{title:"lblNotifications"}],this.sections=[]}ngOnInit(){super.ngOnInit(),this.createSectionsTemplate();let e=this.sectionsTemplate.filter(t=>t.items.some(i=>i.condition()));e.forEach(t=>t.items=t.items.filter(i=>i.condition())),this.sections=e}getComponentTitle(){return"lblNotifications"}createSectionsTemplate(){this.sectionsTemplate=[{label:this.translateService.translate("lblNotifications"),items:[{label:this.translateService.translate("lblSubscribers"),icon:"dvl-icon-notifications-subscriber",link:"subscribers",condition:()=>!0},{label:this.translateService.translate("lblSubscriberGroups"),icon:"dvl-icon-notifications-subscriber-group",link:"groups",condition:()=>!0},{label:this.translateService.translate("lblSubscriptions"),icon:"dvl-icon-notifications-bold",link:"subscriptions",condition:()=>!0},{label:this.translateService.translate("lblUserSubscriptions"),icon:"dvl-icon-user-bold",link:"user-subscriptions",condition:()=>!0}]}]}static{this.\u0275fac=function(t){return new(t||n)(r(d))}}static{this.\u0275cmp=a({type:n,selectors:[["ng-component"]],features:[l],decls:2,vars:2,consts:[[3,"titlePage"],[3,"sections"]],template:function(t,i){t&1&&(c(0,"dvl-page-wrapper-v2",0),m(1,"dvl-menu-dashboard",1),p()),t&2&&(o("titlePage",i.titleHeader),s(),o("sections",i.sections))},dependencies:[u,f],encapsulation:2})}}return n})()});T();export{P as NotificationsDashboardComponent};
