import{a as t,b as o}from"./chunk-EXP5MDOK.js";import{e}from"./chunk-SZAW6ZBW.js";var s,u=e(()=>{o();s=class n extends t{toQueryString(){let r=[];return r.push(super.toQueryString()),r.join("&")}constructor(r=null){super(r),this.clone=()=>new n(this),this.data=r&&r.data?r.data:[]}}});export{s as a,u as b};
