import{a as E,b as x}from"./chunk-SN34CLEC.js";import{a as f,b as I}from"./chunk-ZENOWJOG.js";import{A as S,B as _,Wa as T,Xa as g,w as i,x as h,y as o,z as u}from"./chunk-7YUL6BIM.js";import{O as t,Rd as w,ma as A,na as b,r as v,sa as P,x as s,xa as c}from"./chunk-4HFZZ7HS.js";import{e as l}from"./chunk-SZAW6ZBW.js";var N,j=l(()=>{A();b();_();x();h();u();w();g();I();N=(()=>{class p extends E{constructor(e,r){super(),this.pamAssetServices=e,this.pamErrorService=r,this.propertyIdsBasedOnType=new Map([[i.Password,"05799f12-c6d9-4266-94f7-42b50e3d4a6b"],[i.Username,"c24049c5-9ec4-4f26-832c-86f283067c73"],[i.UniqueIdentifier,"54b6d680-0b89-4ce6-a512-bc98e0008fd7"]])}create(e){return new S(e)}get(e){return this.pamAssetServices.PamProviderTemplate_Get(e).pipe(t(r=>this.pamErrorService.handleError(r)))}createNewProperty(e,r,m,n){let a=new o;return a.id=e,a.name=r,a.type=m,a.isMandatory=n,a}getAll(){return this.pamAssetServices.PamProviderTemplate_GetProviderTemplates().pipe(t(e=>this.pamErrorService.handleError(e)))}getAllSummaries(e=!1,r=null,m=!1){return this.pamAssetServices.PamProviderTemplate_GetProviderTemplateSummaries(e,r,m).pipe(t(n=>this.pamErrorService.handleError(n)))}delete(e){return this.pamAssetServices.PamProviderTemplate_DeleteProviderTemplate(e).pipe(s(()=>!0),t(r=>this.pamErrorService.handleError(r)))}save(e){return e.id?this.update(e).pipe(s(r=>r?.id)):this.createTemplate(e).pipe(s(r=>r?.id))}export(e){return this.pamAssetServices.PamProviderTemplate_Export(e).pipe(s(r=>new Blob([r],{type:"application/json"})),t(r=>(this.pamErrorService.handleError(r),v(null))))}analyze(e){return this.pamAssetServices.PamProviderTemplate_Analyze(e).pipe(t(r=>this.pamErrorService.handleError(r)))}import(e){return this.pamAssetServices.PamProviderTemplate_Import(e).pipe(t(r=>this.pamErrorService.handleError(r)))}testScript(e){return this.pamAssetServices.PamProviderTemplate_TestCommandScript(e).pipe(t(r=>this.pamErrorService.handleError(r)))}createTemplate(e){return this.pamAssetServices.PamProviderTemplate_PostPamProviderTemplate(e)}update(e){return this.pamAssetServices.PamProviderTemplate_PutProviderTemplates(e).pipe(t(r=>this.pamErrorService.handleError(r)))}static{this.\u0275fac=function(r){return new(r||p)(c(T),c(f))}}static{this.\u0275prov=P({token:p,factory:p.\u0275fac,providedIn:"root"})}}return p})()});var y,U=l(()=>{u();h();y=class extends o{constructor(d=null){super(d)}isStringType(){return[i.String,i.Description,i.Username,i.UniqueIdentifier].includes(this.type)}}});export{N as a,j as b,y as c,U as d};
