import{a as G,b as U,c as $}from"./chunk-KLB4BWDP.js";import{C as H,E as x,a as F,f as N,m as j,w as A}from"./chunk-TCUK52QT.js";import{wa as k,ya as Q}from"./chunk-VYRWAFSM.js";import{D as B,K as V,p as P}from"./chunk-UOI75ONS.js";import{Ab as o,Ac as I,Ad as q,Cc as y,Gc as E,Hc as D,Ia as u,Ic as M,Ja as m,Jc as T,Ka as h,Kd as s,Qb as g,Rd as O,Ta as c,Vb as f,Wb as b,Xc as w,Zc as L,_c as R,ic as v,jc as l,jd as S,lc as C,ra as p,wc as r,xc as d,zc as _}from"./chunk-4HFZZ7HS.js";import{e as K}from"./chunk-SZAW6ZBW.js";function X(i,z){if(i&1&&(_(0),L(1),I()),i&2){let e=D();o(),R(e.label)}}function Y(i,z){i&1&&T(0)}var W,Z,ge,ee=K(()=>{O();V();x();$();O();Q();V();x();$();W=["*"];Z=0,ge=(()=>{class i{constructor(){this.disabled=!1,this.required=!1,this.value=null,this.labelPosition="right",this.changed=new c,this.model=null,this.onChange=()=>{},this.onTouched=()=>{},this._autoId=`dvl-rb-${++Z}`}get resolvedId(){return this.inputId??this._autoId}ngOnChanges(e){}onModelChange(e){this.model=e,this.onChange(e),this.changed.emit(e)}writeValue(e){this.model=e}registerOnChange(e){this.onChange=e}registerOnTouched(e){this.onTouched=e}setDisabledState(e){this.disabled=e}static{this.\u0275fac=function(n){return new(n||i)}}static{this.\u0275cmp=g({type:i,selectors:[["dvl-radio"]],hostAttrs:[1,"inline-block"],inputs:{disabled:[2,"disabled","disabled",s],required:[2,"required","required",s],value:"value",name:"name",label:"label",labelPosition:"labelPosition",inputId:"inputId",ariaLabel:"ariaLabel",ariaLabelledBy:"ariaLabelledBy"},outputs:{changed:"changed"},features:[S([{provide:F,useExisting:p(()=>i),multi:!0}]),f([{directive:k,inputs:["pTooltip","dvlTooltip","tooltipPosition","tooltipPosition","tooltipEvent","tooltipEvent","appendTo","appendTo","autoHide","autoHide","showDelay","showDelay","hideDelay","hideDelay","tooltipOptions","tooltipOptions","tooltipStyleClass","tooltipStyleClass"]}]),u],ngContentSelectors:W,decls:6,vars:13,consts:[["projected",""],[1,"inline-flex","items-center","gap-2"],[3,"ngModelChange","name","inputId","value","disabled","required","ariaLabel","ariaLabelledBy","ngModel"],[1,"cursor-pointer","select-none"],[4,"ngIf","ngIfElse"]],template:function(n,t){if(n&1){let a=y();M(),r(0,"div",1)(1,"p-radiobutton",2),E("ngModelChange",function(J){return m(a),h(t.onModelChange(J))}),d(),r(2,"label",3),b(3,X,2,1,"ng-container",4)(4,Y,1,0,"ng-template",null,0,q),d()()}if(n&2){let a=w(5);C("flex-row-reverse",t.labelPosition==="left"),o(),l("name",t.name)("inputId",t.resolvedId)("value",t.value)("disabled",t.disabled)("required",t.required)("ariaLabel",t.ariaLabel)("ariaLabelledBy",t.ariaLabelledBy)("ngModel",t.model),o(),v("for",t.resolvedId),o(),l("ngIf",t.label!==void 0)("ngIfElse",a)}},dependencies:[B,P,H,N,A,j,U,G],encapsulation:2})}}return i})()});export{ge as a,ee as b};
