import{b as I,c as j}from"./chunk-YTY7HHO3.js";import{Rd as h,d as p,ma as d,sa as b,ya as v}from"./chunk-4HFZZ7HS.js";import{e as O}from"./chunk-SZAW6ZBW.js";var k,m=O(()=>{h();j();d();h();k=(()=>{class l{constructor(){this.translatorService=v(I)}initComboEnum(t,e=[],o=!1,n=!1,s=0,i=null){let f=[],S=this.getEnumValues(t);for(let a of S){if(e.indexOf(a)>=0)continue;let r=this.translatorService.translateEnum(t,a),g=n?r:+a;if(t[a]==="Default"&&t[s]&&t[a]!==t[s]&&e.indexOf(s)===-1){let c=this.translatorService.translateEnum(t,s);if(s!=null&&i!=null&&t[s]==="Inherited"){let $=this.translatorService.translateEnum(t,i);r=`${r} (${c}) \u21A6 ${$}`}else r=`${r} (${c})`}else if(t[a]==="Inherited"&&t[i]&&t[a]!==t[i]&&e.indexOf(i)===-1){let c=this.translatorService.translateEnum(t,i);r=`${r} (${c})`}f.push({label:r,value:g})}return o&&f.sort((a,r)=>a.label.localeCompare(r.label,void 0,{usage:"sort",numeric:!0})),f}getComboEnum(t){return new p(e=>{e.next(this.getEnumValues(t).map(o=>({label:this.translatorService.translateEnum(t,o),value:+o}))),e.complete()})}getComboEnumAsString(t){return new p(e=>{let o=[];for(let n of Object.keys(t))parseInt(n,10)>=0&&o.push({label:this.translatorService.translateEnum(t,n),value:t[n]});e.next(o),e.complete()})}getEnumValues(t){return Object.values(t).filter(e=>!isNaN(e))}static{this.\u0275fac=function(e){return new(e||l)}}static{this.\u0275prov=b({token:l,factory:l.\u0275fac,providedIn:"root"})}}return l})()});export{k as a,m as b};
