import{Wj as m,Xj as b}from"./chunk-GDJMZZ4C.js";import{a as n,b as S,g as f,h as d}from"./chunk-V3J5YJ3L.js";import{b as o,c as u}from"./chunk-YTY7HHO3.js";import{J as a,Rd as v,na as l,sa as c,x as s,xa as i}from"./chunk-4HFZZ7HS.js";import{e as p}from"./chunk-SZAW6ZBW.js";var A,j=p(()=>{l();S();v();b();d();u();A=(()=>{class e{static{this.PassphraseDictionaryIdEnglish="5ec83fd8-3e08-443d-bd95-b4f5edf123c2"}constructor(r,t,h){this.assetsService=r,this.errorService=t,this.translatorService=h}getPassphraseDictionaries(){return this.assetsService.GetPassphraseDictionaries(!1).pipe(a(r=>!!r),s(r=>{if(r.result!==n.Success){this.errorService.pushErrorMessage(r.errorMessage);return}return r.data.forEach(t=>{t.name=this.translatorService.translate(t.name)}),r.data}))}static{this.\u0275fac=function(t){return new(t||e)(i(m),i(f),i(o))}}static{this.\u0275prov=c({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});export{A as a,j as b};
