import{i as e,j as t}from"./chunk-2JML2BCE.js";import{e as l}from"./chunk-SZAW6ZBW.js";var r=l(()=>{t();e.setLanguage(["sv","sv-SE"],{Markdown:"Markdown",WYSIWYG:"WYSIWYG",Write:"Skriv",Preview:"F\xF6rhandsgranska",Headings:"\xD6verskrifter",Paragraph:"Paragraf",Bold:"Fet",Italic:"Kursiv",Strike:"Genomstruken",Code:"Kodrad",Line:"Linje",Blockquote:"Citatblock","Unordered list":"Punktlista","Ordered list":"Numrerad lista",Task:"Att g\xF6ra",Indent:"\xD6ka indrag",Outdent:"Minska indrag","Insert link":"Infoga l\xE4nk","Insert CodeBlock":"Infoga kodblock","Insert table":"Infoga tabell","Insert image":"Infoga bild",Heading:"\xD6verskrift","Image URL":"Bildadress","Select image file":"V\xE4lj en bildfil","Choose a file":"V\xE4lj en fil","No file":"Ingen fil",Description:"Beskrivning",OK:"OK",More:"Mer",Cancel:"Avbryt",File:"Fil",URL:"Adress","Link text":"L\xE4nktext","Add row to up":"L\xE4gg till rad till upp","Add row to down":"L\xE4gg till rad till ner","Add column to left":"L\xE4gg till kolumn till v\xE4nster","Add column to right":"L\xE4gg till kolumn till h\xF6ger","Remove row":"Radera rad","Remove column":"Radera kolumn","Align column to left":"V\xE4nsterjustera","Align column to center":"Centrera","Align column to right":"H\xF6gerjustera","Remove table":"Radera tabell","Would you like to paste as table?":"Vill du klistra in som en tabell?","Text color":"Textf\xE4rg","Auto scroll enabled":"Automatisk scroll aktiverad","Auto scroll disabled":"Automatisk scroll inaktiverad","Choose language":"V\xE4lj spr\xE5k"})});r();
