import{d as J,e as Y,i as K,j as Z}from"./chunk-VYRWAFSM.js";import{Fa as H,Ga as V,Ha as G,Ia as g,La as X,ba as A,xa as W}from"./chunk-2XIOYYFS.js";import{D as z,K as P,m as w,p as O,s as N,t as B}from"./chunk-UOI75ONS.js";import{Ab as i,Ac as v,Bc as y,Hc as _,Ic as j,Jc as h,Na as T,Qb as E,Qc as p,Rb as M,Rd as R,Sc as d,Tc as s,Ub as k,Wb as c,Za as x,Zc as C,_c as Q,ic as D,jc as r,jd as q,nc as $,sa as S,ta as F,wc as f,xc as u,ya as I,zc as b}from"./chunk-4HFZZ7HS.js";import{e as U}from"./chunk-SZAW6ZBW.js";function le(e,o){e&1&&y(0)}function ce(e,o){if(e&1&&(f(0,"div",8),h(1,1),c(2,le,1,0,"ng-container",6),u()),e&2){let t=_();i(2),r("ngTemplateOutlet",t.headerTemplate||t._headerTemplate)}}function pe(e,o){if(e&1&&(b(0),C(1),v()),e&2){let t=_(2);i(),Q(t.header)}}function de(e,o){e&1&&y(0)}function se(e,o){if(e&1&&(f(0,"div",9),c(1,pe,2,1,"ng-container",10)(2,de,1,0,"ng-container",6),u()),e&2){let t=_();i(),r("ngIf",t.header&&!t._titleTemplate&&!t.titleTemplate),i(),r("ngTemplateOutlet",t.titleTemplate||t._titleTemplate)}}function me(e,o){if(e&1&&(b(0),C(1),v()),e&2){let t=_(2);i(),Q(t.subheader)}}function fe(e,o){e&1&&y(0)}function ue(e,o){if(e&1&&(f(0,"div",11),c(1,me,2,1,"ng-container",10)(2,fe,1,0,"ng-container",6),u()),e&2){let t=_();i(),r("ngIf",t.subheader&&!t._subtitleTemplate&&!t.subtitleTemplate),i(),r("ngTemplateOutlet",t.subtitleTemplate||t._subtitleTemplate)}}function _e(e,o){e&1&&y(0)}function ye(e,o){e&1&&y(0)}function he(e,o){if(e&1&&(f(0,"div",12),h(1,2),c(2,ye,1,0,"ng-container",6),u()),e&2){let t=_();i(2),r("ngTemplateOutlet",t.footerTemplate||t._footerTemplate)}}var ee,te,ne,ae,ie,re,oe,ge,Te,L,be,Ne,ve=U(()=>{P();P();R();R();W();X();Z();Y();ee=["header"],te=["title"],ne=["subtitle"],ae=["content"],ie=["footer"],re=["*",[["p-header"]],[["p-footer"]]],oe=["*","p-header","p-footer"];ge=({dt:e})=>`
.p-card {
    background: ${e("card.background")};
    color: ${e("card.color")};
    box-shadow: ${e("card.shadow")};
    border-radius: ${e("card.border.radius")};
    display: flex;
    flex-direction: column;
}

.p-card-caption {
    display: flex;
    flex-direction: column;
    gap: ${e("card.caption.gap")};
}

.p-card-body {
    padding: ${e("card.body.padding")};
    display: flex;
    flex-direction: column;
    gap: ${e("card.body.gap")};
}

.p-card-title {
    font-size: ${e("card.title.font.size")};
    font-weight: ${e("card.title.font.weight")};
}

.p-card-subtitle {
    color: ${e("card.subtitle.color")};
}
`,Te={root:"p-card p-component",header:"p-card-header",body:"p-card-body",caption:"p-card-caption",title:"p-card-title",subtitle:"p-card-subtitle",content:"p-card-content",footer:"p-card-footer"},L=(()=>{class e extends J{name="card";theme=ge;classes=Te;static \u0275fac=(()=>{let t;return function(n){return(t||(t=T(e)))(n||e)}})();static \u0275prov=S({token:e,factory:e.\u0275fac})}return e})(),be=(()=>{class e extends K{header;subheader;set style(t){A(this._style(),t)||this._style.set(t)}styleClass;headerFacet;footerFacet;headerTemplate;titleTemplate;subtitleTemplate;contentTemplate;footerTemplate;_headerTemplate;_titleTemplate;_subtitleTemplate;_contentTemplate;_footerTemplate;_style=x(null);_componentStyle=I(L);getBlockableElement(){return this.el.nativeElement.children[0]}templates;ngAfterContentInit(){this.templates.forEach(t=>{switch(t.getType()){case"header":this._headerTemplate=t.template;break;case"title":this._titleTemplate=t.template;break;case"subtitle":this._subtitleTemplate=t.template;break;case"content":this._contentTemplate=t.template;break;case"footer":this._footerTemplate=t.template;break;default:this._contentTemplate=t.template;break}})}static \u0275fac=(()=>{let t;return function(n){return(t||(t=T(e)))(n||e)}})();static \u0275cmp=E({type:e,selectors:[["p-card"]],contentQueries:function(l,n,m){if(l&1&&(p(m,H,5),p(m,V,5),p(m,ee,4),p(m,te,4),p(m,ne,4),p(m,ae,4),p(m,ie,4),p(m,G,4)),l&2){let a;d(a=s())&&(n.headerFacet=a.first),d(a=s())&&(n.footerFacet=a.first),d(a=s())&&(n.headerTemplate=a.first),d(a=s())&&(n.titleTemplate=a.first),d(a=s())&&(n.subtitleTemplate=a.first),d(a=s())&&(n.contentTemplate=a.first),d(a=s())&&(n.footerTemplate=a.first),d(a=s())&&(n.templates=a)}},inputs:{header:"header",subheader:"subheader",style:"style",styleClass:"styleClass"},features:[q([L]),k],ngContentSelectors:oe,decls:9,vars:10,consts:[[3,"ngClass","ngStyle"],["class","p-card-header",4,"ngIf"],[1,"p-card-body"],["class","p-card-title",4,"ngIf"],["class","p-card-subtitle",4,"ngIf"],[1,"p-card-content"],[4,"ngTemplateOutlet"],["class","p-card-footer",4,"ngIf"],[1,"p-card-header"],[1,"p-card-title"],[4,"ngIf"],[1,"p-card-subtitle"],[1,"p-card-footer"]],template:function(l,n){l&1&&(j(re),f(0,"div",0),c(1,ce,3,1,"div",1),f(2,"div",2),c(3,se,3,2,"div",3)(4,ue,3,2,"div",4),f(5,"div",5),h(6),c(7,_e,1,0,"ng-container",6),u(),c(8,he,3,1,"div",7),u()()),l&2&&($(n.styleClass),r("ngClass","p-card p-component")("ngStyle",n._style()),D("data-pc-name","card"),i(),r("ngIf",n.headerFacet||n.headerTemplate||n._headerTemplate),i(2),r("ngIf",n.header||n.titleTemplate||n._titleTemplate),i(),r("ngIf",n.subheader||n.subtitleTemplate||n._subtitleTemplate),i(3),r("ngTemplateOutlet",n.contentTemplate||n._contentTemplate),i(),r("ngIf",n.footerFacet||n.footerTemplate||n._footerTemplate))},dependencies:[z,w,O,B,N,g],encapsulation:2,changeDetection:0})}return e})(),Ne=(()=>{class e{static \u0275fac=function(l){return new(l||e)};static \u0275mod=M({type:e});static \u0275inj=F({imports:[be,g,g]})}return e})()});export{be as a,Ne as b,ve as c};
