import{a as c,b as l}from"./chunk-LFKM2JDJ.js";import{$c as r,Oa as n,Qb as o,Rd as f,Zc as p,ud as s,vd as a}from"./chunk-4HFZZ7HS.js";import{e as d}from"./chunk-SZAW6ZBW.js";var v,u=d(()=>{l();f();v=(()=>{class t{constructor(i){this.key=i}static{this.\u0275fac=function(e){return new(e||t)(n("key"))}}static{this.\u0275cmp=o({type:t,selectors:[["sd-text"]],decls:2,vars:3,template:function(e,m){e&1&&(p(0),s(1,"translate")),e&2&&r("",a(1,1,m.key),`
`)},dependencies:[c],encapsulation:2})}}return t})()});export{v as a,u as b};
