import{a as u,b as A}from"./chunk-I6KHI5I6.js";import{a as w,b as y}from"./chunk-GKRJ4DYR.js";import"./chunk-CD2FEHIR.js";import"./chunk-A7OV5QSE.js";import{a as v,b as k,c as p,d as N}from"./chunk-GY6XZ2EK.js";import"./chunk-TWOXH2JP.js";import"./chunk-5VIETITT.js";import"./chunk-SWDR6UFF.js";import"./chunk-M6JBMHZW.js";import"./chunk-MRFU7ZH5.js";import"./chunk-JU5LF7AN.js";import"./chunk-VYRWAFSM.js";import"./chunk-LFKM2JDJ.js";import{D as b,E as T}from"./chunk-JE2X3YFA.js";import"./chunk-HW6LKGMV.js";import"./chunk-XRGTMJIH.js";import"./chunk-GDJMZZ4C.js";import"./chunk-V3J5YJ3L.js";import"./chunk-UIVS6623.js";import"./chunk-2XIOYYFS.js";import"./chunk-HYPRVELK.js";import{b as h,c as D}from"./chunk-YTY7HHO3.js";import"./chunk-UOI75ONS.js";import{Ab as a,Qb as s,Rd as m,T as i,jc as l,na as f,wc as r,xc as c,ya as t,yc as d}from"./chunk-4HFZZ7HS.js";import{e as S}from"./chunk-SZAW6ZBW.js";var E,C=S(()=>{m();D();y();A();k();N();f();T();m();E=(()=>{class o{constructor(){this.hasAccessToOnBoarding=!1,this.translateService=t(h),this.whatsNewService=t(v),this.onboardingService=t(p),this.serverInformationService=t(b),this.sections=[]}ngOnInit(){this.initMenu(),this.onboardingService.getSteps().pipe(i(1)).subscribe(e=>{this.hasAccessToOnBoarding=(e||[]).length>0,this.initMenu()})}initMenu(){this.sections=[{label:this.translateService.translate("lblAppsAndCompanionTools"),items:[{label:this.translateService.translate("DevolutionsWorkspace"),icon:"dvl-icon-logo-workspace",command:()=>window.open("https://redirection.devolutions.com/5e01e501-7b16-48b8-a058-9b914a20307d","_blank")},{label:this.translateService.translate("DevolutionsPowerShell"),icon:"dvl-icon-logo-hub-cli",command:()=>window.open("https://redirection.devolutions.com/Devolutions.PowerShell","_blank")},{label:this.translateService.translate("DevolutionsGateway"),icon:"dvl-icon-logo-gateway",command:()=>window.open("https://redirection.devolutions.com/ea6ea152-743a-4025-b358-ce9b5debc910","_blank")},{label:this.translateService.translate("lblApiDocumentation"),icon:"dvl-icon-documentation-group-2",link:"/api-documentation"},{label:"im the design system",icon:"dvl-icon-tools",link:"/ui/design-system",condition:()=>this.serverInformationService.isFeatureFlagEnabled("designSystem")}]},{label:this.translateService.translate("lblSupportCenter"),items:[{label:this.translateService.translate("lblDocumentation"),icon:"dvl-icon-logo-knowledge-base",command:()=>window.open("https://helpserver.devolutions.net/index.html?webinterface.htm","_blank")},{label:this.translateService.translate("lblForum"),icon:"dvl-icon-logo-forum",command:()=>window.open("https://redirection.devolutions.com/forum-dvls","_blank")},{label:this.translateService.translate("lblTipsAndTricks"),icon:"dvl-icon-tools",command:()=>window.open("https://redirection.devolutions.com/f230985a-67cd-433e-894f-8885786cec67","_blank")}]},{label:this.translateService.translate("lblReleaseNews"),items:[{label:this.translateService.translate("lblReleaseNotes"),icon:"dvl-icon-new-year",command:()=>this.showWhatsNewDashboard("releaseNotes")},{label:this.translateService.translate("lblWhatsNew"),icon:"dvl-icon-news",command:()=>this.showWhatsNewDashboard("whatsNews")},{label:this.translateService.translate("lblProductInsights"),icon:"dvl-icon-idea",command:()=>this.showWhatsNewDashboard("productInsights")}]},{label:this.translateService.translate("lblHowTos"),items:[{label:this.translateService.translate("lblOnboarding"),icon:"dvl-icon-onboarding-wizard",command:()=>this.onboardingService.toggleOnboarding(),condition:()=>this.hasAccessToOnBoarding}]}]}getComponentTitle(){return"lblHelpAndTools"}showWhatsNewDashboard(e){this.whatsNewService.toggleWhatsNew(!0,e)}static{this.\u0275fac=function(n){return new(n||o)}}static{this.\u0275cmp=s({type:o,selectors:[["ng-component"]],decls:2,vars:1,consts:[["titlePage","lblHelpAndTools"],[3,"sections"]],template:function(n,g){n&1&&(r(0,"dvl-page-wrapper-v2",0),d(1,"dvl-menu-dashboard",1),c()),n&2&&(a(),l("sections",g.sections))},dependencies:[w,u],encapsulation:2})}}return o})()});C();export{E as ToolsDashboardComponent};
