import{e as a}from"./chunk-SZAW6ZBW.js";var s,c=a(()=>{s=(()=>{class t{static{this.allApproversID="094199C4-EC56-483A-AEA8-EE32744EA2F6"}static{this.standaloneProviderID="EB90A4B5-C708-4E39-B4C0-3E64C4F69A2D"}static{this.defaultCheckoutPolicyID="16592205-9deb-4b27-8b25-0bf600571f6e"}static{this.defaultAccountLifecyclePolicyID="14acd411-cada-4abd-b424-9a993df98374"}static{this.customPamAccountLifecycleID="f74722b8-0cf6-4f3d-ba6a-375d16a7b2b9"}static{this.pamSystemVaultID="ff7b757a-cdca-4a11-9dc6-d59989647279"}static{this.pamSystemSQLSchedulerAccountID="86133f36-68d8-498f-8e4a-fa7b9a30bb74"}static{this.pamSystemSQLWebsiteAccountID="0da50fed-3366-4cd6-9d30-025135ce5efc"}static{this.pamSystemSQLManagementAccountID="3e458337-802c-4ac2-8dca-45910c5c01b1"}static{this.pamSystemSQLAccountID="b5e955a4-5a88-4852-9523-15be0d62d6bc"}static{this.pamSystemSQLProviderID="b6647a41-481a-436f-bfb1-074acf7ad026"}static{this.pamVaultDashboardId="385e22dc-aab2-4d2c-99c4-107105de165a"}static{this.pamSystemPropagationTemplateID="edc8877f-4696-4725-ba69-b3acdb2900e5"}static{this.pamSystemPropagationScriptID="b5c11a38-6c19-441e-ba1d-93e910193560"}static{this.pamMinimumCheckoutDurationMinutes=10}static{this.pamMinimumHeartbeatScheduleMinutes=10}static{this.mySqlUserBuiltInTemplateId="7c2c2daf-f5c5-4d84-b669-42770feefc99"}static{this.ciscoBuiltInTemplateId="9706d561-19f6-4e02-a950-4b1fd1ce294d"}static{this.oracleBuiltInTemplateId="dc72f02f-bffa-4003-af98-ceb28a49a797"}}return t})()});export{s as a,c as b};
