import{i,j as o}from"./chunk-2JML2BCE.js";import{e}from"./chunk-SZAW6ZBW.js";var a=e(()=>{o();i.setLanguage(["it","it-IT"],{Markdown:"Markdown",WYSIWYG:"WYSIWYG",Write:"Scrivere",Preview:"Anteprima",Headings:"Intestazioni",Paragraph:"Paragrafo",Bold:"Grassetto",Italic:"Corsivo",Strike:"Barrato",Code:"Codice",Line:"Linea",Blockquote:"Blocco citazione","Unordered list":"Lista puntata","Ordered list":"Lista numerata",Task:"Attivit\xE0",Indent:"Aggiungi indentazione",Outdent:"Rimuovi indentazione","Insert link":"Inserisci link","Insert CodeBlock":"Inserisci blocco di codice","Insert table":"Inserisci tabella","Insert image":"Inserisci immagine",Heading:"Intestazione","Image URL":"URL immagine","Select image file":"Seleziona file immagine","Choose a file":"Scegli un file","No file":"Nessun file",Description:"Descrizione",OK:"OK",More:"Pi\xF9",Cancel:"Cancella",File:"File",URL:"URL","Link text":"Testo del collegamento","Add row to up":"Aggiungi riga in alto","Add row to down":"Aggiungi riga in basso","Add column to left":"Aggiungi colonna a sinistra","Add column to right":"Aggiungi colonna a destra","Remove row":"Rimuovi riga","Remove column":"Rimuovi colonna","Align column to left":"Allinea a sinistra","Align column to center":"Allinea al centro","Align column to right":"Allinea a destra","Remove table":"Rimuovi tabella","Would you like to paste as table?":"Desideri incollare sotto forma di tabella?","Text color":"Colore del testo","Auto scroll enabled":"Scrolling automatico abilitato","Auto scroll disabled":"Scrolling automatico disabilitato","Choose language":"Scegli la lingua"})});a();
