import{a as u}from"./chunk-YTY7HHO3.js";import{Rd as s,Tb as i}from"./chunk-4HFZZ7HS.js";import{e as m,h as n}from"./chunk-SZAW6ZBW.js";var o,f,a=m(()=>{o=n(u());s();f=(()=>{class e{transform(t){let r=this.parseDate(t);return r?this.getFromNow(r):""}parseDate(t){let r=o.default.utc(t);return r.isValid()?r:null}getFromNow(t){return t.diff(o.default.utc(),"seconds")>0?o.default.utc().local().fromNow():t.local().fromNow()}static{this.\u0275fac=function(r){return new(r||e)}}static{this.\u0275pipe=i({name:"fromNowNoFuture",type:e,pure:!0})}}return e})()});export{f as a,a as b};
