import{b as ne,c as ge}from"./chunk-XMJUUJNO.js";import{d as ee,e as te,f as ie}from"./chunk-MAVIT7QG.js";import{a as $,c as be}from"./chunk-UGQXGBKF.js";import{C as q,E as H,b as R,f as G,m as U}from"./chunk-TCUK52QT.js";import{a as X,b as ce}from"./chunk-LFKM2JDJ.js";import{ei as Z,fi as de}from"./chunk-GDJMZZ4C.js";import{a as Y,b as me}from"./chunk-UIVS6623.js";import{Ca as z,Ha as J,La as pe}from"./chunk-2XIOYYFS.js";import{b as K,c as ue}from"./chunk-YTY7HHO3.js";import{K as ae,t as Q}from"./chunk-UOI75ONS.js";import{Ab as l,Bc as D,Cc as N,Fb as E,Gc as u,Hc as g,Ja as I,Ka as B,Qb as L,Qc as O,R as S,Rd as W,Sc as P,Tc as j,Ub as V,W as w,Wb as m,Zc as f,_c as h,dd as C,ed as _,fd as T,g as v,ia as x,jc as p,kc as k,ma as se,na as oe,rc as b,ud as c,vd as d,wc as o,xc as s,ya as A}from"./chunk-4HFZZ7HS.js";import{e as y}from"./chunk-SZAW6ZBW.js";function fe(n,F){if(n&1){let t=N();o(0,"i",7),u("click",function(){I(t);let e=g();return B(e.onClearFilterClick())}),s()}}function he(n,F){n&1&&D(0)}function Ce(n,F){if(n&1&&m(0,he,1,0,"ng-container",8),n&2){let t=g();p("ngTemplateOutlet",t.centerTemplate)}}var M,re,je,_e=y(()=>{W();pe();oe();se();me();ae();ce();ge();H();ie();be();W();ue();H();ie();M="boolean",re=!1,je=(()=>{class n extends Y{constructor(t){super(),this.translatorService=t,this.isMemberField="isMember",this.isMemberLabel="lblIsMember",this.filterSubject=new v,this.filterService=A(z),re||(this.filterService.register(M,(r,e)=>e==null?!0:!!r===e),re=!0),this.isMemberOptions=[{label:this.translatorService.translate("lblYes"),value:"true"},{label:this.translatorService.translate("lblNo"),value:"false"}],this.subscribeToFilterValue()}ngAfterContentInit(){this.templates.forEach(t=>{switch(t.getType()){case"centerTemplate":this.centerTemplate=t.template;break}})}onClearFilterClick(){this.textFilter="",this.table.filterGlobal(null,null)}onTextChange(t){this.filterSubject.next(t.target.value)}onIsMemberChange(t){t==null?this.table.filter(null,this.isMemberField,M):this.table.filter(t==="true",this.isMemberField,M)}subscribeToFilterValue(){this.filterSubject.pipe(w(),S(500),x(this.destroyed$)).subscribe(t=>this.table.filterGlobal(t,"contains"))}static{this.\u0275fac=function(r){return new(r||n)(E(K))}}static{this.\u0275cmp=L({type:n,selectors:[["dvls-table-filter-assignments"]],contentQueries:function(r,e,a){if(r&1&&O(a,J,4),r&2){let i;P(i=j())&&(e.templates=i)}},inputs:{table:"table",isMemberField:"isMemberField",isMemberLabel:"isMemberLabel"},features:[V],decls:15,vars:17,consts:[[1,"flex","gap-4","mb-4"],[1,"form-group"],[1,"relative"],["pInputText","",1,"form-control","w-full","pr-8",3,"ngModelChange","input","ngModel"],[1,"pi","pi-times","absolute","top-1/2","-translate-y-1/2","right-3","cursor-pointer"],[1,"form-group","flex-1"],["styleClass","form-control",3,"onChange","ngModelChange","options","showClear","ngModel","placeholder"],[1,"pi","pi-times","absolute","top-1/2","-translate-y-1/2","right-3","cursor-pointer",3,"click"],[4,"ngTemplateOutlet"]],template:function(r,e){r&1&&(o(0,"div",0)(1,"div",1)(2,"label"),f(3),c(4,"translate"),s(),o(5,"div",2)(6,"input",3),T("ngModelChange",function(i){return _(e.textFilter,i)||(e.textFilter=i),i}),u("input",function(i){return e.onTextChange(i)}),s(),m(7,fe,1,0,"i",4),s()(),m(8,Ce,1,1,"ng-container"),o(9,"div",5)(10,"label"),f(11),c(12,"translate"),s(),o(13,"p-select",6),c(14,"translate"),u("onChange",function(i){return e.onIsMemberChange(i.value)}),T("ngModelChange",function(i){return _(e.isMember,i)||(e.isMember=i),i}),s()()()),r&2&&(l(),k("flex",e.centerTemplate?"1":"2"),l(2),h(d(4,11,"lblFilter")),l(3),C("ngModel",e.textFilter),l(),b(e.textFilter?7:-1),l(),b(e.centerTemplate?8:-1),l(3),h(d(12,13,e.isMemberLabel)),l(2),p("options",e.isMemberOptions)("showClear",!0),C("ngModel",e.isMember),p("placeholder",d(14,15,"lblSelect")))},dependencies:[X,ne,q,R,G,U,Q,te,ee,$],encapsulation:2})}}return n})()});var le,Te=y(()=>{de();le=class extends Z{updatePasswordDisplayValue(){if(this.repositorySettings.masterPasswordHash||this.password){this.passwordDisplayValue="\u25CF\u25CF\u25CF\u25CF\u25CF\u25CF";return}this.passwordDisplayValue=""}getImage(){return this.imageBytes&&this.imageBytes.length>0?"data:image/png;base64,"+this.imageBytes:this.imageName?this.imageName:""}}});export{je as a,_e as b,le as c,Te as d};
