import{Rd as m,g as u,ma as c,sa as o}from"./chunk-4HFZZ7HS.js";import{e as i}from"./chunk-SZAW6ZBW.js";var r,l=i(()=>{r=function(e){return e[e.AddToExisting=0]="AddToExisting",e[e.ReplaceAllWith=1]="ReplaceAllWith",e[e.FindAndRemoveThese=2]="FindAndRemoveThese",e}(r||{});(function(e){function a(){return"BatchEditAction"}e.getClass=a;function t(s){let n="";return typeof e.getClass=="function"&&(n=e.getClass()),n+"_"+e[s]}e.getEnumKey=t})(r||(r={}))});var w,p=i(()=>{c();l();m();w=(()=>{class e{constructor(){this.viewRequestSource=new u,this.viewRequest=this.viewRequestSource.asObservable()}view(t){this.viewRequestSource.next(t)}getBatchEditSelectedMembers(t,s,n){switch(s){case r.AddToExisting:return[...new Set([...t,...n])];case r.ReplaceAllWith:return n;case r.FindAndRemoveThese:return t.filter(f=>!n.includes(f));default:return[]}}static{this.\u0275fac=function(s){return new(s||e)}}static{this.\u0275prov=o({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});export{r as a,l as b,w as c,p as d};
