import{i,j as t}from"./chunk-2JML2BCE.js";import{e}from"./chunk-SZAW6ZBW.js";var o=e(()=>{t();i.setLanguage(["fi","fi-FI"],{Markdown:"Markdown",WYSIWYG:"WYSIWYG",Write:"Kirjoita",Preview:"Esikatselu",Headings:"Otsikot",Paragraph:"Kappale",Bold:"Lihavointi",Italic:"Kursivointi",Strike:"Yliviivaus",Code:"Koodi",Line:"Vaakaviiva",Blockquote:"Lainaus","Unordered list":"Luettelo","Ordered list":"Numeroitu luettelo",Task:"Teht\xE4v\xE4",Indent:"Suurenna sisennyst\xE4",Outdent:"Pienenn\xE4 sisennyst\xE4","Insert link":"Lis\xE4\xE4 linkki","Insert CodeBlock":"Lis\xE4\xE4 koodia","Insert table":"Lis\xE4\xE4 taulukko","Insert image":"Lis\xE4\xE4 kuva",Heading:"Otsikko","Image URL":"Kuvan URL","Select image file":"Valitse kuvatiedosto","Choose a file":"Valitse tiedosto","No file":"Ei tiedosto",Description:"Kuvaus",OK:"OK",More:"Lis\xE4\xE4",Cancel:"Peruuta",File:"Tiedosto",URL:"URL","Link text":"Linkkiteksti","Add row to up":"Lis\xE4\xE4 rivi yl\xF6sp\xE4in","Add row to down":"Lis\xE4\xE4 rivi alasp\xE4in","Add column to left":"Lis\xE4\xE4 sarake vasemmalla","Add column to right":"Lis\xE4\xE4 sarake oikealle","Remove row":"Poista rivi","Remove column":"Poista sarake","Align column to left":"Tasaus vasemmalle","Align column to center":"Keskit\xE4","Align column to right":"Tasaus oikealle","Remove table":"Poista taulukko","Would you like to paste as table?":"Haluatko liitt\xE4\xE4 taulukkomuodossa?","Text color":"Tekstin v\xE4ri","Auto scroll enabled":"Automaattinen skrollaus k\xE4yt\xF6ss\xE4","Auto scroll disabled":"Automaattinen skrollaus pois k\xE4yt\xF6st\xE4","Choose language":"Valitse kieli"})});o();
