import{a as o,b as d}from"./chunk-EKSSJ22J.js";import{a as n,b as p}from"./chunk-T6GVOQU2.js";import{a as i,c}from"./chunk-VHKJVCTZ.js";import"./chunk-BCH4TWDY.js";import{a,b as m}from"./chunk-CXJOJS5Y.js";import"./chunk-ILBO5ZGN.js";import"./chunk-ZSHAELBT.js";import"./chunk-MWQIPWDG.js";import"./chunk-UTF7W4HP.js";import"./chunk-OJRDBZWF.js";import"./chunk-6G52AAKU.js";import"./chunk-QQZCALHS.js";import"./chunk-EXP5MDOK.js";import"./chunk-M55KBHC5.js";import"./chunk-JE2X3YFA.js";import"./chunk-HW6LKGMV.js";import{Yb as e,Zb as s}from"./chunk-GDJMZZ4C.js";import"./chunk-V3J5YJ3L.js";import"./chunk-UIVS6623.js";import"./chunk-2XIOYYFS.js";import"./chunk-HYPRVELK.js";import"./chunk-YTY7HHO3.js";import"./chunk-UOI75ONS.js";import"./chunk-4HFZZ7HS.js";import{e as r}from"./chunk-SZAW6ZBW.js";var A,v=r(()=>{m();d();p();c();s();A=[{path:"",loadComponent:()=>import("./chunk-65KHW5GG.js").then(t=>t.ConfigurationComponent),canActivate:[a,o],children:[{loadComponent:()=>import("./chunk-ZSJ7FPWF.js").then(t=>t.GeneralSettingsComponent),path:"general-settings",canActivate:[i],data:{permissions:[e.ServerSettingsGeneralView],editPermission:e.ServerSettingsGeneralEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-ME7UODHF.js").then(t=>t.TwoFactorConfigurationComponent),path:"two-factor",canActivate:[i],data:{permissions:[e.ServerSettingsMfaView],editPermission:e.ServerSettingsMfaEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-DLMXTFY3.js").then(t=>t.SecurityConfigurationComponent),path:"security",canActivate:[i],data:{permissions:[e.ServerSettingsSecurityView],editPermission:e.ServerSettingsSecurityEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-JQDGHHI3.js").then(t=>t.GeoIpConfigurationComponent),path:"geo-ip",canActivate:[i],data:{permissions:[e.ServerSettingsGeoIPView],editPermission:e.ServerSettingsGeoIPEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-G7LZNDPO.js").then(t=>t.ConfigurationLoggingComponent),path:"logging",canActivate:[i],data:{permissions:[e.ServerSettingsLogsView],editPermission:e.ServerSettingsLogsEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-ZSRUX6Q2.js").then(t=>t.FeaturesConfigurationComponent),path:"features",canActivate:[i],data:{permissions:[e.ServerSettingsFeaturesView],editPermission:e.ServerSettingsFeaturesEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-AUDC2UOI.js").then(t=>t.AdvancedConfigurationComponent),path:"advanced",canActivate:[i],data:{permissions:[e.ServerSettingsAdvancedView],editPermission:e.ServerSettingsAdvancedEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-RPJOSXDP.js").then(t=>t.ApprovalConfigurationComponent),path:"approval",canActivate:[i],data:{permissions:[e.ServerSettingsApprovalsView],editPermission:e.ServerSettingsApprovalsEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-J45LLHKT.js").then(t=>t.EmailConfigurationComponent),path:"email",canActivate:[i],data:{permissions:[e.ServerSettingsEmailView],editPermission:e.ServerSettingsEmailEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-JTZLXSHV.js").then(t=>t.AuthenticationComponent),path:"authentication",canActivate:[i],data:{permissions:[e.ServerSettingsAuthenticationView],editPermission:e.ServerSettingsAuthenticationEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-SOH67TP7.js").then(t=>t.AzureComponent),path:"authentication/office365",canActivate:[i],data:{permissions:[e.ServerSettingsAuthenticationView],editPermission:e.ServerSettingsAuthenticationEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-3KSTMHYH.js").then(t=>t.OktaComponent),path:"authentication/okta",canActivate:[i],data:{permissions:[e.ServerSettingsAuthenticationView],editPermission:e.ServerSettingsAuthenticationEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-CKY4GAE6.js").then(t=>t.PingOneComponent),path:"authentication/pingone",canActivate:[i],data:{permissions:[e.ServerSettingsAuthenticationView],editPermission:e.ServerSettingsAuthenticationEdit},canDeactivate:[n]},{path:"",loadComponent:()=>import("./chunk-FRICEANB.js").then(t=>t.ConfigurationDashboardComponent)},{path:"authentication/multi-domain",loadComponent:()=>import("./chunk-UXIJ7LDY.js").then(t=>t.MultiDomainListComponent),canActivate:[i],data:{permissions:[e.ServerSettingsAuthenticationView],editPermission:e.ServerSettingsAuthenticationEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-XKM7EC5X.js").then(t=>t.SecurityPolicyComponent),path:"security-policies",canActivate:[i],data:{permissions:[e.SecurityPoliciesView]}},{path:"user-behavior-analytics",loadComponent:()=>import("./chunk-TBHINUSQ.js").then(t=>t.UserBehaviorAnalyticsComponent),canActivate:[i],data:{permissions:[e.UserBehaviorAnalyticsConfigurationView]}}]}]});v();export{A as CONFIGURATION_ROUTES};
