import{Rd as m,ea as i,ka as n,na as b,sa as h}from"./chunk-4HFZZ7HS.js";import{e as f}from"./chunk-SZAW6ZBW.js";var w,d=f(()=>{b();m();w=(()=>{class t{constructor(){this.cache=new Map}get(e,c,o=6e4){let r=Date.now(),s=this.cache.get(e);if(s&&s.expiry>r)return s.observable;{let a=c().pipe(i(1),n(()=>{let l=r+o;this.cache.set(e,{observable:a,expiry:l})})),p=r+o;return this.cache.set(e,{observable:a,expiry:p}),a}}clearCache(e){e?this.cache.delete(e):this.cache.clear()}static{this.\u0275fac=function(c){return new(c||t)}}static{this.\u0275prov=h({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});export{w as a,d as b};
