import{b as u,c as l}from"./chunk-YTY7HHO3.js";import{Fb as a,Rd as h,Tb as n,g as i,ia as s,ma as p,na as c}from"./chunk-4HFZZ7HS.js";import{e as o}from"./chunk-SZAW6ZBW.js";var b,m=o(()=>{p();c();h();l();b=(()=>{class e{constructor(t){this.translatorService=t,this.update=!0,this.currentValue="",this.onDestroy$=new i,this.lastKeyValue="",t.updatedLang$.pipe(s(this.onDestroy$)).subscribe(()=>{this.update=!0})}ngOnDestroy(){this.onDestroy$.next()}transform(t,r=[]){return(!this.update||!t)&&this.lastKeyValue===t?this.currentValue:(this.lastKeyValue=t,this.currentValue=this.translatorService.translate(t,r),this.update=!!r,this.currentValue)}static{this.\u0275fac=function(r){return new(r||e)(a(u,16))}}static{this.\u0275pipe=n({name:"translate",type:e,pure:!1})}}return e})()});export{b as a,m as b};
