import{a as r,b as h}from"./chunk-UMVPH7GH.js";import"./chunk-36JWHXS5.js";import{a as o,b as p}from"./chunk-EKSSJ22J.js";import"./chunk-OETKJM3Q.js";import"./chunk-6WF6MHOA.js";import"./chunk-SADKIAAT.js";import"./chunk-YINSOKTV.js";import{a as t,b as e}from"./chunk-CXJOJS5Y.js";import"./chunk-ILBO5ZGN.js";import"./chunk-677WCWHS.js";import"./chunk-WMHCILDY.js";import"./chunk-UUSZN4EV.js";import"./chunk-OOETPT6P.js";import"./chunk-M6JBMHZW.js";import"./chunk-YSSBDB5I.js";import"./chunk-BXBF54ET.js";import"./chunk-5DATZV72.js";import"./chunk-3RHWAFOM.js";import"./chunk-LG3SCC2E.js";import"./chunk-R2ULAEP2.js";import"./chunk-ZSHAELBT.js";import"./chunk-UGQXGBKF.js";import"./chunk-TCUK52QT.js";import"./chunk-JU5LF7AN.js";import"./chunk-AFLM5QFV.js";import"./chunk-DCQJQBJ7.js";import"./chunk-VYRWAFSM.js";import"./chunk-MWQIPWDG.js";import"./chunk-UTF7W4HP.js";import"./chunk-OJRDBZWF.js";import"./chunk-LFKM2JDJ.js";import"./chunk-6G52AAKU.js";import"./chunk-QQZCALHS.js";import"./chunk-EXP5MDOK.js";import"./chunk-M55KBHC5.js";import"./chunk-JE2X3YFA.js";import"./chunk-HW6LKGMV.js";import"./chunk-XRGTMJIH.js";import"./chunk-GDJMZZ4C.js";import"./chunk-V3J5YJ3L.js";import"./chunk-UIVS6623.js";import"./chunk-2XIOYYFS.js";import"./chunk-HYPRVELK.js";import"./chunk-YTY7HHO3.js";import"./chunk-UOI75ONS.js";import"./chunk-4HFZZ7HS.js";import{e as m}from"./chunk-SZAW6ZBW.js";var s,i=m(()=>{e();p();h();s=[{canActivate:[t,o],providers:[r],path:"",pathMatch:"full",loadComponent:()=>import("./chunk-56VWDGHK.js").then(a=>a.SystemDashboardComponent)}]});i();export{s as SYSDASHBOARD_ROUTES};
