import{P as n,Q as h}from"./chunk-JE2X3YFA.js";import{a as l}from"./chunk-YTY7HHO3.js";import{Fb as a,Rd as p,Tb as m}from"./chunk-4HFZZ7HS.js";import{e as c,h as u}from"./chunk-SZAW6ZBW.js";var s,F,f=c(()=>{s=u(l());p();h();F=(()=>{class r{constructor(e){this.currentUserService=e}get displayDateFormat(){return this.currentUserService.webSettingsSnapshot&&(this.currentUserService.displayDateFormat||"").trim().length>0?this.currentUserService.displayDateFormat:"M/D/YYYY"}get displayTimeFormat(){return this.currentUserService.webSettingsSnapshot&&(this.currentUserService.displayTimeFormat||"").trim().length>0?this.currentUserService.displayTimeFormat:"HH:mm"}get displayDateTimeFormat(){return this.currentUserService.webSettingsSnapshot&&(this.currentUserService.displayDateTimeFormat||"").trim().length>0?this.currentUserService.displayDateTimeFormat:"M/D/YYYY HH:mm"}transform(e,t,o){if(!e)return"";let i;return o?i=s.default.unix(Number.parseInt(e,10)).local():i=s.default.utc(e).local(),i.isValid()?this.formatDate(t,i):""}formatDate(e,t){switch(e){case"date":return t.format(this.displayDateFormat);case"time":return t.format(this.displayTimeFormat);case"datetime":return t.format(this.displayDateTimeFormat)}return""}static{this.\u0275fac=function(t){return new(t||r)(a(n,16))}}static{this.\u0275pipe=m({name:"dateFormat",type:r,pure:!0})}}return r})()});export{F as a,f as b};
