import{a as m,b as S}from"./chunk-ZENOWJOG.js";import{Wa as f,Xa as v}from"./chunk-7YUL6BIM.js";import{O as a,Rd as h,g as i,ma as u,na as l,r as n,sa as s,xa as o}from"./chunk-4HFZZ7HS.js";import{e as c}from"./chunk-SZAW6ZBW.js";var y,d=c(()=>{u();l();h();v();S();y=(()=>{class e{constructor(r,t){this.pamAssetServices=r,this.errorService=t}getJob(r){return this.handleError(this.pamAssetServices.AccountDiscoveryResults_GetScan(r))}getLastJobs(){return this.handleError(this.pamAssetServices.AccountDiscoveryResults_GetLastScans())}handleError(r,t=null){return r.pipe(a(p=>(this.errorService.handleError(p),n(t))))}static{this.\u0275fac=function(t){return new(t||e)(o(f),o(m))}}static{this.\u0275prov=s({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});var x,R=c(()=>{u();h();x=(()=>{class e{constructor(){this.viewRequestSource=new i,this.refreshRequestSource=new i,this.viewRequest=this.viewRequestSource.asObservable(),this.refreshRequest=this.refreshRequestSource.asObservable()}openDialog(r){this.viewRequestSource.next(r)}refresh(){this.refreshRequestSource.next()}static{this.\u0275fac=function(t){return new(t||e)}}static{this.\u0275prov=s({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});export{y as a,d as b,x as c,R as d};
