import{i as a,j as l}from"./chunk-2JML2BCE.js";import{e}from"./chunk-SZAW6ZBW.js";var t=e(()=>{l();a.setLanguage(["tr","tr-TR"],{Markdown:"Markdown",WYSIWYG:"WYSIWYG",Write:"D\xFCzenle",Preview:"\xD6n izleme",Headings:"Ba\u015Fl\u0131klar",Paragraph:"Paragraf",Bold:"Kal\u0131n",Italic:"\u0130talik",Strike:"Alt\u0131 \xE7izgili",Code:"Sat\u0131r i\xE7i kod",Line:"\xC7izgi",Blockquote:"Al\u0131nt\u0131","Unordered list":"S\u0131ralanmam\u0131\u015F liste","Ordered list":"S\u0131ral\u0131 liste",Task:"G\xF6rev kutusu",Indent:"Girintiyi artt\u0131r",Outdent:"Girintiyi azalt","Insert link":"Ba\u011Flant\u0131 ekle","Insert CodeBlock":"Kod bloku ekle","Insert table":"Tablo ekle","Insert image":"\u0130maj ekle",Heading:"Ba\u015Fl\u0131k","Image URL":"\u0130maj URL","Select image file":"\u0130maj dosyas\u0131 se\xE7","Choose a file":"Bir dosya se\xE7in","No file":"Dosya yok",Description:"A\xE7\u0131klama",OK:"Onay",More:"Daha Fazla",Cancel:"\u0130ptal",File:"Dosya",URL:"URL","Link text":"Ba\u011Flant\u0131 yaz\u0131s\u0131","Add row to up":"Yukar\u0131 sat\u0131r ekle","Add row to down":"A\u015Fa\u011F\u0131 sat\u0131r ekle","Add column to left":"Sola s\xFCtun ekleyin","Add column to right":"Sa\u011Fa s\xFCtun ekle","Remove row":"Sat\u0131r sil","Remove column":"S\xFCtun sil","Align column to left":"Sola hizala","Align column to center":"Merkeze hizala","Align column to right":"Sa\u011Fa hizala","Remove table":"Tabloyu kald\u0131r","Would you like to paste as table?":"Tablo olarak yap\u0131\u015Ft\u0131rmak ister misiniz?","Text color":"Metin rengi","Auto scroll enabled":"Otomatik kayd\u0131rma a\xE7\u0131k","Auto scroll disabled":"Otomatik kayd\u0131rma kapal\u0131","Choose language":"Dil se\xE7iniz"})});t();
