import{e as o,f as t}from"./chunk-OOETPT6P.js";import{Rd as l,sa as a}from"./chunk-4HFZZ7HS.js";import{e}from"./chunk-SZAW6ZBW.js";var v,i=e(()=>{t();l();v=(()=>{class n{constructor(){this.productConfig={[o.Unknown]:{name:"Unknown",icon:"dvl-icon-logo-devolutions",bgColor:"var(--accent-brand-100)",iconColor:"var(--accent-brand-400)"},[o.CAL]:{name:"CAL",icon:"dvl-icon-logo-dps",bgColor:"var(--products-dvls-200)",iconColor:"var(--products-dvls-logo)"},[o.RDM]:{name:"RDM",icon:"dvl-icon-logo-rdm",bgColor:"var(--products-rdm-200)",iconColor:"var(--products-rdm-logo)"},[o.DCL]:{name:"Launcher",icon:"dvl-icon-activate-launch",bgColor:"var(--products-launcher-200)",iconColor:"var(--products-launcher-logo)"},[o.DVLS]:{name:"DVLS",icon:"dvl-icon-logo-dps",bgColor:"var(--products-dvls-200)",iconColor:"var(--products-dvls-logo)"},[o.HUB]:{name:"Hub",icon:"dvl-icon-logo-hub-enterprise",bgColor:"var(--accent-brand-100)",iconColor:"var(--accent-brand-400)"},[o.CYB]:{name:"CyberArk",icon:"dvl-icon-entry-CyberArkPSMConnection",bgColor:"var(--accent-brand-100)",iconColor:"var(--accent-brand-400)"},[o.DEL]:{name:"Delinea",icon:"dvl-icon-entry-delinea-secret-server",bgColor:"var(--accent-brand-100)",iconColor:"var(--accent-brand-400)"},[o.BET]:{name:"BeyondTrust",icon:"dvl-icon-entry-BeyondTrustConnection",bgColor:"var(--accent-brand-100)",iconColor:"var(--accent-brand-400)"},[o.GAT]:{name:"Gateway",icon:"dvl-icon-logo-gateway",bgColor:"var(--products-gateway-200)",iconColor:"var(--products-gateway-logo)"},[o.PAM]:{name:"PAM",icon:"dvl-icon-logo-pam",bgColor:"var(--products-pam-200)",iconColor:"var(--products-pam-logo)"},[o.DWS]:{name:"Workspace",icon:"dvl-icon-logo-workspace",bgColor:"var(--products-workspace-200)",iconColor:"var(--products-workspace-logo)"},[o.PSU]:{name:"PowerShellUniversal",icon:"dvl-icon-logo-powershell-universal",bgColor:"var(--products-powershell-200)",iconColor:"var(--products-powershell-logo)"}},this.defaultConfig={name:"Unknown",icon:"dvl-icon-logo-devolutions",bgColor:"var(--accent-brand-100)",iconColor:"var(--accent-brand-400)"}}getConfig(r){return this.productConfig[r]??this.defaultConfig}getIcon(r){return this.getConfig(r).icon}getColors(r){let c=this.getConfig(r);return{bgColor:c.bgColor,iconColor:c.iconColor}}getName(r){return this.getConfig(r).name}static{this.\u0275fac=function(c){return new(c||n)}}static{this.\u0275prov=a({token:n,factory:n.\u0275fac,providedIn:"root"})}}return n})()});export{v as a,i as b};
