import{d as w,e as x,i as I,j as L}from"./chunk-VYRWAFSM.js";import{Ia as i,La as $}from"./chunk-2XIOYYFS.js";import{D as C,K as F,m as S,s as M}from"./chunk-UOI75ONS.js";import{Ab as a,La as g,Na as o,Qb as f,Rb as m,Rd as D,Ub as u,ic as t,jc as y,jd as b,kc as h,sa as l,ta as c,wc as p,xc as k,ya as d,yc as v}from"./chunk-4HFZZ7HS.js";import{e as B}from"./chunk-SZAW6ZBW.js";var N,W,j,P,X,T=B(()=>{F();F();D();D();$();L();x();N=({dt:e})=>`
.p-progressspinner {
    position: relative;
    margin: 0 auto;
    width: 100px;
    height: 100px;
    display: inline-block;
}

.p-progressspinner::before {
    content: "";
    display: block;
    padding-top: 100%;
}

.p-progressspinner-spin {
    height: 100%;
    transform-origin: center center;
    width: 100%;
    position: absolute;
    top: 0;
    bottom: 0;
    left: 0;
    right: 0;
    margin: auto;
    animation: p-progressspinner-rotate 2s linear infinite;
}

.p-progressspinner-circle {
    stroke-dasharray: 89, 200;
    stroke-dashoffset: 0;
    stroke: ${e("progressspinner.colorOne")};
    animation: p-progressspinner-dash 1.5s ease-in-out infinite, p-progressspinner-color 6s ease-in-out infinite;
    stroke-linecap: round;
}

@keyframes p-progressspinner-rotate {
    100% {
        transform: rotate(360deg);
    }
}
@keyframes p-progressspinner-dash {
    0% {
        stroke-dasharray: 1, 200;
        stroke-dashoffset: 0;
    }
    50% {
        stroke-dasharray: 89, 200;
        stroke-dashoffset: -35px;
    }
    100% {
        stroke-dasharray: 89, 200;
        stroke-dashoffset: -124px;
    }
}
@keyframes p-progressspinner-color {
    100%,
    0% {
        stroke: ${e("progressspinner.colorOne")};
    }
    40% {
        stroke: ${e("progressspinner.colorTwo")};
    }
    66% {
        stroke: ${e("progressspinner.colorThree")};
    }
    80%,
    90% {
        stroke: ${e("progressspinner.colorFour")};
    }
}
`,W={root:"p-progressspinner",spin:"p-progressspinner-spin",circle:"p-progressspinner-circle"},j=(()=>{class e extends w{name="progressspinner";theme=N;classes=W;static \u0275fac=(()=>{let s;return function(r){return(s||(s=o(e)))(r||e)}})();static \u0275prov=l({token:e,factory:e.\u0275fac})}return e})(),P=(()=>{class e extends I{styleClass;style;strokeWidth="2";fill="none";animationDuration="2s";ariaLabel;_componentStyle=d(j);static \u0275fac=(()=>{let s;return function(r){return(s||(s=o(e)))(r||e)}})();static \u0275cmp=f({type:e,selectors:[["p-progressSpinner"],["p-progress-spinner"],["p-progressspinner"]],inputs:{styleClass:"styleClass",style:"style",strokeWidth:"strokeWidth",fill:"fill",animationDuration:"animationDuration",ariaLabel:"ariaLabel"},features:[b([j]),u],decls:3,vars:11,consts:[["role","progressbar",1,"p-progressspinner",3,"ngStyle","ngClass"],["viewBox","25 25 50 50",1,"p-progressspinner-spin"],["cx","50","cy","50","r","20","stroke-miterlimit","10",1,"p-progressspinner-circle"]],template:function(n,r){n&1&&(p(0,"div",0),g(),p(1,"svg",1),v(2,"circle",2),k()()),n&2&&(y("ngStyle",r.style)("ngClass",r.styleClass),t("aria-label",r.ariaLabel)("aria-busy",!0)("data-pc-name","progressspinner")("data-pc-section","root"),a(),h("animation-duration",r.animationDuration),t("data-pc-section","root"),a(),t("fill",r.fill)("stroke-width",r.strokeWidth))},dependencies:[C,S,M,i],encapsulation:2,changeDetection:0})}return e})(),X=(()=>{class e{static \u0275fac=function(n){return new(n||e)};static \u0275mod=m({type:e});static \u0275inj=c({imports:[P,i,i]})}return e})()});export{P as a,X as b,T as c};
