import{i as b,j as S}from"./chunk-ILBO5ZGN.js";import"./chunk-MWQIPWDG.js";import"./chunk-UTF7W4HP.js";import"./chunk-OJRDBZWF.js";import{a as v,b as _}from"./chunk-LFKM2JDJ.js";import"./chunk-6G52AAKU.js";import"./chunk-QQZCALHS.js";import"./chunk-EXP5MDOK.js";import"./chunk-M55KBHC5.js";import{h as f,i as C}from"./chunk-JE2X3YFA.js";import"./chunk-HW6LKGMV.js";import"./chunk-GDJMZZ4C.js";import"./chunk-V3J5YJ3L.js";import"./chunk-UIVS6623.js";import"./chunk-2XIOYYFS.js";import"./chunk-HYPRVELK.js";import"./chunk-YTY7HHO3.js";import"./chunk-UOI75ONS.js";import{$c as c,Ab as o,Fb as s,Qb as g,Rd as h,Zc as a,jc as x,ud as p,vb as l,vd as d,wc as i,xc as n,yc as m}from"./chunk-4HFZZ7HS.js";import{e as u}from"./chunk-SZAW6ZBW.js";var z,M=u(()=>{_();h();S();C();z=(()=>{class t{constructor(r,e){this.authService=r,this.storage=e}ngOnInit(){this.authService.clearLoginInformationAfterExternalLoginSuccess(),this.storage.removeApplicationStorage("oauth")}static{this.\u0275fac=function(e){return new(e||t)(s(b),s(f))}}static{this.\u0275cmp=g({type:t,selectors:[["dvls-login-success"]],decls:10,vars:7,consts:[[1,"box"],[1,"box__logo"],["alt","Devolutions Server",1,"dvls-home-logo",3,"src"],[1,"box__title"],[1,"box__message"]],template:function(e,P){e&1&&(i(0,"div")(1,"div",0)(2,"div",1),m(3,"img",2),n(),i(4,"div",3),a(5),p(6,"translate"),n(),i(7,"div",4),a(8),p(9,"translate"),n()()()),e&2&&(o(3),x("src","assets/images/devolutions-password-server-logo.svg",l),o(2),c(" ",d(6,3,"lblSuccess")," "),o(3),c(" ",d(9,5,"YouCanNowCloseThisPage")," "))},dependencies:[v],styles:[".box[_ngcontent-%COMP%]{color:#424242;text-align:center;width:460px;box-shadow:10px 10px 115px 40px #00336624;border-radius:4px;padding:31px 44px 12px;z-index:1;background-color:#fff;margin-top:10vh;position:fixed;left:50%;transform:translate(-50%)}.box[_ngcontent-%COMP%]   .box__logo[_ngcontent-%COMP%]   .dvls-home-logo[_ngcontent-%COMP%]{box-sizing:border-box;margin:0 auto;max-width:300px;overflow:hidden;padding:25px;position:relative;width:100%}.box[_ngcontent-%COMP%]   .box__title[_ngcontent-%COMP%]{font-size:1.4rem;color:#0068c3;font-weight:700}.box[_ngcontent-%COMP%]   .box__message[_ngcontent-%COMP%]{margin:25px;color:#406900;background-color:#f8fbf4;padding:12px;border-radius:4px;border-left:#78b41b 2px solid}"]})}}return t})()});M();export{z as LoginSuccessComponent};
