import{e as C}from"./chunk-SZAW6ZBW.js";var s,E=C(()=>{s=function(n){return n[n.Contains=0]="Contains",n[n.StartsWith=1]="StartsWith",n[n.EndsWith=2]="EndsWith",n[n.ExactExpression=3]="ExactExpression",n[n.DoesNotContain=4]="DoesNotContain",n[n.Regex=5]="Regex",n}(s||{});(function(n){function i(){return"SearchItemText"}n.getClass=i;function u(g){let o="";return typeof n.getClass=="function"&&(o=n.getClass()),o+"_"+n[g]}n.getEnumKey=u})(s||(s={}))});export{s as a,E as b};
