import{e as a}from"./chunk-SZAW6ZBW.js";var e,r=a(()=>{e=class{constructor(s,i,t,h=null,l=null){this.connectionType=s,this.connectionSubType=i,this.label=t,this.masterSubType=l,this.imageKey="",this.canHaveTemplate=!0,this.isBlank=!1,this.section="general",this.groupLabel="ConnectionTypeGroup_General",this.section=h??"general",["Session"].includes(s)?this.key=i:l?this.key=`${s}.${i}.${l}`:this.key=`${s}.${i}`,this.groupLabel=`ConnectionType_${s}`}}});export{e as a,r as b};
