import{a as P,b as G}from"./chunk-T6GVOQU2.js";import{b as R,c as B}from"./chunk-ZEZBKBHR.js";import{a as T,b as x}from"./chunk-IA4ZN6SL.js";import"./chunk-BCH4TWDY.js";import"./chunk-HH37EZMD.js";import"./chunk-ZSHAELBT.js";import"./chunk-6G52AAKU.js";import"./chunk-QQZCALHS.js";import"./chunk-EXP5MDOK.js";import"./chunk-M55KBHC5.js";import{D as U,E as y,P as O,Q as k,c as g,e as M,f as I,g as _,h as A,i as D}from"./chunk-JE2X3YFA.js";import{q as h,v as C}from"./chunk-HW6LKGMV.js";import"./chunk-GDJMZZ4C.js";import"./chunk-V3J5YJ3L.js";import"./chunk-UIVS6623.js";import"./chunk-2XIOYYFS.js";import"./chunk-HYPRVELK.js";import"./chunk-YTY7HHO3.js";import"./chunk-UOI75ONS.js";import{J as u,Rd as l,ha as S,ka as v,ma as N,na as w,r as p,sa as s,x as f,xa as a,ya as c}from"./chunk-4HFZZ7HS.js";import{e as m}from"./chunk-SZAW6ZBW.js";var E,L=m(()=>{w();l();C();B();k();M();_();E=(()=>{class e{constructor(i,t,o,r,n){this.router=i,this.urlService=t,this.currentUserService=o,this.oauthService=r,this.baseUrlService=n}canActivate(i,t){return this.currentUserService.currentUserChanged.pipe(u(o=>!!o),S(()=>this.urlService.getDefaultUrl().pipe(v(o=>{let r=decodeURIComponent(this.oauthService.state);if(this.oauthService.state="",r){let n=new URL(this.baseUrlService.getCompletePathForRoute(r)),j=this.baseUrlService.getParamsAsObject(n.searchParams);r=r.split("?")[0],this.router.navigate([r],{queryParams:j})}else this.router.navigateByUrl(o)}),f(()=>!0))))}static{this.\u0275fac=function(t){return new(t||e)(a(h),a(R),a(O),a(g),a(I))}}static{this.\u0275prov=s({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});var d,b=m(()=>{l();C();N();D();y();l();d=(()=>{class e{constructor(){this.router=c(h),this.storageService=c(A),this.serverInformationService=c(U)}canActivate(i,t){return this.checkLicenseManagement(t)}canActivateChild(i,t){return this.checkLicenseManagement(t)}checkLicenseManagement(i){return this.serverInformationService.privateServerInformation?.limitAccessToLicensePage&&!["/administration/licenses","/logout"].some(n=>i.url.startsWith(n))?(this.router.navigate(["/administration/licenses"]),p(!1)):p(!0)}static{this.\u0275fac=function(t){return new(t||e)}}static{this.\u0275prov=s({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});var se,q=m(()=>{L();G();x();b();se=[{path:"",loadComponent:()=>import("./chunk-44UM73SS.js").then(e=>e.ApplicationComponent),canActivate:[d],canActivateChild:[d],children:[{path:"",loadComponent:()=>import("./chunk-CM4MDL6Y.js").then(e=>e.LandingPageComponent),canActivate:[E],pathMatch:"full"},{path:"connections",loadChildren:()=>import("./chunk-GVMMJ4Y7.js").then(e=>e.CONNECTIONS_ROUTES),providers:[T]},{path:"dashboard",redirectTo:""},{path:"administration",loadChildren:()=>import("./chunk-UEGRH3NM.js").then(e=>e.ADMINISTRATIONS_ROUTES)},{path:"pam",loadChildren:()=>import("./chunk-QACG7WLB.js").then(e=>e.PAM_ROUTES)},{path:"privileged-session-monitoring",loadChildren:()=>import("./chunk-J3IJCBCJ.js").then(e=>e.PRIVILEGED_SESSIONS_MONITORING)},{path:"reports",loadChildren:()=>import("./chunk-RVT5OOR4.js").then(e=>e.REPORTS_ROUTES)},{path:"api-documentation",loadComponent:()=>import("./chunk-UR2J6VAS.js").then(e=>e.ApiDocumentationComponent)},{path:"tools",loadChildren:()=>import("./chunk-RDJIL35X.js").then(e=>e.TOOLS_ROUTES)},{path:"ui",loadChildren:()=>import("./chunk-UVXQMFGD.js").then(e=>e.UI_ROUTES)},{path:"system-dashboard",redirectTo:"admin-dashboard",pathMatch:"full"},{path:"admin-dashboard",loadChildren:()=>import("./chunk-H5PCMGGQ.js").then(e=>e.SYSDASHBOARD_ROUTES)},{path:"web-client",loadChildren:()=>import("./chunk-YMTBHGSC.js").then(e=>e.WEB_CLIENT_ROUTES)},{path:"notification-preferences",loadComponent:()=>import("./chunk-ADUPO3UA.js").then(e=>e.NotificationPreferencesDashboardComponent),canDeactivate:[P]}]}]});q();export{se as APPLICATION_ROUTES};
