import{wa as p,ya as m}from"./chunk-VYRWAFSM.js";import{Fb as e,Gc as l,Rd as h,Sb as s,Ya as n}from"./chunk-4HFZZ7HS.js";import{e as d}from"./chunk-SZAW6ZBW.js";var T,u=d(()=>{h();m();T=(()=>{class i{constructor(o,t){this.tooltip=o,this.elementRef=t,this.initialTooltipPosition=this.tooltip._tooltipOptions.tooltipPosition}setTooltipState(){let t=this.elementRef.nativeElement.getBoundingClientRect();(this.tooltip._tooltipOptions.tooltipPosition===void 0||this.tooltip._tooltipOptions.tooltipPosition==="right"||this.tooltip._tooltipOptions.tooltipPosition!==this.initialTooltipPosition)&&window.innerWidth-t.right<200?this.tooltip._tooltipOptions.tooltipPosition="left":this.tooltip._tooltipOptions.tooltipPosition=this.initialTooltipPosition??"right"}static{this.\u0275fac=function(t){return new(t||i)(e(p),e(n))}}static{this.\u0275dir=s({type:i,selectors:[["","pTooltip","","dvlsTooltipPositionAdjustment",""],["","dvlTooltip","","dvlsTooltipPositionAdjustment",""]],hostBindings:function(t,r){t&1&&l("mouseenter",function(c){return r.setTooltipState(c)})}})}}return i})()});export{T as a,u as b};
