import{a as c,b as p}from"./chunk-5JRUNHF6.js";import{Fb as t,Rd as s,Tb as r}from"./chunk-4HFZZ7HS.js";import{e as o}from"./chunk-SZAW6ZBW.js";var m,a=o(()=>{s();p();m=(()=>{class e{constructor(i){this.fileService=i}transform(i){return this.fileService.getHumanFriendlyFileSize(i)}static{this.\u0275fac=function(n){return new(n||e)(t(c,16))}}static{this.\u0275pipe=r({name:"fileSize",type:e,pure:!0})}}return e})()});export{m as a,a as b};
