import{a as u,b as v}from"./chunk-KXKR53UI.js";import{a as h,b as y}from"./chunk-R6R2UCTL.js";import{D as d,K as f,m as g}from"./chunk-UOI75ONS.js";import{Ab as i,Qb as s,Rd as T,Zc as p,_c as m,jc as o,ud as c,wc as a,wd as l,xc as r}from"./chunk-4HFZZ7HS.js";import{e as M}from"./chunk-SZAW6ZBW.js";var b,x=M(()=>{f();v();y();T();f();b=(()=>{class e{constructor(){this.systemUserTags=u}static{this.\u0275fac=function(t){return new(t||e)}}static{this.\u0275cmp=s({type:e,selectors:[["sd-user-tag-chip"]],inputs:{VMTag:"VMTag"},decls:5,vars:5,consts:[[1,"chip-container",3,"ngClass"],[1,"chip-label"]],template:function(t,n){t&1&&(a(0,"div",0)(1,"div",1)(2,"span"),p(3),c(4,"translateEnum"),r()()()),t&2&&(o("ngClass",n.VMTag.theme),i(3),m(l(4,2,n.VMTag.name,n.systemUserTags)))},dependencies:[d,g,h],styles:["[_nghost-%COMP%]{margin:0 5px}[_nghost-%COMP%]   .chip-container[_ngcontent-%COMP%]{cursor:default;display:inline-flex;align-items:center;flex:0 0 auto;border-radius:4px;line-height:24px}[_nghost-%COMP%]   .chip-container[_ngcontent-%COMP%]   .chip-label[_ngcontent-%COMP%]{font-size:14px;padding:0 .5em}"]})}}return e})()});export{b as a,x as b};
