import{a as n,c as m}from"./chunk-VHKJVCTZ.js";import{a as o,b as s}from"./chunk-CXJOJS5Y.js";import"./chunk-ILBO5ZGN.js";import"./chunk-ZSHAELBT.js";import"./chunk-MWQIPWDG.js";import"./chunk-UTF7W4HP.js";import"./chunk-OJRDBZWF.js";import"./chunk-6G52AAKU.js";import"./chunk-QQZCALHS.js";import"./chunk-EXP5MDOK.js";import"./chunk-M55KBHC5.js";import"./chunk-JE2X3YFA.js";import"./chunk-HW6LKGMV.js";import{Yb as i,Zb as e}from"./chunk-GDJMZZ4C.js";import"./chunk-V3J5YJ3L.js";import"./chunk-UIVS6623.js";import"./chunk-2XIOYYFS.js";import"./chunk-HYPRVELK.js";import"./chunk-YTY7HHO3.js";import"./chunk-UOI75ONS.js";import"./chunk-4HFZZ7HS.js";import{e as t}from"./chunk-SZAW6ZBW.js";var I,a=t(()=>{m();s();e();I=[{canActivate:[o,n],data:{permissions:[i.MonitoringPrivilegedSessionsView]},path:"",loadComponent:()=>import("./chunk-VNQHTNOD.js").then(r=>r.PrivilegedSessionMonitoringComponent)}]});a();export{I as PRIVILEGED_SESSIONS_MONITORING};
