import{a,b as V}from"./chunk-3RHWAFOM.js";import{a as n,b as R}from"./chunk-ZSHAELBT.js";import{f as m,g as H,h as p,i as T}from"./chunk-JE2X3YFA.js";import{f as c,j as v}from"./chunk-HYPRVELK.js";import{b as l,c as y}from"./chunk-YTY7HHO3.js";import{Rd as j,sa as u,xa as i}from"./chunk-4HFZZ7HS.js";import{e as b,f as C,h as U}from"./chunk-SZAW6ZBW.js";var f=C((L,g)=>{var A=["login","login-success","message"];g.exports=A});var h,O,F=b(()=>{R();V();h=U(f());y();j();v();T();H();O=(()=>{class s extends l{constructor(t,e,r){super(t,e),this.baseUrlService=r}translate(t,e){return e!==null?super.translate(t,this.escapeValuesToInclude(e)):super.translate(t)}getTranslationUrl(){let t=this.storageService.get().culture,e=a.getLanguageFilename(a.fromValue(t)),r="./assets/translations/",o=this.baseUrlService.getCurrentRoute().substring(1);return h.indexOf(o)>-1&&(r="./assets/translations",r+=`${this.baseUrlService.getCurrentRoute()}/`),`${r+n.i18nPath+e}.json`}getCustomTranslationUrl(){let t=["fr","de","en_US"],e=this.storageService.get().culture,r=a.getLanguageFilename(a.fromValue(e));return t.some(S=>S===r)||(r="en_US"),`${"./assets/translations/custom/"+n.i18nPath+r}.json`}escapeHtmlTags(t){return t.replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/>/g,"&gt;").replace(/"/g,"&quot;").replace(/'/g,"&#039;")}escapeValuesToInclude(t){return t&&(Array.isArray(t)?t.map(e=>typeof e=="string"?this.escapeHtmlTags(e):e):typeof t=="string"?this.escapeHtmlTags(t):t)}static{this.\u0275fac=function(e){return new(e||s)(i(c),i(p),i(m))}}static{this.\u0275prov=u({token:s,factory:s.\u0275fac,providedIn:"root"})}}return s})()});export{O as a,F as b};
