import{a as k,b as z}from"./chunk-LFKM2JDJ.js";import{a as B,b as P}from"./chunk-UIVS6623.js";import{K as O,m as F}from"./chunk-UOI75ONS.js";import{$c as _,Ab as t,Cc as w,Gc as E,Hc as r,Ja as x,Ka as D,Na as h,Qb as S,Rd as I,Ta as V,Ub as b,Wb as v,Zc as d,_c as f,jc as T,md as R,rc as s,tc as c,uc as u,ud as g,vc as m,vd as C,wc as a,xc as o,yc as p}from"./chunk-4HFZZ7HS.js";import{e as M}from"./chunk-SZAW6ZBW.js";function j(e,n){e&1&&(a(0,"div",6),d(1),g(2,"translate"),o()),e&2&&(t(),f(C(2,1,"lblHideDetails")))}function H(e,n){e&1&&(a(0,"div",6),d(1),g(2,"translate"),o()),e&2&&(t(),f(C(2,1,"lblShowDetails")))}function N(e,n){if(e&1){let i=w();a(0,"div",4),E("click",function(){x(i);let l=r();return D(l.toggleDetails())}),a(1,"div",5),v(2,j,3,3,"div",6)(3,H,3,3,"div",6),a(4,"div"),p(5,"i",7),o()()()}if(e&2){let i=r();t(2),s(i.detailsVisible?2:-1),t(),s(i.detailsVisible?-1:3),t(2),T("ngClass",R(3,$,i.detailsVisible,!i.detailsVisible))}}function U(e,n){if(e&1&&(a(0,"li",8)(1,"div",9),p(2,"i",10),d(3),o()()),e&2){let i=n.$implicit;t(3),_(" ",i," ")}}function q(e,n){if(e&1&&(a(0,"ul",3),u(1,U,4,1,"li",8,c),o()),e&2){let i=r();t(),m(i.roleDisplayValues)}}function A(e,n){if(e&1&&(a(0,"li",8)(1,"div",9),p(2,"i",11),d(3),o()()),e&2){let i=n.$implicit;t(3),_(" ",i," ")}}function G(e,n){if(e&1&&(a(0,"ul",3),u(1,A,4,1,"li",8,c),o()),e&2){let i=r();t(),m(i.userDisplayValues)}}function J(e,n){if(e&1&&(a(0,"li",8)(1,"div",9),p(2,"i",12),d(3),o()()),e&2){let i=n.$implicit;t(3),_(" ",i," ")}}function K(e,n){if(e&1&&(a(0,"ul",3),u(1,J,4,1,"li",8,c),o()),e&2){let i=r();t(),m(i.applicationsDisplayValues)}}function L(e,n){if(e&1&&(a(0,"li",8)(1,"div",9),p(2,"i",13),d(3),o()()),e&2){let i=n.$implicit;t(3),_(" ",i," ")}}function Q(e,n){if(e&1&&(a(0,"ul",3),u(1,L,4,1,"li",8,c),o()),e&2){let i=r();t(),m(i.vaultDisplayValues)}}var $,ae,W=M(()=>{I();P();O();z();I();$=(e,n)=>({"dvl-icon-arrow-up":e,"dvl-icon-arrow-down":n});ae=(()=>{class e extends B{constructor(){super(...arguments),this.userDisplayValues=[],this.roleDisplayValues=[],this.applicationsDisplayValues=[],this.vaultDisplayValues=[],this.detailsVisible=!1,this.detailsVisibleChange=new V}toggleDetails(){this.detailsVisibleChange?.emit(!this.detailsVisible)}static{this.\u0275fac=(()=>{let i;return function(l){return(i||(i=h(e)))(l||e)}})()}static{this.\u0275cmp=S({type:e,selectors:[["dvls-connection-security-right-details"]],inputs:{userDisplayValues:"userDisplayValues",roleDisplayValues:"roleDisplayValues",applicationsDisplayValues:"applicationsDisplayValues",vaultDisplayValues:"vaultDisplayValues",detailsVisible:"detailsVisible"},outputs:{detailsVisibleChange:"detailsVisibleChange"},features:[b],decls:7,vars:5,consts:[[2,"width","100%"],[2,"display","flex","padding-right","16px","cursor","pointer","justify-content","flex-end"],[2,"display","flex","margin-left","11px","border-left","var(--dialog-menu-border-color) 1px solid"],[2,"margin","0 0 5px"],[2,"display","flex","padding-right","16px","cursor","pointer","justify-content","flex-end",3,"click"],[2,"display","flex","align-items","center","font-size","11px"],[1,"details-label"],[1,"dvl-icon","caret-icon",3,"ngClass"],[1,"username-display"],[2,"display","flex","align-items","center"],[1,"di","dvl-icon-roles","detail-icon"],[1,"di","dvl-icon-entry-SampleUser","detail-icon"],[1,"di","dvl-icon-entry-session-command-line","detail-icon"],[1,"di","dvl-icon-vault","detail-icon"]],template:function(y,l){y&1&&(a(0,"div",0),v(1,N,6,6,"div",1),a(2,"div",2),v(3,q,3,0,"ul",3)(4,G,3,0,"ul",3)(5,K,3,0,"ul",3)(6,Q,3,0,"ul",3),o()()),y&2&&(t(),s(l.roleDisplayValues!=null&&l.roleDisplayValues.length||l.userDisplayValues!=null&&l.userDisplayValues.length||l.applicationsDisplayValues!=null&&l.applicationsDisplayValues.length||l.vaultDisplayValues!=null&&l.vaultDisplayValues.length?1:-1),t(2),s(l.detailsVisible?3:-1),t(),s(l.detailsVisible?4:-1),t(),s(l.detailsVisible?5:-1),t(),s(l.detailsVisible?6:-1))},dependencies:[k,F],styles:[".username-display[_ngcontent-%COMP%]{padding:5px 10px;width:100%;white-space:nowrap;overflow:hidden;text-overflow:ellipsis;font-size:12px;margin:0}.label[_ngcontent-%COMP%]{align-items:center}.caret-icon[_ngcontent-%COMP%]{font-size:20px;color:var(--fieldset-title-color)}.detail-icon[_ngcontent-%COMP%]{font-size:16px;padding-right:2px}.details-label[_ngcontent-%COMP%]{color:var(--fieldset-title-color)}"]})}}return e})()});export{ae as a,W as b};
