import{a as y}from"./chunk-PQQRTQYF.js";import{d as I}from"./chunk-QCYFTXYO.js";import{a as i,b as T}from"./chunk-3RHWAFOM.js";import{L as p,M as h,P as f,Q as d}from"./chunk-JE2X3YFA.js";import{Rd as u,sa as l,va as m,ya as t}from"./chunk-4HFZZ7HS.js";import{e as n,h as s}from"./chunk-SZAW6ZBW.js";var a,c,O,v=n(()=>{u();T();d();a=s(I()),c=s(y());h();O=new m("Toast UI Editor Options",{providedIn:"root",factory(){let{culture:e}=t(f).currentUserSnapshot,{isDarkTheme:g}=t(p);return{initialEditType:"wysiwyg",plugins:[[c.default,{highlighter:a.default}]],usageStatistics:!1,height:"100%",autofocus:!0,theme:g?"dark":"light",previewStyle:"vertical",language:i.getBCP47Value(i.fromValue(e),"en-US")}}})});var P,S=n(()=>{u();P=(()=>{class e{dynamicallyImportToastUILanguage(o){let r=o.toLowerCase();return r==="ar"?import("./chunk-ARCGYF22.js"):r==="cs-cz"?import("./chunk-CQN3ZSUV.js"):r==="de-de"?import("./chunk-ZARTEKAT.js"):r==="es-es"?import("./chunk-A2W4UOAA.js"):r==="fi-fi"?import("./chunk-7ZQD5W5K.js"):r==="fr-fr"?import("./chunk-WWSRRJBE.js"):r==="gl-es"?import("./chunk-2MPXWIWX.js"):r==="hr-hr"?import("./chunk-CRKWC6MV.js"):r==="it-it"?import("./chunk-3TDCVMUA.js"):r==="ja-jp"?import("./chunk-UDNVEDJ7.js"):r==="ko-kr"?import("./chunk-F7C36GZ2.js"):r==="nb-no"?import("./chunk-A422VI5U.js"):r==="nl-nl"?import("./chunk-OUYTEQ35.js"):r==="pl-pl"?import("./chunk-MQZPN5AB.js"):r==="pt-br"?import("./chunk-P477NV2B.js"):r==="ru-ru"?import("./chunk-LBKHJMCA.js"):r==="sv-se"?import("./chunk-WL2BS2KA.js"):r==="tr-tr"?import("./chunk-QOCSYHSP.js"):r==="uk-ua"?import("./chunk-GKF2D46L.js"):r==="zh-cn"?import("./chunk-4VWG4OJK.js"):r==="zh-tw"?import("./chunk-RMXLDZ4X.js"):Promise.resolve(null)}static{this.\u0275fac=function(r){return new(r||e)}}static{this.\u0275prov=l({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});export{O as a,v as b,P as c,S as d};
