import{E as m,l as n,m as s}from"./chunk-TCUK52QT.js";import{Fb as i,Rd as d,Sb as e}from"./chunk-4HFZZ7HS.js";import{e as c}from"./chunk-SZAW6ZBW.js";var h,l=c(()=>{d();m();h=(()=>{class o{constructor(t,r){this.control=t,this.modelGroup=r}ngOnInit(){(this.modelGroup?.control??this.sdAddToForm?.control)?.addControl(this.control.name,this.control.control)}ngOnDestroy(){(this.modelGroup?.control??this.sdAddToForm?.control)?.removeControl(this.control.name)}static{this.\u0275fac=function(r){return new(r||o)(i(s),i(n,8))}}static{this.\u0275dir=e({type:o,selectors:[["","sdAddToForm",""]],inputs:{sdAddToForm:"sdAddToForm"}})}}return o})()});export{h as a,l as b};
