import{a as U,b as Q}from"./chunk-MBRDXADW.js";import{a as K,b as j,c as G}from"./chunk-UGQXGBKF.js";import{C as L,E as N,a as P,b as q,f as O,m as D,w as F}from"./chunk-TCUK52QT.js";import{K as J,t as E}from"./chunk-UOI75ONS.js";import{Ab as p,Ad as S,Bc as W,Cc as x,Gc as T,Hc as r,Ic as v,Ja as l,Jc as A,Ka as s,Qb as V,Rd as k,Ta as u,Wb as I,Xc as B,b,dd as f,ed as w,fd as y,jc as a,jd as R,kc as m,lc as g,ma as H,nc as h,ra as M,rc as C,wc as d,xc as c}from"./chunk-4HFZZ7HS.js";import{e as z}from"./chunk-SZAW6ZBW.js";function Y(i,_){if(i&1){let e=x();d(0,"input",6),y("ngModelChange",function(t){l(e);let n=r();return w(n.value,t)||(n.value=t),s(t)}),T("focus",function(t){l(e);let n=r();return s(n.onFocus(t))})("input",function(t){l(e);let n=r();return s(n.onInput(t))})("keyup",function(t){l(e);let n=r();return s(n.onKeyup(t))})("blur",function(t){l(e);let n=r();return s(n.onBlur(t))}),c()}if(i&2){let e=r();h(e.styleClass),m("border-top-right-radius",e.noBorderRight?"0":null)("border-bottom-right-radius",e.noBorderRight?"0":null),g("dwl-ignore",e.dwlIgnore),a("disabled",e.disabled)("readonly",e.readonly),f("ngModel",e.value)}}function Z(i,_){if(i&1){let e=x();d(0,"input",7),y("ngModelChange",function(t){l(e);let n=r();return w(n.value,t)||(n.value=t),s(t)}),T("focus",function(t){l(e);let n=r();return s(n.onFocus(t))})("input",function(t){l(e);let n=r();return s(n.onInput(t))})("keyup",function(t){l(e);let n=r();return s(n.onKeyup(t))})("blur",function(t){l(e);let n=r();return s(n.onBlur(t))}),c()}if(i&2){let e=r();h(e.styleClass),m("border-top-right-radius",e.noBorderRight?"0":null)("border-bottom-right-radius",e.noBorderRight?"0":null),g("dwl-ignore",e.dwlIgnore),a("disabled",e.disabled)("readonly",e.readonly),f("ngModel",e.value),a("required",e.required)}}function $(i,_){if(i&1){let e=x();d(0,"textarea",8),y("ngModelChange",function(t){l(e);let n=r();return w(n.value,t)||(n.value=t),s(t)}),c()}if(i&2){let e=r();h(e.styleClass),m("border-top-right-radius",e.noBorderRight?"0":null)("border-bottom-right-radius",e.noBorderRight?"0":null),g("dwl-ignore",e.dwlIgnore),a("disabled",e.disabled)("readonly",e.readonly),f("ngModel",e.value)}}function ee(i,_){i&1&&W(0)}function te(i,_){i&1&&A(0)}var X,ge,ne=z(()=>{k();N();H();J();G();Q();k();G();N();X=["*"];ge=(()=>{class i{constructor(){this.required=!1,this.dwlIgnore=!1,this.noBorderRight=!1,this.focus=new u,this.input=new u,this.keyup=new u,this.blur=new u,this.onModelChange=b,this.onModelTouched=b,this._value=""}get value(){return this._value}set value(e){e!==this._value&&(this._value=e,this.onModelChange(this._value))}registerOnChange(e){this.onModelChange=e}registerOnTouched(e){this.onModelTouched=e}setDisabledState(e){}writeValue(e){e!=null&&(this.value=e)}onFocus(e){this.focus.emit(e)}onInput(e){this.input.emit(e)}onKeyup(e){this.keyup.emit(e)}onBlur(e){this.blur.emit(e)}static{this.\u0275fac=function(o){return new(o||i)}}static{this.\u0275cmp=V({type:i,selectors:[["dvl-input-with-action"]],inputs:{copyable:"copyable",sensitive:"sensitive",type:"type",readonly:"readonly",disabled:"disabled",styleClass:"styleClass",required:"required",dwlIgnore:"dwlIgnore",noBorderRight:"noBorderRight"},outputs:{focus:"focus",input:"input",keyup:"keyup",blur:"blur"},features:[R([{provide:P,useExisting:M(()=>i),multi:!0}])],ngContentSelectors:X,decls:7,vars:4,consts:[["template",""],[1,"flex","w-full"],["autocomplete","new-password","pInputText","","type","text",1,"flex-1",3,"dwl-ignore","class","disabled","readonly","ngModel","border-top-right-radius","border-bottom-right-radius"],["autocomplete","new-password","pInputText","","type","password",1,"flex-1",3,"dwl-ignore","class","disabled","readonly","ngModel","border-top-right-radius","border-bottom-right-radius","required"],["pInputTextarea","",1,"flex-1",3,"class","dwl-ignore","disabled","readonly","ngModel","border-top-right-radius","border-bottom-right-radius"],[4,"ngTemplateOutlet"],["autocomplete","new-password","pInputText","","type","text",1,"flex-1",3,"ngModelChange","focus","input","keyup","blur","disabled","readonly","ngModel"],["autocomplete","new-password","pInputText","","type","password",1,"flex-1",3,"ngModelChange","focus","input","keyup","blur","disabled","readonly","ngModel","required"],["pInputTextarea","",1,"flex-1",3,"ngModelChange","disabled","readonly","ngModel"]],template:function(o,t){if(o&1&&(v(),d(0,"div",1),I(1,Y,1,11,"input",2)(2,Z,1,12,"input",3)(3,$,1,11,"textarea",4)(4,ee,1,0,"ng-container",5),c(),I(5,te,1,0,"ng-template",null,0,S)),o&2){let n=B(6);p(),C(t.type==="text"?1:-1),p(),C(t.type==="password"?2:-1),p(),C(t.type==="textarea"?3:-1),p(),a("ngTemplateOutlet",n)}},dependencies:[j,K,L,q,O,F,D,U,E],encapsulation:2})}}return i})()});export{ge as a,ne as b};
