import{c as l,d as f,e as c,f as v}from"./chunk-YQFKMBOB.js";import{Fb as o,Qb as t,Rd as h,Wb as a,rc as r,wc as s,xc as d,yc as g}from"./chunk-4HFZZ7HS.js";import{e as p}from"./chunk-SZAW6ZBW.js";function u(e,L){e&1&&(s(0,"div",0),g(1,"div",1),d())}var x,b=p(()=>{f();h();v();x=(()=>{class e{constructor(i){this.loadingService=i,this.isLoading=!1}ngOnInit(){this.loadingService.addReceiver(this),this.loadingService.isWorking().subscribe(i=>{(!i.isLoading||i.mode===l.Local&&i.receiver===this)&&(this.isLoading=i.isLoading)})}ngAfterContentChecked(){this.show=this.isLoading}ngOnDestroy(){this.loadingService.removeReceiver(this)}static{this.\u0275fac=function(n){return new(n||e)(o(c))}}static{this.\u0275cmp=t({type:e,selectors:[["sd-loading-bar"]],decls:1,vars:1,consts:[[1,"loading-bar"],[1,"loading-bar-progress"]],template:function(n,m){n&1&&a(0,u,2,0,"div",0),n&2&&r(m.show?0:-1)},styles:[`@keyframes slide{0%{left:-33%}to{left:100%}}.loading-bar{position:absolute;margin:0;padding:0;top:0;left:0;right:0;z-index:99999;background-color:orange;opacity:0;overflow:hidden;transition:opacity 1s ease}.loading-bar-progress{width:33%;margin:0;padding:0;z-index:99998;position:relative;background-color:orange;height:4px;animation-name:slide;animation-duration:2s;animation-timing-function:linear;animation-iteration-count:infinite}.navigation-tree-holder .loading-bar{position:absolute!important}.navigation-tree-holder .loading-bar-progress{height:2px}
`],encapsulation:2})}}return e})()});export{x as a,b};
