import{Rd as c,Tb as p}from"./chunk-4HFZZ7HS.js";import{e as s}from"./chunk-SZAW6ZBW.js";var m,f=s(()=>{c();m=(()=>{class e{transform(t,r,n){let o=i=>i.toLocaleLowerCase().normalize("NFD").replace(/[\u0300-\u036f]/g,""),a=n?o(n):null;return a?t.filter(i=>o(i[r]).indexOf(a)!==-1):t}static{this.\u0275fac=function(r){return new(r||e)}}static{this.\u0275pipe=p({name:"entryFilter",type:e,pure:!0})}}return e})()});export{m as a,f as b};
