import{a as e,b as n}from"./chunk-DY655T2D.js";import{Rd as c,sa as r}from"./chunk-4HFZZ7HS.js";import{e as i}from"./chunk-SZAW6ZBW.js";var m,a=i(()=>{n();c();m=(()=>{class t extends e{constructor(){super()}static{this.\u0275fac=function(o){return new(o||t)}}static{this.\u0275prov=r({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});export{m as a,a as b};
