import{Rd as e,g as i,ma as c,sa as n}from"./chunk-4HFZZ7HS.js";import{e as o}from"./chunk-SZAW6ZBW.js";var d,s,u=o(()=>{c();e();d=(()=>{class t{constructor(){this.viewRequest=new i}view(r){this.viewRequest.next(r)}static{this.\u0275fac=function(a){return new(a||t)}}static{this.\u0275prov=n({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})(),s=function(t){return t[t.VaultConnection=0]="VaultConnection",t[t.PamCredential=1]="PamCredential",t[t.OtpTemplate=2]="OtpTemplate",t[t.PasswordList=3]="PasswordList",t}(s||{})});export{d as a,s as b,u as c};
