import{a as y,b as M,c as O}from"./chunk-UAPDCKDX.js";import{a as P,b as I}from"./chunk-UIVS6623.js";import{K as A,m as x,s as h}from"./chunk-UOI75ONS.js";import{Ab as i,Hc as c,Lc as _,Na as v,Qb as p,Rd as w,Ub as b,Wb as u,jc as d,kc as C,ld as f,oc as z,rc as n,vb as m,wc as l,xc as g,yc as o}from"./chunk-4HFZZ7HS.js";import{e as S}from"./chunk-SZAW6ZBW.js";function V(a,s){if(a&1&&o(0,"ngx-avatars",1),a&2){let e=c();d("name",e.name)("size",e.avatarSize)("textSizeRatio",2.25)("initialsSize",3)}}function k(a,s){a&1&&o(0,"i",2)}function T(a,s){if(a&1&&(l(0,"span",3),o(1,"img",6),g()),a&2){let e=c();i(),_("src","data:image/png;base64,",e.imageBytes,"",m)}}function F(a,s){if(a&1&&o(0,"i",7),a&2){let e=c();z("repo-icon di dvl-icon-",e.imageIcon.icon,""),d("ngStyle",f(4,B,e.imageIcon.color))}}function D(a,s){if(a&1&&(l(0,"div",5),o(1,"i",8)(2,"i",9),g()),a&2){let e=c();d("ngClass",e.size)}}var B,$,E=S(()=>{I();O();A();w();O();B=a=>({color:a});$=(()=>{class a extends P{constructor(){super(...arguments),this.isPam=!1,this.isPrivate=!1,this.avatarSize=28}get cssAvatarSize(){return`${this.avatarSize}px`}static{this.\u0275fac=(()=>{let e;return function(t){return(e||(e=v(a)))(t||a)}})()}static{this.\u0275cmp=p({type:a,selectors:[["dvls-vault-avatar"]],hostVars:2,hostBindings:function(r,t){r&2&&C("--avatar-size",t.cssAvatarSize)},inputs:{imageIcon:"imageIcon",imageBytes:"imageBytes",isPam:"isPam",isPrivate:"isPrivate",name:"name",avatarSize:"avatarSize",size:"size"},features:[b],decls:6,vars:5,consts:[[1,"avatar-container"],[3,"name","size","textSizeRatio","initialsSize"],[1,"repo-private-icon","di","dvl-icon-user-vault"],[1,"repo-custom-image-container"],[3,"class","ngStyle"],[1,"avatar-badge",3,"ngClass"],["alt","",1,"repo-custom-image",3,"src"],[3,"ngStyle"],[1,"avatar-badge-icon","avatar-badge-border","di","dvl-icon-pam"],[1,"avatar-badge-icon","di","dvl-icon-pam"]],template:function(r,t){r&1&&(l(0,"div",0),u(1,V,1,4,"ngx-avatars",1)(2,k,1,0,"i",2)(3,T,2,2,"span",3)(4,F,1,6,"i",4)(5,D,3,1,"div",5),g()),r&2&&(i(),n(!t.isPrivate&&!t.imageBytes&&!t.imageIcon?1:-1),i(),n(t.isPrivate?2:-1),i(),n(t.imageBytes?3:-1),i(),n(t.imageIcon?4:-1),i(),n(t.isPam?5:-1))},dependencies:[M,y,h,x],styles:['[_nghost-%COMP%]{--badge-icon-size: 16px;--badge-border-width: 4px}.tiny[_ngcontent-%COMP%]{--badge-icon-size: 10px;--badge-border-width: 3px}.small[_ngcontent-%COMP%]{--badge-icon-size: 15px}.medium[_ngcontent-%COMP%]{--badge-icon-size: 16px}.repo-custom-image-container[_ngcontent-%COMP%]{display:flex;align-items:center;justify-content:center;width:var(--avatar-size, 28px);height:var(--avatar-size, 28px)}.repo-custom-image[_ngcontent-%COMP%]{max-width:var(--avatar-size, 28px);max-height:var(--avatar-size, 28px)}.repo-icon[_ngcontent-%COMP%]{font-size:var(--avatar-size, 28px)}.avatar-container[_ngcontent-%COMP%]{position:relative;display:inline-block}.avatar-badge-icon[_ngcontent-%COMP%]{font-size:var(--badge-icon-size)}.avatar-badge[_ngcontent-%COMP%]{position:absolute;bottom:calc(var(--badge-border-width) * -1);right:calc(var(--badge-border-width) * -1);color:rgb(var(--base-color-rgb))}.avatar-badge[_ngcontent-%COMP%], .avatar-badge-icon[_ngcontent-%COMP%]{width:var(--badge-icon-size);height:var(--badge-icon-size)}.avatar-badge-icon[_ngcontent-%COMP%]:not(.avatar-badge-border){width:calc(var(--badge-icon-size) - var(--badge-border-width));height:calc(var(--badge-icon-size) - var(--badge-border-width));font-size:calc(var(--badge-icon-size) - var(--badge-border-width))}.avatar-badge-icon[_ngcontent-%COMP%]{transform:translate(-50%,-50%);font-size:var(--badge-icon-size);position:absolute;top:50%;left:50%}.avatar-badge-border[_ngcontent-%COMP%]{color:var(--side-panel-background-color)}.avatar-badge-border.dvl-icon-role-owner[_ngcontent-%COMP%], .avatar-badge-border.dvl-icon-role-admin[_ngcontent-%COMP%]{text-shadow:0 -2px 0 var(--side-panel-background-color),0 -1px var(--side-panel-background-color)}.avatar-badge-border.dvl-icon-pam[_ngcontent-%COMP%]:after{width:calc(var(--badge-icon-size) - 5px);height:calc(var(--badge-icon-size) - 5px);position:absolute;inset:0;margin:auto;background-color:var(--side-panel-background-color);border-radius:50%;content:""}']})}}return a})()});export{$ as a,E as b};
