import{a as v,b as N}from"./chunk-BPRWOLIG.js";import"./chunk-G3XZJQZX.js";import"./chunk-PQQRTQYF.js";import{a as k,b as M}from"./chunk-4UZNKZHD.js";import"./chunk-CD2FEHIR.js";import"./chunk-A7OV5QSE.js";import"./chunk-CY3Q2PRR.js";import"./chunk-5ZECTPCZ.js";import"./chunk-U3VV32SL.js";import"./chunk-5VIETITT.js";import"./chunk-N342MCNF.js";import"./chunk-T7OUJAKP.js";import"./chunk-2GVLCGGD.js";import"./chunk-2JML2BCE.js";import"./chunk-KA2SPUWT.js";import"./chunk-KLB4BWDP.js";import"./chunk-YINSOKTV.js";import"./chunk-RLDUKVM7.js";import{a as f,b as w}from"./chunk-I7PQ4XZJ.js";import"./chunk-JPQXNP44.js";import"./chunk-QCYFTXYO.js";import"./chunk-EBWMNYGM.js";import"./chunk-VKHMVPMU.js";import"./chunk-YQFKMBOB.js";import"./chunk-MRFU7ZH5.js";import"./chunk-S6WI4OAL.js";import"./chunk-FESI7YJY.js";import"./chunk-5JRUNHF6.js";import"./chunk-4WJQGABW.js";import"./chunk-MAQNKK2A.js";import"./chunk-XMICNJPL.js";import"./chunk-SHCLTTYO.js";import"./chunk-BXBF54ET.js";import"./chunk-MAVIT7QG.js";import"./chunk-6Q7YU3GA.js";import"./chunk-3RHWAFOM.js";import"./chunk-3OCEUHZW.js";import"./chunk-QUK73RJT.js";import"./chunk-WSU244SW.js";import"./chunk-DZTIA5CA.js";import"./chunk-R2ULAEP2.js";import"./chunk-ZSHAELBT.js";import"./chunk-2FOKRXOS.js";import"./chunk-UGQXGBKF.js";import"./chunk-TCUK52QT.js";import"./chunk-JU5LF7AN.js";import"./chunk-AFLM5QFV.js";import"./chunk-DCQJQBJ7.js";import"./chunk-VYRWAFSM.js";import"./chunk-UTF7W4HP.js";import"./chunk-OJRDBZWF.js";import"./chunk-LFKM2JDJ.js";import"./chunk-6G52AAKU.js";import"./chunk-QQZCALHS.js";import"./chunk-EXP5MDOK.js";import"./chunk-M55KBHC5.js";import{J as g,K as S}from"./chunk-JE2X3YFA.js";import{o as u,v as H}from"./chunk-HW6LKGMV.js";import"./chunk-XRGTMJIH.js";import"./chunk-THDAWOZM.js";import"./chunk-GDJMZZ4C.js";import"./chunk-V3J5YJ3L.js";import{a as b,b as _}from"./chunk-UIVS6623.js";import"./chunk-2XIOYYFS.js";import"./chunk-HYPRVELK.js";import"./chunk-YTY7HHO3.js";import"./chunk-UOI75ONS.js";import{Fb as a,Gc as l,J as r,Qb as d,Rd as y,Ub as p,ia as t,jc as c,na as D,wc as m,x as s,xc as h}from"./chunk-4HFZZ7HS.js";import{e as C}from"./chunk-SZAW6ZBW.js";var K,T=C(()=>{D();M();_();N();y();H();S();w();K=(()=>{class i extends b{constructor(e,n,o){super(),this.route=e,this.displayModeService=n,this.connectionHandbookNavigationService=o,this.permissions=new k,this.displayModeService.isKioskMode=!0}ngOnInit(){this.route.params.pipe(t(this.destroyed$),r(e=>!!e.id)).subscribe(e=>this.handbookId=e.id),this.route.queryParams.pipe(t(this.destroyed$)).subscribe(e=>this.pageId=e.page||this.handbookId),this.route.data.pipe(t(this.destroyed$),s(e=>e.permissions)).subscribe(e=>this.permissions=e)}onPageChange(e){this.connectionHandbookNavigationService.navigateToDocumentation(this.handbookId,e,!1)}static{this.\u0275fac=function(n){return new(n||i)(a(u),a(g),a(f))}}static{this.\u0275cmp=d({type:i,selectors:[["ng-component"]],features:[p],decls:1,vars:5,consts:[[3,"pageChange","handbookId","pageId","canEditHandbook","canDeleteHandbook","allowOpenInNewTab"]],template:function(n,o){n&1&&(m(0,"dvls-doc-handbook-dashboard",0),l("pageChange",function(I){return o.onPageChange(I)}),h()),n&2&&c("handbookId",o.handbookId)("pageId",o.pageId)("canEditHandbook",o.permissions==null?null:o.permissions.canEditHandbook)("canDeleteHandbook",o.permissions==null?null:o.permissions.canDeleteHandbook)("allowOpenInNewTab",!1)},dependencies:[v],encapsulation:2})}}return i})()});T();export{K as ConnectionDocumentationComponent};
