import{k as B,l as E}from"./chunk-OOETPT6P.js";import{a as O,b as x,f as M}from"./chunk-LG3SCC2E.js";import{a as V,b as I}from"./chunk-JU5LF7AN.js";import{a as D,b as k}from"./chunk-LFKM2JDJ.js";import{$c as g,Ab as n,Cc as y,Gc as d,Ib as C,Ja as c,Ka as m,Qb as w,Rd as b,Vc as h,Wc as S,Zc as f,jc as p,ud as i,vd as o,wc as t,xc as r,ya as s,yc as _}from"./chunk-4HFZZ7HS.js";import{e as N}from"./chunk-SZAW6ZBW.js";var P,z,T=N(()=>{b();M();k();E();I();b();P=["footer"],z=(()=>{class l{constructor(){this.ref=s(x,{optional:!0}),this.config=s(O,{optional:!0}),this.licensesService=s(B),this.footerInstance=C.required("footer")}onContactSales(){this.licensesService.openContactSalesWindow(),this.ref?.close(null)}onClose(){this.ref?.close(null)}static{this.\u0275fac=function(e){return new(e||l)}}static{this.\u0275cmp=w({type:l,selectors:[["dvls-now-on-free-version-dialog"]],viewQuery:function(e,a){e&1&&h(a.footerInstance,P,5),e&2&&S()},decls:16,vars:15,consts:[["footer",""],[1,"now-on-free-version-dialog-content"],["src","./assets/images/maurice-transparent.png",1,"h-[150px]","mx-auto",3,"alt"],[1,"now-on-free-version-message"],[1,"p-dialog-footer"],[2,"margin-right","auto"],["variant","secondary",3,"onClick","label"]],template:function(e,a){if(e&1){let u=y();t(0,"div",1),_(1,"img",2),i(2,"translate"),t(3,"p",3),f(4),i(5,"translate"),r(),t(6,"p",3),f(7),i(8,"translate"),r()(),t(9,"div",4,0)(11,"div",5)(12,"dvl-button",6),i(13,"translate"),d("onClick",function(){return c(u),m(a.onContactSales())}),r()(),t(14,"dvl-button",6),i(15,"translate"),d("onClick",function(){return c(u),m(a.onClose())}),r()()}e&2&&(n(),p("alt",o(2,5,"lblIllustration")),n(3),g(" ",o(5,7,"lblNowOnFreeVersionBody1")," "),n(3),g(" ",o(8,9,"lblNowOnFreeVersionBody2")," "),n(5),p("label",o(13,11,"lblContactSales")),n(2),p("label",o(15,13,"lblClose")))},dependencies:[D,V],styles:[".now-on-free-version-dialog-content[_ngcontent-%COMP%]{padding:1rem 0;text-align:center}.now-on-free-version-dialog-content[_ngcontent-%COMP%]   .now-on-free-version-message[_ngcontent-%COMP%]{margin:0 0 16px;line-height:1.5}"]})}}return l})()});export{z as a,T as b};
