import{c as t,d as n}from"./chunk-V3J5YJ3L.js";import{e as s}from"./chunk-SZAW6ZBW.js";var o,r=s(()=>{n();o=class i{toQueryString(){let e=[];return this.activeTime!==null&&this.activeTime.toString()&&e.push("activeTime="+encodeURIComponent(this.activeTime.toString())),this.closeMode!==null&&this.closeMode.toString()&&e.push("closeMode="+encodeURIComponent(this.closeMode.toString())),this.closePrompt!==null&&this.closePrompt.toString()&&e.push("closePrompt="+encodeURIComponent(this.closePrompt.toString())),this.dateTime!==null&&this.dateTime.toString()&&e.push("dateTime="+encodeURIComponent(this.dateTime.toString())),this.hasSessionRecording!==null&&this.hasSessionRecording.toString()&&e.push("hasSessionRecording="+encodeURIComponent(this.hasSessionRecording.toString())),this.processExitCode!==null&&this.processExitCode.toString()&&e.push("processExitCode="+encodeURIComponent(this.processExitCode.toString())),this.resultID!==null&&this.resultID.toString()&&e.push("resultID="+encodeURIComponent(this.resultID.toString())),this.ticketNumber!==null&&this.ticketNumber.toString()&&e.push("ticketNumber="+encodeURIComponent(this.ticketNumber.toString())),this.userName!==null&&this.userName.toString()&&e.push("userName="+encodeURIComponent(this.userName.toString())),e.join("&")}constructor(e=null){this.clone=()=>new i(this),this.activeTime=t.manageDefaultValue(e,"activeTime",null),this.closeMode=t.manageDefaultValue(e,"closeMode",null),this.closePrompt=t.manageDefaultValue(e,"closePrompt",""),this.dateTime=t.manageDefaultValue(e,"dateTime",void 0),this.hasSessionRecording=t.manageDefaultValue(e,"hasSessionRecording",null),this.processExitCode=t.manageDefaultValue(e,"processExitCode",null),this.resultID=t.manageDefaultValue(e,"resultID",""),this.ticketNumber=t.manageDefaultValue(e,"ticketNumber",""),this.userName=t.manageDefaultValue(e,"userName","")}}});function h(i){return i!=null&&typeof i.sessionId=="string"&&typeof i.recordingGatewayId=="string"}var l=s(()=>{});export{o as a,r as b,h as c,l as d};
