import{Rd as g,sa as f}from"./chunk-4HFZZ7HS.js";import{e as i}from"./chunk-SZAW6ZBW.js";var s,r=i(()=>{s=function(t){return t[t.Unknown=0]="Unknown",t[t.Email=1]="Email",t[t.SecureMessage=2]="SecureMessage",t[t.Sms=3]="Sms",t[t.Syslog=4]="Syslog",t[t.User=5]="User",t[t.System=6]="System",t}(s||{});(function(t){function e(){return"SubscriberType"}t.getClass=e;function o(u){let h="";return typeof t.getClass=="function"&&(h=t.getClass()),h+"_"+t[u]}t.getEnumKey=o})(s||(s={}))});var n,l=i(()=>{n=class{constructor(e=null){e?(this.id=e.id,this.name=e.name,this.type=e.type,this.subscriptions=e.subscriptions||[],this.groups=e.groups||[]):(this.subscriptions=[],this.groups=[])}}});var m,p=i(()=>{l();r();m=class extends n{constructor(e=null){super(e),this.type=s.Syslog,e?this.syslogContent=e.syslogContent||{}:this.syslogContent={}}get hostname(){return this.syslogContent.host}set hostname(e){this.syslogContent.host=e}get port(){return this.syslogContent.port}set port(e){this.syslogContent.port=Number(e)}get protocol(){return this.syslogContent.protocol}set protocol(e){this.syslogContent.protocol=e}get useDefaultSettings(){return this.syslogContent.useDefaultSettings}set useDefaultSettings(e){this.syslogContent.useDefaultSettings=e}}});var a,c=i(()=>{l();r();a=class extends n{constructor(e=null){super(e),this.type=s.Email,e?this.emailContent=e.emailContent||{}:this.emailContent={}}get emailAddress(){return this.emailContent.email}set emailAddress(e){this.emailContent.email=e}get recipient(){return this.emailContent.recipientName}set recipient(e){this.emailContent.recipientName=e}}});var A,C=i(()=>{l();c();p();r();g();A=(()=>{class t{toSubscriber(o){if(!o)return null;switch(o.type){case s.Email:return new a(o);case s.Syslog:return new m(o);default:return new n(o)}}toSubscribers(o){return(o||[]).map(this.toSubscriber)}static{this.\u0275fac=function(u){return new(u||t)}}static{this.\u0275prov=f({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});export{s as a,r as b,m as c,p as d,A as e,C as f};
