import{a as S,b as P}from"./chunk-LFKM2JDJ.js";import{D as b,K as _,m as I}from"./chunk-UOI75ONS.js";import{$c as p,Ab as e,Hc as m,Qb as x,Rd as y,Wb as C,Zc as o,_c as d,jc as g,rc as c,ud as l,vd as r,wc as t,xc as i,yc as u}from"./chunk-4HFZZ7HS.js";import{e as w}from"./chunk-SZAW6ZBW.js";function E(n,f){if(n&1&&(t(0,"div",1)(1,"span",2),o(2),l(3,"translate"),i(),t(4,"span",5),o(5),i()()),n&2){let s=m();e(2),p("",r(3,2,"lblGatewayName"),":"),e(3),d(s.sessionConfig.gatewayName)}}function h(n,f){if(n&1&&(t(0,"div",1)(1,"span",2),o(2),l(3,"translate"),i(),t(4,"span",5),o(5),i()()),n&2){let s=m();e(2),p("",r(3,2,"lblGatewaySessionId"),":"),e(3),d(s.sessionConfig.sessionId)}}function M(n,f){if(n&1&&(t(0,"div",1)(1,"span",2),o(2),l(3,"translate"),i(),t(4,"span",5),o(5),i()()),n&2){let s=m();e(2),p("",r(3,2,"lblGatewayUrl"),":"),e(3),d(s.sessionConfig.gatewayUrl)}}function B(n,f){if(n&1&&(t(0,"div",1)(1,"span",2),o(2),l(3,"translate"),i(),t(4,"span",5),o(5),i()()),n&2){let s=m();e(2),p("",r(3,2,"lblUserName"),":"),e(3),d(s.sessionConfig.username)}}function T(n,f){if(n&1&&(t(0,"div",1)(1,"span",2),o(2),l(3,"translate"),i(),t(4,"span",5),o(5),i()()),n&2){let s=m();e(2),p("",r(3,2,"lblDomain"),":"),e(3),d(s.sessionConfig.domain)}}var U,O=w(()=>{_();P();y();_();U=(()=>{class n{get isCredentialInjectionActive(){return!!(this.sessionConfig?.proxyUsername&&this.sessionConfig?.proxyPassword)}get recordingStatus(){return this.sessionConfig?.recordingTokenContainer?"lblActive":"lblInactive"}static{this.\u0275fac=function(v){return new(v||n)}}static{this.\u0275cmp=x({type:n,selectors:[["web-client-session-info-panel"]],inputs:{sessionConfig:"sessionConfig"},decls:21,vars:19,consts:[[1,"session-info-panel"],[1,"session-info-row"],[1,"session-info-label"],[1,"session-info-value",3,"ngClass"],[1,"dvl-icon",3,"ngClass"],[1,"session-info-value"]],template:function(v,a){v&1&&(t(0,"div",0)(1,"div",1)(2,"span",2),o(3),l(4,"translate"),i(),t(5,"span",3),u(6,"i",4),o(7),l(8,"translate"),i()(),C(9,E,6,4,"div",1)(10,h,6,4,"div",1)(11,M,6,4,"div",1),t(12,"div",1)(13,"span",2),o(14),l(15,"translate"),i(),t(16,"span",5),o(17),l(18,"translate"),i()(),C(19,B,6,4,"div",1)(20,T,6,4,"div",1),i()),v&2&&(e(3),p("",r(4,11,"lblCredentialInjection"),":"),e(2),g("ngClass",a.isCredentialInjectionActive?"status-active":"status-inactive"),e(),g("ngClass",a.isCredentialInjectionActive?"dvl-icon-checkmark-circle":"dvl-icon-close-circle"),e(),p(" ",r(8,13,a.isCredentialInjectionActive?"lblActive":"lblInactive")," "),e(2),c(a.sessionConfig!=null&&a.sessionConfig.gatewayName?9:-1),e(),c(a.sessionConfig!=null&&a.sessionConfig.sessionId?10:-1),e(),c(a.sessionConfig!=null&&a.sessionConfig.gatewayUrl?11:-1),e(3),p("",r(15,15,"lblRecordingServer"),":"),e(3),d(r(18,17,a.recordingStatus)),e(2),c(a.sessionConfig!=null&&a.sessionConfig.username?19:-1),e(),c(a.sessionConfig!=null&&a.sessionConfig.domain?20:-1))},dependencies:[b,I,S],styles:[".session-info-panel[_ngcontent-%COMP%]{padding:1rem;min-width:300px}.session-info-row[_ngcontent-%COMP%]{display:flex;justify-content:space-between;padding:.5rem 0;border-bottom:1px solid var(--border-secondary-color);gap:1rem}.session-info-row[_ngcontent-%COMP%]:last-child{border-bottom:none}.session-info-label[_ngcontent-%COMP%]{font-weight:600;color:var(--text-primary-color);white-space:nowrap}.session-info-value[_ngcontent-%COMP%]{color:var(--text-secondary-color);text-align:right;white-space:nowrap;overflow:hidden;text-overflow:ellipsis}.session-info-value.status-active[_ngcontent-%COMP%]{color:var(--p-green-500);font-weight:600;display:flex;align-items:center;gap:.25rem}.session-info-value.status-inactive[_ngcontent-%COMP%]{color:var(--p-surface-500);display:flex;align-items:center;gap:.25rem}"]})}}return n})()});export{U as a,O as b};
