import{a as F,b as P}from"./chunk-MBRDXADW.js";import{d as I,e as k,i as T,j as B}from"./chunk-VYRWAFSM.js";import{a as C,b as L}from"./chunk-LFKM2JDJ.js";import{Ab as p,Ic as h,Jc as g,Na as n,Qb as l,Rd as o,Ub as b,Xa as u,Zc as y,_c as x,jc as m,jd as w,sa as f,ud as M,vd as D,wc as r,xc as d,ya as c,yc as v}from"./chunk-4HFZZ7HS.js";import{e as s}from"./chunk-SZAW6ZBW.js";var E,J,N,S,$,j=s(()=>{o();o();B();k();E=["*"],J=({dt:e})=>`
.p-iftalabel {
    display: block;
    position: relative;
}

.p-iftalabel label {
    position: absolute;
    pointer-events: none;
    top: ${e("iftalabel.top")};
    transition-property: all;
    transition-timing-function: ease;
    line-height: 1;
    font-size: ${e("iftalabel.font.size")};
    font-weight: ${e("iftalabel.font.weight")};
    inset-inline-start: ${e("iftalabel.position.x")};
    color: ${e("iftalabel.color")};
    transition-duration: ${e("iftalabel.transition.duration")};
}

.p-iftalabel .p-inputtext,
.p-iftalabel .p-textarea,
.p-iftalabel .p-select-label,
.p-iftalabel .p-multiselect-label-container,
.p-iftalabel .p-autocomplete-input-multiple,
.p-iftalabel .p-cascadeselect-label,
.p-iftalabel .p-treeselect-label,
.p-iftalabel .p-datepicker-input,
.p-iftalabel .p-inputmask,
.p-iftalabel .p-autocomplete .p-inputtext,
.p-iftalabel .p-inputnumber .p-inputtext,
.p-iftalabel .p-password .p-inputtext,
.p-iftalabel > .p-iconfield .p-inputtext {
    padding-top: ${e("iftalabel.input.padding.top")};
}

.p-iftalabel:has(.ng-invalid.ng-dirty) label {
    color: ${e("iftalabel.invalid.color")};
}

.p-iftalabel:has(input:focus) label,
.p-iftalabel:has(input:-webkit-autofill) label,
.p-iftalabel:has(textarea:focus) label,
.p-iftalabel:has(.p-inputwrapper-focus) label {
    color: ${e("iftalabel.focus.color")};
}

.p-iftalabel .p-inputicon {
    top: ${e("iftalabel.input.padding.top")};
    transform: translateY(25%);
    margin-top: 0;
}

/*.p-iftalabel .p-placeholder,
.p-iftalabel input::placeholder,
.p-iftalabel .p-inputtext::placeholder {
    opacity: 0;
    transition-property: all;
    transition-timing-function: ease;
}

.p-iftalabel .p-focus .p-placeholder,
.p-iftalabel input:focus::placeholder,
.p-iftalabel .p-inputtext:focus::placeholder {
    opacity: 1;
    transition-property: all;
    transition-timing-function: ease;
}*/
`,N={root:"p-iftalabel"},S=(()=>{class e extends I{name="iftalabel";theme=J;classes=N;static \u0275fac=(()=>{let t;return function(i){return(t||(t=n(e)))(i||e)}})();static \u0275prov=f({token:e,factory:e.\u0275fac})}return e})(),$=(()=>{class e extends T{_componentStyle=c(S);static \u0275fac=(()=>{let t;return function(i){return(t||(t=n(e)))(i||e)}})();static \u0275cmp=l({type:e,selectors:[["p-iftalabel"],["p-iftaLabel"],["p-ifta-label"]],hostAttrs:[1,"p-iftalabel"],features:[w([S]),b],ngContentSelectors:E,decls:1,vars:0,template:function(a,i){a&1&&(h(),g(0))},encapsulation:2,changeDetection:0})}return e})()});var ae,A=s(()=>{o();L();j();P();o();ae=(()=>{class e{constructor(){this.previewText=u("")}static{this.\u0275fac=function(a){return new(a||e)}}static{this.\u0275cmp=l({type:e,selectors:[["dvls-ssh-jit-template-preview"]],inputs:{previewText:[1,"previewText"]},decls:5,vars:4,consts:[[1,"mt-2"],["pSize","small","pTextarea","","disabled","","id","preview","rows","4",1,"resize-none","w-full",3,"value"],["for","preview",1,"!p-0"]],template:function(a,i){a&1&&(r(0,"p-iftalabel",0),v(1,"textarea",1),r(2,"label",2),y(3),M(4,"translate"),d()()),a&2&&(p(),m("value",i.previewText()),p(2),x(D(4,2,"lblPreview")))},dependencies:[C,$,F],encapsulation:2})}}return e})()});export{ae as a,A as b};
