import{a as F,b as M,c as N}from"./chunk-MRFU7ZH5.js";import{a as q,b as z}from"./chunk-JU5LF7AN.js";import{K as B,m as V,t as j}from"./chunk-UOI75ONS.js";import{Ab as r,Ad as R,Bc as f,Cc as E,Fb as x,Gc as k,Hc as p,Ic as I,Ja as T,Jc as Q,Ka as w,Qb as D,Rc as u,Rd as S,Sc as _,Ta as P,Tc as d,Ub as b,Wb as o,Xc as C,Ya as O,jc as m,jd as L,ra as h,rc as v,wc as s,xc as c}from"./chunk-4HFZZ7HS.js";import{e as y}from"./chunk-SZAW6ZBW.js";var g,A=y(()=>{g=class{}});function U(t,a){if(t&1){let e=E();s(0,"dvl-button",7),k("onClick",function(n){T(e);let l=p();return w(l.onClickToggleOverlay(n))}),c()}if(t&2){let e=p();m("ngClass",e.switchClass)("label",e.switchLabel)("icon",e.switchIcon)}}function W(t,a){t&1&&f(0)}function X(t,a){if(t&1&&(s(0,"div",6,2),o(2,W,1,0,"ng-container",8),c()),t&2){p();let e=C(6);r(2),m("ngTemplateOutlet",e)}}function Y(t,a){t&1&&f(0)}function Z(t,a){if(t&1&&(s(0,"div",6,3),o(2,Y,1,0,"ng-container",8),c()),t&2){p();let e=C(6);r(2),m("ngTemplateOutlet",e)}}function $(t,a){t&1&&Q(0)}var G,H,J,K,ve,ee=y(()=>{S();A();N();B();z();S();N();G=["inLine"],H=["inOverlay"],J=["op"],K=["*"];ve=(()=>{class t extends g{constructor(e){super(),this.el=e,this.switchLabel="Show",this.switchIcon="pi pi-bars",this.onClickToggle=new P}get isPanel(){return this._isPanel}set isPanel(e){e!=null&&(this._isPanel=e,this.op.hide())}ngAfterViewInit(){this.el.nativeElement.className="c-overlay-panel-toggle"}closePanel(){this.op.hide()}onClickToggleOverlay(e){e.preventDefault(),e.stopPropagation(),this.onClickToggle.emit(e),this.op.toggle(e)}static{this.\u0275fac=function(i){return new(i||t)(x(O))}}static{this.\u0275cmp=D({type:t,selectors:[["dvl-overlay-panel-toggle"]],viewQuery:function(i,n){if(i&1&&(u(G,5),u(H,5),u(J,7)),i&2){let l;_(l=d())&&(n.inLine=l.first),_(l=d())&&(n.inOverlay=l.first),_(l=d())&&(n.op=l.first)}},inputs:{switchLabel:"switchLabel",switchIcon:"switchIcon",switchClass:"switchClass",isPanel:"isPanel"},outputs:{onClickToggle:"onClickToggle"},features:[L([{provide:g,useExisting:h(()=>t)}]),b],ngContentSelectors:K,decls:7,vars:3,consts:[["op",""],["tp",""],["inOverlay",""],["inLine",""],[1,"c-overlay-panel-toggle__switch-button",3,"ngClass","label","icon"],["appendTo","body","styleClass","c-overlay-panel-toggle__panel"],[1,"c-overlay-panel-toggle__content-wrapper"],[1,"c-overlay-panel-toggle__switch-button",3,"onClick","ngClass","label","icon"],[4,"ngTemplateOutlet"]],template:function(i,n){i&1&&(I(),o(0,U,1,3,"dvl-button",4),s(1,"p-overlayPanel",5,0),o(3,X,3,1,"div",6),c(),o(4,Z,3,1,"div",6)(5,$,1,0,"ng-template",null,1,R)),i&2&&(v(n.isPanel?0:-1),r(3),v(n.isPanel?3:-1),r(),v(n.isPanel?-1:4))},dependencies:[q,M,F,V,j],encapsulation:2})}}return t})()});export{ve as a,ee as b};
