import{P as g,Q as h}from"./chunk-JE2X3YFA.js";import"./chunk-HW6LKGMV.js";import"./chunk-GDJMZZ4C.js";import"./chunk-V3J5YJ3L.js";import"./chunk-UIVS6623.js";import"./chunk-2XIOYYFS.js";import"./chunk-HYPRVELK.js";import"./chunk-YTY7HHO3.js";import"./chunk-UOI75ONS.js";import{Gc as s,Qb as a,Rd as m,Wb as l,Zc as i,rc as p,wc as o,xc as t,ya as c,yc as r}from"./chunk-4HFZZ7HS.js";import{e as f}from"./chunk-SZAW6ZBW.js";function u(e,b){e&1&&(o(0,"div",2)(1,"div",3)(2,"a",4),r(3,"i",5),t()()(),r(4,"div",6))}var P,v=f(()=>{m();h();m();P=(()=>{class e{constructor(){this.currentUserService=c(g)}get isLogged(){return this.currentUserService.isLogged()}goBack(){window.history.back()}static{this.\u0275fac=function(n){return new(n||e)}}static{this.\u0275cmp=a({type:e,selectors:[["sd-error-404"]],decls:8,vars:1,consts:[[1,"Box"],[3,"click"],["id","header"],["id","mm-devo"],["href","https://devolutions.net"],[1,"di","dvl-icon-devolutions"],[1,"dvls-homeLogo","dvl-icon"]],template:function(n,d){n&1&&(l(0,u,5,0),o(1,"div",0)(2,"h2")(3,"b"),i(4,"404"),t(),i(5," PAGE NOT FOUND"),t(),o(6,"button",1),s("click",function(){return d.goBack()}),i(7,"Back"),t()()),n&2&&p(d.isLogged?-1:0)},styles:['.Box[_ngcontent-%COMP%]{box-sizing:border-box;margin:0 auto;max-width:600px;overflow:hidden;padding:10px;text-align:center}.Box[_ngcontent-%COMP%]   h2[_ngcontent-%COMP%]{color:#0068c3;font-family:var(--font-open-sans);font-size:34px;font-weight:400;letter-spacing:-.06em;line-height:76px;margin:0;padding:0}.dvls-homeLogo[_ngcontent-%COMP%]{box-sizing:border-box;margin:0 auto;max-width:600px;overflow:hidden;padding:25px;position:relative;width:100%}#header[_ngcontent-%COMP%]{color:#fff;left:0;margin:0 auto;min-width:700px;top:0;width:100%;background:#0068c3;box-shadow:0 2px 10px -2px #66666673}i.dvl-icon-devolutions[_ngcontent-%COMP%]{font-size:40px;opacity:.95}#mm-devo[_ngcontent-%COMP%]{height:50px;margin:0 auto;max-width:1024px}#mm-devo[_ngcontent-%COMP%]   a[_ngcontent-%COMP%]{box-sizing:border-box;color:#fff;display:block;height:50px;line-height:1;padding:4px 0 0 16px;text-align:left}button[_ngcontent-%COMP%]{background:#0068c3;border:1px solid #0068c3;-ms-border-radius:3px;border-radius:3px;color:#fff;text-decoration:none;font-family:var(--font-open-sans);font-size:1.15em;padding:.2em 1.2em;cursor:pointer;position:relative;vertical-align:middle}button[_ngcontent-%COMP%]:after{box-shadow:0 8px 5px -8px #dcdcdc;content:"";display:inline-block;height:100%;left:0;overflow:hidden;position:absolute;top:1px;width:100%;z-index:0}']})}}return e})()});v();export{P as Error404Component};
