import{i as de,j as _e,k as ge}from"./chunk-DCQJQBJ7.js";import{Q as ce,R as se,d as re,e as Te,i as pe,j as ve,na as Ce}from"./chunk-VYRWAFSM.js";import{c as oe,d as S,g as Q,h as B,i as M,s as he}from"./chunk-XRGTMJIH.js";import{Ga as ae,Ha as le,Ia as V,La as ye,wa as ie,xa as be}from"./chunk-2XIOYYFS.js";import{D as te,K as ne,m as X,p as Y,s as Z,t as ee}from"./chunk-UOI75ONS.js";import{Ab as a,Ac as I,Ad as J,Bc as P,Cc as w,Gc as k,Hc as s,Ic as H,Ja as b,Jc as E,Ka as y,Kd as F,Na as D,Qb as j,Qc as u,Rb as q,Rc as N,Rd as U,Sc as _,Ta as v,Tc as g,Ub as R,Wb as c,Zc as W,_c as K,ic as f,jc as i,jd as L,ld as h,md as z,nc as C,nd as G,sa as O,ta as $,wc as d,xc as m,ya as A,yc as T,zc as x}from"./chunk-4HFZZ7HS.js";import{e as fe}from"./chunk-SZAW6ZBW.js";function Ae(e,o){if(e&1&&(d(0,"span",12),W(1),m()),e&2){let t=s(2);f("id",t.id+"_header"),a(),K(t._header)}}function je(e,o){e&1&&P(0)}function qe(e,o){}function Re(e,o){e&1&&c(0,qe,0,0,"ng-template")}function He(e,o){if(e&1&&T(0,"span"),e&2){let t=s(6);C(t.expandIcon)}}function Ne(e,o){e&1&&T(0,"MinusIcon")}function We(e,o){if(e&1&&(x(0),c(1,He,1,2,"span",16)(2,Ne,1,0,"MinusIcon",14),I()),e&2){let t=s(5);a(),i("ngIf",t.expandIcon),a(),i("ngIf",!t.expandIcon)}}function Ke(e,o){if(e&1&&T(0,"span"),e&2){let t=s(6);C(t.collapseIcon)}}function Le(e,o){e&1&&T(0,"PlusIcon")}function ze(e,o){if(e&1&&(x(0),c(1,Ke,1,2,"span",16)(2,Le,1,0,"PlusIcon",14),I()),e&2){let t=s(5);a(),i("ngIf",t.collapseIcon),a(),i("ngIf",!t.collapseIcon)}}function Ge(e,o){if(e&1&&(x(0),c(1,We,3,2,"ng-container",14)(2,ze,3,2,"ng-container",14),I()),e&2){let t=s(4);a(),i("ngIf",!t.collapsed),a(),i("ngIf",t.collapsed)}}function Je(e,o){}function Ue(e,o){e&1&&c(0,Je,0,0,"ng-template")}function Xe(e,o){if(e&1&&c(0,Ge,3,2,"ng-container",14)(1,Ue,1,0,null,15),e&2){let t=s(3);i("ngIf",!t.headerIconsTemplate&&!t._headerIconsTemplate&&!(t.toggleButtonProps!=null&&t.toggleButtonProps.icon)),a(),i("ngTemplateOutlet",t.headerIconsTemplate||t._headerIconsTemplate)("ngTemplateOutletContext",h(3,$e,t.collapsed))}}function Ye(e,o){if(e&1){let t=w();d(0,"p-button",13),k("click",function(n){b(t);let r=s(2);return y(r.onIconClick(n))})("keydown",function(n){b(t);let r=s(2);return y(r.onKeyDown(n))}),c(1,Xe,2,5,"ng-template",null,1,J),m()}if(e&2){let t=s(2);i("text",!0)("rounded",!0)("buttonProps",t.toggleButtonProps),f("id",t.id+"_header")("aria-label",t.buttonAriaLabel)("aria-controls",t.id+"_content")("aria-expanded",!t.collapsed)}}function Ze(e,o){if(e&1){let t=w();d(0,"div",8),k("click",function(n){b(t);let r=s();return y(r.onHeaderClick(n))}),c(1,Ae,2,2,"span",9),E(2,1),c(3,je,1,0,"ng-container",6),d(4,"div",10),c(5,Re,1,0,null,6)(6,Ye,3,7,"p-button",11),m()()}if(e&2){let t=s();f("id",t.id+"-titlebar"),a(),i("ngIf",t._header),a(2),i("ngTemplateOutlet",t.headerTemplate||t._headerTemplate),a(),i("ngClass",G(6,Oe,t.iconPos==="start",t.iconPos==="end",t.iconPos==="center")),a(),i("ngTemplateOutlet",t.iconTemplate||t._iconTemplate),a(),i("ngIf",t.toggleable)}}function et(e,o){e&1&&P(0)}function tt(e,o){e&1&&P(0)}function nt(e,o){if(e&1&&(d(0,"div",17),E(1,2),c(2,tt,1,0,"ng-container",6),m()),e&2){let t=s();a(2),i("ngTemplateOutlet",t.footerTemplate||t._footerTemplate)}}var xe,Ie,Pe,we,ke,Ee,Fe,Se,Qe,Be,Me,Ve,De,Oe,$e,ot,it,ue,at,Ft,lt=fe(()=>{he();ne();ne();U();U();be();ye();ve();ge();ge();Ce();Te();xe=["header"],Ie=["icons"],Pe=["content"],we=["footer"],ke=["headericons"],Ee=["contentWrapper"],Fe=["*",[["p-header"]],[["p-footer"]]],Se=["*","p-header","p-footer"],Qe=(e,o)=>({"p-panel p-component":!0,"p-panel-toggleable":e,"p-panel-expanded":o}),Be=e=>({transitionParams:e,height:"0",opacity:"0"}),Me=e=>({value:"hidden",params:e}),Ve=e=>({transitionParams:e,height:"*",opacity:"1"}),De=e=>({value:"visible",params:e}),Oe=(e,o,t)=>({"p-panel-icons-start":e,"p-panel-icons-end":o,"p-panel-icons-center":t}),$e=e=>({$implicit:e});ot=({dt:e})=>`
.p-panel {
    border: 1px solid ${e("panel.border.color")};
    border-radius: ${e("panel.border.radius")};
    background: ${e("panel.background")};
    color: ${e("panel.color")};
}

.p-panel-header {
    display: flex;
    justify-content: space-between;
    align-items: center;
    padding: ${e("panel.header.padding")};
    background: ${e("panel.header.background")};
    color: ${e("panel.header.color")};
    border-style: solid;
    border-width: ${e("panel.header.border.width")};
    border-color: ${e("panel.header.border.color")};
    border-radius: ${e("panel.header.border.radius")};
}

.p-panel-toggleable .p-panel-header {
    padding: ${e("panel.toggleable.header.padding")};
}

.p-panel-title {
    line-height: 1;
    font-weight: ${e("panel.title.font.weight")};
}

.p-panel-content {
    padding: ${e("panel.content.padding")};
}

.p-panel-footer {
    padding: ${e("panel.footer.padding")};
}

/* For PrimeNG */
.p-panel-toggleable.p-panel-expanded > .p-panel-content-container:not(.ng-animating) {
    overflow: visible
}

.p-panel-toggleable .p-panel-content-container {
    overflow: hidden;
}
`,it={root:({props:e})=>["p-panel p-component",{"p-panel-toggleable":e.toggleable}],header:"p-panel-header",title:"p-panel-title",headerActions:"p-panel-header-actions",pcToggleButton:"p-panel-toggle-button",contentContainer:"p-panel-content-container",content:"p-panel-content",footer:"p-panel-footer"},ue=(()=>{class e extends re{name="panel";theme=ot;classes=it;static \u0275fac=(()=>{let t;return function(n){return(t||(t=D(e)))(n||e)}})();static \u0275prov=O({token:e,factory:e.\u0275fac})}return e})(),at=(()=>{class e extends pe{toggleable;_header;collapsed;style;styleClass;iconPos="end";expandIcon;collapseIcon;showHeader=!0;toggler="icon";transitionOptions="400ms cubic-bezier(0.86, 0, 0.07, 1)";toggleButtonProps;collapsedChange=new v;onBeforeToggle=new v;onAfterToggle=new v;footerFacet;animating;headerTemplate;iconTemplate;contentTemplate;footerTemplate;headerIconsTemplate;_headerTemplate;_iconTemplate;_contentTemplate;_footerTemplate;_headerIconsTemplate;contentWrapperViewChild;id=ie("pn_id_");get buttonAriaLabel(){return this._header}_componentStyle=A(ue);onHeaderClick(t){this.toggler==="header"&&this.toggle(t)}onIconClick(t){this.toggler==="icon"&&this.toggle(t)}toggle(t){if(this.animating)return!1;this.animating=!0,this.onBeforeToggle.emit({originalEvent:t,collapsed:this.collapsed}),this.toggleable&&(this.collapsed?this.expand():this.collapse()),this.cd.markForCheck(),t.preventDefault()}expand(){this.collapsed=!1,this.collapsedChange.emit(this.collapsed),this.updateTabIndex()}collapse(){this.collapsed=!0,this.collapsedChange.emit(this.collapsed),this.updateTabIndex()}getBlockableElement(){return this.el.nativeElement.children[0]}updateTabIndex(){this.contentWrapperViewChild&&this.contentWrapperViewChild.nativeElement.querySelectorAll('input, button, select, a, textarea, [tabindex]:not([tabindex="-1"])').forEach(l=>{this.collapsed?l.setAttribute("tabindex","-1"):l.removeAttribute("tabindex")})}onKeyDown(t){(t.code==="Enter"||t.code==="Space")&&(this.toggle(t),t.preventDefault())}onToggleDone(t){this.animating=!1,this.onAfterToggle.emit({originalEvent:t,collapsed:this.collapsed})}templates;ngAfterContentInit(){this.templates.forEach(t=>{switch(t.getType()){case"header":this._headerTemplate=t.template;break;case"content":this._contentTemplate=t.template;break;case"footer":this._footerTemplate=t.template;break;case"icons":this._iconTemplate=t.template;break;case"headericons":this._headerIconsTemplate=t.template;break;default:this._contentTemplate=t.template;break}})}static \u0275fac=(()=>{let t;return function(n){return(t||(t=D(e)))(n||e)}})();static \u0275cmp=j({type:e,selectors:[["p-panel"]],contentQueries:function(l,n,r){if(l&1&&(u(r,ae,5),u(r,xe,4),u(r,Ie,4),u(r,Pe,4),u(r,we,4),u(r,ke,4),u(r,le,4)),l&2){let p;_(p=g())&&(n.footerFacet=p.first),_(p=g())&&(n.headerTemplate=p.first),_(p=g())&&(n.iconTemplate=p.first),_(p=g())&&(n.contentTemplate=p.first),_(p=g())&&(n.footerTemplate=p.first),_(p=g())&&(n.headerIconsTemplate=p.first),_(p=g())&&(n.templates=p)}},viewQuery:function(l,n){if(l&1&&N(Ee,5),l&2){let r;_(r=g())&&(n.contentWrapperViewChild=r.first)}},inputs:{toggleable:[2,"toggleable","toggleable",F],_header:[0,"header","_header"],collapsed:[2,"collapsed","collapsed",F],style:"style",styleClass:"styleClass",iconPos:"iconPos",expandIcon:"expandIcon",collapseIcon:"collapseIcon",showHeader:[2,"showHeader","showHeader",F],toggler:"toggler",transitionOptions:"transitionOptions",toggleButtonProps:"toggleButtonProps"},outputs:{collapsedChange:"collapsedChange",onBeforeToggle:"onBeforeToggle",onAfterToggle:"onAfterToggle"},features:[L([ue]),R],ngContentSelectors:Se,decls:8,vars:25,consts:[["contentWrapper",""],["icon",""],[3,"ngClass","ngStyle"],["class","p-panel-header",3,"click",4,"ngIf"],["role","region",1,"p-panel-content-container",3,"id"],[1,"p-panel-content"],[4,"ngTemplateOutlet"],["class","p-panel-footer",4,"ngIf"],[1,"p-panel-header",3,"click"],["class","p-panel-title",4,"ngIf"],[1,"p-panel-icons",3,"ngClass"],["severity","secondary","type","button","role","button","styleClass","p-panel-header-icon p-panel-toggler p-link",3,"text","rounded","buttonProps","click","keydown",4,"ngIf"],[1,"p-panel-title"],["severity","secondary","type","button","role","button","styleClass","p-panel-header-icon p-panel-toggler p-link",3,"click","keydown","text","rounded","buttonProps"],[4,"ngIf"],[4,"ngTemplateOutlet","ngTemplateOutletContext"],[3,"class",4,"ngIf"],[1,"p-panel-footer"]],template:function(l,n){if(l&1){let r=w();H(Fe),d(0,"div",2),c(1,Ze,7,10,"div",3),d(2,"div",4),k("@panelContent.done",function(me){return b(r),y(n.onToggleDone(me))}),d(3,"div",5,0),E(5),c(6,et,1,0,"ng-container",6),m(),c(7,nt,3,1,"div",7),m()()}l&2&&(C(n.styleClass),i("ngClass",z(14,Qe,n.toggleable,!n.collapsed&&n.toggleable))("ngStyle",n.style),f("id",n.id)("data-pc-name","panel"),a(),i("ngIf",n.showHeader),a(),i("id",n.id+"_content")("@panelContent",n.collapsed?h(19,Me,h(17,Be,n.animating?n.transitionOptions:"0ms")):h(23,De,h(21,Ve,n.animating?n.transitionOptions:"0ms"))),f("aria-labelledby",n.id+"_header")("aria-hidden",n.collapsed)("tabindex",n.collapsed?"-1":void 0),a(4),i("ngTemplateOutlet",n.contentTemplate||n._contentTemplate),a(),i("ngIf",n.footerFacet||n.footerTemplate||n._footerTemplate))},dependencies:[te,X,Y,ee,Z,se,ce,_e,de,V],encapsulation:2,data:{animation:[oe("panelContent",[B("hidden",Q({height:"0"})),B("void",Q({height:"{{height}}"}),{params:{height:"0"}}),B("visible",Q({height:"*"})),M("visible <=> hidden",[S("{{transitionParams}}")]),M("void => hidden",S("{{transitionParams}}")),M("void => visible",S("{{transitionParams}}"))])]},changeDetection:0})}return e})(),Ft=(()=>{class e{static \u0275fac=function(l){return new(l||e)};static \u0275mod=q({type:e});static \u0275inj=$({imports:[at,V,V]})}return e})()});export{at as a,Ft as b,lt as c};
