import{Rd as o,sa as r}from"./chunk-4HFZZ7HS.js";import{e as n}from"./chunk-SZAW6ZBW.js";var a,e=n(()=>{o();a=(()=>{class t{clear(){this.provider=null,this.scanConfig=null,this.accounts=null,this.folderId=null}static{this.\u0275fac=function(i){return new(i||t)}}static{this.\u0275prov=r({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});export{a,e as b};
