import{b as s,c as f}from"./chunk-YTY7HHO3.js";import{Rd as n,sa as r,xa as l}from"./chunk-4HFZZ7HS.js";import{e as o}from"./chunk-SZAW6ZBW.js";var b,u=o(()=>{n();f();b=(()=>{class e{constructor(t){this.translatorService=t,this.translate=a=>this.translatorService.translate(a)}addDefaultValue(t,a,i=!1){return[{value:t.defaultValue,label:`${this.translate(i?"lblRoot":"lblDefault")} (${this.translate(t.resolvedDefaultValueKey)})`},...a.filter(c=>c.value!==t.defaultValue)]}static{this.\u0275fac=function(a){return new(a||e)(l(s))}}static{this.\u0275prov=r({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});export{b as a,u as b};
