import{a as n,c as s}from"./chunk-VHKJVCTZ.js";import{a as t,b as r}from"./chunk-CXJOJS5Y.js";import"./chunk-ILBO5ZGN.js";import"./chunk-ZSHAELBT.js";import"./chunk-MWQIPWDG.js";import"./chunk-UTF7W4HP.js";import"./chunk-OJRDBZWF.js";import"./chunk-6G52AAKU.js";import"./chunk-QQZCALHS.js";import"./chunk-EXP5MDOK.js";import"./chunk-M55KBHC5.js";import"./chunk-JE2X3YFA.js";import"./chunk-HW6LKGMV.js";import{Yb as i,Zb as e}from"./chunk-GDJMZZ4C.js";import"./chunk-V3J5YJ3L.js";import"./chunk-UIVS6623.js";import"./chunk-2XIOYYFS.js";import"./chunk-HYPRVELK.js";import"./chunk-YTY7HHO3.js";import"./chunk-UOI75ONS.js";import"./chunk-4HFZZ7HS.js";import{e as a}from"./chunk-SZAW6ZBW.js";var h,p=a(()=>{r();s();e();h=[{path:"",canActivate:[t],loadComponent:()=>import("./chunk-CJHFQR5R.js").then(o=>o.NotificationsComponent),children:[{path:"user-subscriptions",canActivate:[t,n],data:{permissions:[i.NotificationsUserView]},loadComponent:()=>import("./chunk-52A4ZU3Q.js").then(o=>o.UserSubscriptionDashboardComponent)},{path:"subscriptions",canActivate:[t,n],data:{permissions:[i.NotificationsSubscriptionsView]},loadComponent:()=>import("./chunk-WR3NQGFE.js").then(o=>o.SubscriptionDashboardComponent)},{path:"groups",canActivate:[t,n],data:{permissions:[i.NotificationsSubscriberGroupView]},loadComponent:()=>import("./chunk-YG7PIQMK.js").then(o=>o.SubscriberGroupDashboardComponent)},{path:"subscribers",canActivate:[t,n],data:{permissions:[i.NotificationsSubscriberView]},loadComponent:()=>import("./chunk-HELVTTOF.js").then(o=>o.SubscriberDashboardComponent)},{path:"notification-preferences-dashboard",loadComponent:()=>import("./chunk-ADUPO3UA.js").then(o=>o.NotificationPreferencesDashboardComponent)},{path:"",loadComponent:()=>import("./chunk-4WRMER5M.js").then(o=>o.NotificationsDashboardComponent)}]}]});p();export{h as NOTIFICATIONS_ROUTES};
