import{a as St,b as pi}from"./chunk-TJUNXJUJ.js";import{a as yt,b as ci}from"./chunk-AMD4ZSM7.js";import{a as wt,b as si}from"./chunk-R6R2UCTL.js";import{c as ht,d as oi,e as vt,f as ai}from"./chunk-P5NHP2Y5.js";import{a as Ct,b as ri}from"./chunk-PWFFUW3X.js";import{a as mt,b as gt,c as _t}from"./chunk-RLDUKVM7.js";import{a as tt,b as Nt}from"./chunk-7LQVAFOE.js";import{a as at,b as Zt,i as st,j as ei}from"./chunk-ILBO5ZGN.js";import{a as ct,b as ti}from"./chunk-YSSBDB5I.js";import{a as ft,b as ni}from"./chunk-SHCLTTYO.js";import{e as ut,f as ii}from"./chunk-MAVIT7QG.js";import{a as pt,b as lt,c as dt}from"./chunk-UGQXGBKF.js";import{C as Ne,E as We,b as Qe,f as Re,m as Le}from"./chunk-TCUK52QT.js";import{a as et,b as Lt}from"./chunk-JU5LF7AN.js";import{d as Je,e as $t,i as A,j as jt,ua as Ce,va as zt}from"./chunk-VYRWAFSM.js";import{a as Ze,b as Rt}from"./chunk-LFKM2JDJ.js";import{D as ot,E as Jt,P as rt,Q as Xt,v as nt,w as Yt}from"./chunk-JE2X3YFA.js";import{q as je,v as Ut}from"./chunk-HW6LKGMV.js";import{c as ze,d as me,g as ge,h as _e,i as he,s as Vt}from"./chunk-XRGTMJIH.js";import{Eb as P,Fb as qt,Kf as ee,Lf as Gt,Oe as it,Pe as Ht,Qf as we,Rf as Kt,q as ve,r as Wt}from"./chunk-GDJMZZ4C.js";import{a as qe,b as Dt}from"./chunk-UIVS6623.js";import{Ha as Z,Ia as j,La as Ye,da as Ge,t as He,wa as Ke,xa as Bt}from"./chunk-2XIOYYFS.js";import{b as Xe,c as Qt}from"./chunk-YTY7HHO3.js";import{D as O,K as $e,t as fe}from"./chunk-UOI75ONS.js";import{Ab as r,Bc as le,Cc as y,Fb as x,Gc as S,H as Ie,Hc as c,Ic as U,Ja as m,Jb as ne,Jc as V,Ka as g,Kb as oe,Kc as Q,Lb as H,Na as I,Nd as b,Od as ue,Qb as T,Qc as G,Rb as Ee,Rc as Ue,Rd as pe,Sc as D,Ta as q,Tc as B,Ub as M,Uc as $,Wb as l,Wc as ce,Xa as ie,Za as Ae,Zc as C,_c as w,ad as Ve,d as Pe,dd as K,ed as Y,fd as J,ia as N,ic as k,jc as _,jd as De,lc as h,ld as X,ma as At,na as Et,nc as re,nd as de,ra as E,rc as u,sa as ke,ta as Oe,uc as ae,ud as d,vc as se,vd as f,wc as a,wd as Be,xc as s,ya as W,yc as F}from"./chunk-4HFZZ7HS.js";import{e as Me}from"./chunk-SZAW6ZBW.js";function di(t,p){t&1&&F(0,"p-stepper-separator")}function ui(t,p){if(t&1){let e=y();a(0,"button",0),S("click",function(){m(e);let i=c();return g(i.onStepClick())}),a(1,"span",1),C(2),s(),a(3,"span",2),V(4),s()(),l(5,di,1,0,"p-stepper-separator")}if(t&2){let e=c();_("tabindex",e.isStepDisabled()?-1:void 0)("disabled",e.isStepDisabled()),k("id",e.id())("role","tab")("aria-controls",e.ariaControls()),r(2),w(e.value()),r(3),u(e.isSeparatorVisible()?5:-1)}}function fi(t,p){t&1&&le(0)}function mi(t,p){t&1&&F(0,"p-stepper-separator")}function gi(t,p){if(t&1&&l(0,fi,1,0,"ng-container",3)(1,mi,1,0,"p-stepper-separator"),t&2){let e=c();_("ngTemplateOutlet",e.content||e._contentTemplate)("ngTemplateOutletContext",de(3,Tt,e.onStepClick.bind(e),e.value(),e.active())),r(),u(e.isSeparatorVisible()?1:-1)}}function Ci(t,p){t&1&&F(0,"p-stepper-separator")}function vi(t,p){t&1&&le(0)}function wi(t,p){if(t&1&&l(0,vi,1,0,"ng-container",1),t&2){let e=c();_("ngTemplateOutlet",e.contentTemplate||e._contentTemplate)("ngTemplateOutletContext",de(2,Tt,e.updateValue.bind(e),e.value(),e.active()))}}var R,xt,Tt,bt,_i,hi,yi,Si,Ft,ye,be,Se,z,te,Fe,L,Mt,xe=Me(()=>{$e();$e();pe();pe();Vt();Bt();Ye();jt();zt();$t();R=["*"],xt=["content"],Tt=(t,p,e)=>({activateCallback:t,value:p,active:e});bt=t=>({transitionParams:t}),_i=t=>({value:"visible",params:t}),hi=t=>({value:"hidden",params:t});yi=({dt:t})=>`
.p-steplist {
    position: relative;
    display: flex;
    justify-content: space-between;
    align-items: center;
    margin: 0;
    padding: 0;
    list-style-type: none;
    overflow-x: auto;
}

.p-step {
    position: relative;
    display: flex;
    flex: 1 1 auto;
    align-items: center;
    gap: ${t("stepper.step.gap")};
    padding: ${t("stepper.step.padding")};
}

.p-step:last-of-type {
    flex: initial;
}

.p-step-header {
    border: 0 none;
    display: inline-flex;
    align-items: center;
    text-decoration: none;
    cursor: pointer;
    transition: background ${t("stepper.transition.duration")}, color ${t("stepper.transition.duration")}, border-color ${t("stepper.transition.duration")}, outline-color ${t("stepper.transition.duration")}, box-shadow ${t("stepper.transition.duration")};
    border-radius: ${t("stepper.step.header.border.radius")};
    outline-color: transparent;
    background: transparent;
    padding: ${t("stepper.step.header.padding")};
    gap: ${t("stepper.step.header.gap")};
}

.p-step-header:focus-visible {
    box-shadow: ${t("stepper.step.header.focus.ring.shadow")};
    outline: ${t("stepper.step.header.focus.ring.width")} ${t("stepper.step.header.focus.ring.style")} ${t("stepper.step.header.focus.ring.color")};
    outline-offset: ${t("stepper.step.header.focus.ring.offset")};
}

.p-stepper.p-stepper-readonly .p-step {
    cursor: auto;
}

.p-step-title {
    display: block;
    white-space: nowrap;
    overflow: hidden;
    text-overflow: ellipsis;
    max-width: 100%;
    color: ${t("stepper.step.title.color")};
    font-weight: ${t("stepper.step.title.font.weight")};
    transition: background ${t("stepper.transition.duration")}, color ${t("stepper.transition.duration")}, border-color ${t("stepper.transition.duration")}, box-shadow ${t("stepper.transition.duration")}, outline-color ${t("stepper.transition.duration")};
}

.p-step-number {
    display: flex;
    align-items: center;
    justify-content: center;
    color: ${t("stepper.step.number.color")};
    border: 2px solid ${t("stepper.step.number.border.color")};
    background: ${t("stepper.step.number.background")};
    min-width: ${t("stepper.step.number.size")};
    height: ${t("stepper.step.number.size")};
    line-height: ${t("stepper.step.number.size")};
    font-size: ${t("stepper.step.number.font.size")};
    z-index: 1;
    border-radius: ${t("stepper.step.number.border.radius")};
    position: relative;
    font-weight: ${t("stepper.step.number.font.weight")};
}

.p-step-number::after {
    content: " ";
    position: absolute;
    width: 100%;
    height: 100%;
    border-radius: ${t("stepper.step.number.border.radius")};
    box-shadow: ${t("stepper.step.number.shadow")};
}

.p-step-active .p-step-header {
    cursor: default;
}

.p-step-active .p-step-number {
    background: ${t("stepper.step.number.active.background")};
    border-color: ${t("stepper.step.number.active.border.color")};
    color: ${t("stepper.step.number.active.color")};
}

.p-step-active .p-step-title {
    color: ${t("stepper.step.title.active.color")};
}

.p-step:not(.p-disabled):focus-visible {
    outline: ${t("focus.ring.width")} ${t("focus.ring.style")} ${t("focus.ring.color")};
    outline-offset: ${t("focus.ring.offset")};
}

.p-step:has(~ .p-step-active) .p-stepper-separator {
    background: ${t("stepper.separator.active.background")};
}

.p-stepper-separator {
    flex: 1 1 0;
    background: ${t("stepper.separator.background")};
    width: 100%;
    height: ${t("stepper.separator.size")};
    transition: background ${t("stepper.transition.duration")}, color ${t("stepper.transition.duration")}, border-color ${t("stepper.transition.duration")}, box-shadow ${t("stepper.transition.duration")}, outline-color ${t("stepper.transition.duration")};
}

.p-steppanels {
    padding: ${t("stepper.steppanels.padding")};
}

.p-steppanel {
    background: ${t("stepper.steppanel.background")};
    color: ${t("stepper.steppanel.color")};
}

.p-stepper:has(.p-stepitem) {
    display: flex;
    flex-direction: column;
}

.p-stepitem {
    display: flex;
    flex-direction: column;
    flex: initial;
}

.p-stepitem.p-stepitem-active {
    flex: 1 1 auto;
}

.p-stepitem .p-step {
    flex: initial;
}

.p-stepitem .p-steppanel-content {
    width: 100%;
    padding: ${t("stepper.steppanel.padding")};
    margin-inline-start: 1rem;
}

.p-stepitem .p-steppanel {
    display: flex;
    flex: 1 1 auto;
}

.p-stepitem .p-stepper-separator {
    flex: 0 0 auto;
    width: ${t("stepper.separator.size")};
    height: auto;
    margin: ${t("stepper.separator.margin")};
    position: relative;
    left: calc(-1 * ${t("stepper.separator.size")});
}

.p-stepitem .p-stepper-separator:dir(rtl) {
    left: calc(-9 * ${t("stepper.separator.size")});
}

.p-stepitem:has(~ .p-stepitem-active) .p-stepper-separator {
    background: ${t("stepper.separator.active.background")};
}

.p-stepitem:last-of-type .p-steppanel {
    padding-inline-start: ${t("stepper.step.number.size")};
}
/* For PrimeNG */
.p-steppanel {
    overflow: hidden;
}

.p-stepppanel:not(.ng-animating) {
    overflow: inherit;
}
`,Si={root:({props:t})=>["p-stepper p-component",{"p-readonly":t.linear}],separator:"p-stepper-separator"},Ft=(()=>{class t extends Je{name="stepper";theme=yi;classes=Si;static \u0275fac=(()=>{let e;return function(i){return(e||(e=I(t)))(i||t)}})();static \u0275prov=ke({token:t,factory:t.\u0275fac})}return t})(),ye=(()=>{class t extends A{steps=oe(E(()=>z));static \u0275fac=(()=>{let e;return function(i){return(e||(e=I(t)))(i||t)}})();static \u0275cmp=T({type:t,selectors:[["p-step-list"]],contentQueries:function(n,i,o){n&1&&$(o,i.steps,z,4),n&2&&ce()},hostVars:4,hostBindings:function(n,i){n&2&&h("p-steplist",!0)("p-component",!0)},features:[M],ngContentSelectors:R,decls:1,vars:0,template:function(n,i){n&1&&(U(),V(0))},dependencies:[O],encapsulation:2,changeDetection:0})}return t})(),be=(()=>{class t extends A{static \u0275fac=(()=>{let e;return function(i){return(e||(e=I(t)))(i||t)}})();static \u0275cmp=T({type:t,selectors:[["p-stepper-separator"]],hostVars:4,hostBindings:function(n,i){n&2&&h("p-stepper-separator",!0)("p-component",!0)},features:[M],ngContentSelectors:R,decls:1,vars:0,template:function(n,i){n&1&&(U(),V(0))},dependencies:[O],encapsulation:2,changeDetection:0})}return t})(),Se=(()=>{class t extends A{pcStepper=W(E(()=>L));value=H();isActive=b(()=>this.pcStepper.value()===this.value());step=ne(E(()=>z));stepPanel=ne(E(()=>te));constructor(){super(),ue(()=>{this.step().value.set(this.value())}),ue(()=>{this.stepPanel().value.set(this.value())})}static \u0275fac=function(n){return new(n||t)};static \u0275cmp=T({type:t,selectors:[["p-step-item"]],contentQueries:function(n,i,o){n&1&&($(o,i.step,z,5),$(o,i.stepPanel,te,5)),n&2&&ce(2)},hostVars:5,hostBindings:function(n,i){n&2&&(k("data-p-active",i.isActive()),h("p-stepitem",!0)("p-component",!0))},inputs:{value:[1,"value"]},outputs:{value:"valueChange"},features:[M],ngContentSelectors:R,decls:1,vars:0,template:function(n,i){n&1&&(U(),V(0))},dependencies:[O],encapsulation:2,changeDetection:0})}return t})(),z=(()=>{class t extends A{pcStepper=W(E(()=>L));value=H();disabled=ie(!1,{transform:e=>Ce(e)});active=b(()=>this.pcStepper.isStepActive(this.value()));isStepDisabled=b(()=>!this.active()&&(this.pcStepper.linear()||this.disabled()));id=b(()=>`${this.pcStepper.id()}_step_${this.value()}`);ariaControls=b(()=>`${this.pcStepper.id()}_steppanel_${this.value()}`);isSeparatorVisible=b(()=>{if(this.pcStepper.stepList()){let e=this.pcStepper.stepList().steps(),n=e.indexOf(this),i=e.length;return n!==i-1}else return!1});content;templates;_contentTemplate;ngAfterContentInit(){this.templates?.forEach(e=>{switch(e.getType()){case"content":this._contentTemplate=e.template;break}})}onStepClick(){this.pcStepper.updateValue(this.value())}static \u0275fac=(()=>{let e;return function(i){return(e||(e=I(t)))(i||t)}})();static \u0275cmp=T({type:t,selectors:[["p-step"]],contentQueries:function(n,i,o){if(n&1&&(G(o,xt,4),G(o,Z,4)),n&2){let v;D(v=B())&&(i.content=v.first),D(v=B())&&(i.templates=v)}},hostVars:13,hostBindings:function(n,i){n&2&&(k("aria-current",i.active()?"step":void 0)("role","presentation")("data-p-active",i.active())("data-p-disabled",i.isStepDisabled())("data-pc-name","step"),h("p-step",!0)("p-step-active",i.active())("p-disabled",i.isStepDisabled())("p-component",!0))},inputs:{value:[1,"value"],disabled:[1,"disabled"]},outputs:{value:"valueChange"},features:[M],ngContentSelectors:R,decls:2,vars:1,consts:[["type","button",1,"p-step-header",3,"click","tabindex","disabled"],[1,"p-step-number"],[1,"p-step-title"],[4,"ngTemplateOutlet","ngTemplateOutletContext"]],template:function(n,i){n&1&&(U(),l(0,ui,6,7)(1,gi,2,7)),n&2&&u(!i.content&&!i._contentTemplate?0:1)},dependencies:[O,fe,be,j],encapsulation:2,changeDetection:0})}return t})(),te=(()=>{class t extends A{pcStepper=W(E(()=>L));transitionOptions=b(()=>this.pcStepper.transitionOptions());value=H(void 0);active=b(()=>this.pcStepper.value()===this.value());ariaControls=b(()=>`${this.pcStepper.id()}_step_${this.value()}`);id=b(()=>`${this.pcStepper.id()}_steppanel_${this.value()}`);isVertical=b(()=>this.pcStepper.stepItems().length>0);isSeparatorVisible=b(()=>{if(this.pcStepper.stepItems()){let e=this.pcStepper.stepItems().length,n=He(this.pcStepper.el.nativeElement,'[data-pc-name="steppanel"]');return Ge(this.el.nativeElement,n)!==e-1}});contentTemplate;templates;_contentTemplate;ngAfterContentInit(){this.templates?.forEach(e=>{switch(e.getType()){case"content":this._contentTemplate=e.template;break}})}updateValue(e){this.pcStepper.updateValue(e)}static \u0275fac=(()=>{let e;return function(i){return(e||(e=I(t)))(i||t)}})();static \u0275cmp=T({type:t,selectors:[["p-step-panel"]],contentQueries:function(n,i,o){if(n&1&&(G(o,xt,5),G(o,Z,4)),n&2){let v;D(v=B())&&(i.contentTemplate=v.first),D(v=B())&&(i.templates=v)}},hostVars:11,hostBindings:function(n,i){n&2&&(k("role","tabpanel")("aria-controls",i.ariaControls())("id",i.id())("data-p-active",i.active())("data-pc-name","steppanel"),h("p-steppanel",!0)("p-component",!0)("p-steppanel-active",i.active()))},inputs:{value:[1,"value"]},outputs:{value:"valueChange"},features:[M],decls:3,vars:11,consts:[[1,"p-steppanel-content"],[4,"ngTemplateOutlet","ngTemplateOutletContext"]],template:function(n,i){n&1&&(l(0,Ci,1,0,"p-stepper-separator"),a(1,"div",0),l(2,wi,1,6,"ng-container"),s()),n&2&&(u(i.isSeparatorVisible()?0:-1),r(),_("@content",i.isVertical()?i.active()?X(5,_i,X(3,bt,i.transitionOptions())):X(9,hi,X(7,bt,i.transitionOptions())):void 0),r(),u(i.active()?2:-1))},dependencies:[O,fe,be,j],encapsulation:2,data:{animation:[ze("content",[_e("hidden",ge({height:"0",visibility:"hidden"})),_e("visible",ge({height:"*",visibility:"visible"})),he("visible <=> hidden",[me("250ms cubic-bezier(0.86, 0, 0.07, 1)")]),he("void => *",me(0))])]},changeDetection:0})}return t})(),Fe=(()=>{class t extends A{static \u0275fac=(()=>{let e;return function(i){return(e||(e=I(t)))(i||t)}})();static \u0275cmp=T({type:t,selectors:[["p-step-panels"]],hostVars:4,hostBindings:function(n,i){n&2&&h("p-steppanels",!0)("p-component",!0)},features:[M],ngContentSelectors:R,decls:1,vars:0,template:function(n,i){n&1&&(U(),V(0))},dependencies:[O,j],encapsulation:2,changeDetection:0})}return t})(),L=(()=>{class t extends A{value=H(void 0);linear=ie(!1,{transform:e=>Ce(e)});transitionOptions=ie("400ms cubic-bezier(0.86, 0, 0.07, 1)");_componentStyle=W(Ft);id=Ae(Ke("pn_id_"));stepItems=oe(Se);steps=oe(z);stepList=ne(ye);updateValue(e){this.value.set(e)}isStepActive(e){return this.value()===e}static \u0275fac=(()=>{let e;return function(i){return(e||(e=I(t)))(i||t)}})();static \u0275cmp=T({type:t,selectors:[["p-stepper"]],contentQueries:function(n,i,o){n&1&&($(o,i.stepItems,Se,4),$(o,i.steps,z,4),$(o,i.stepList,ye,5)),n&2&&ce(3)},hostVars:6,hostBindings:function(n,i){n&2&&(k("role","tablist")("id",i.id()),h("p-stepper",!0)("p-component",!0))},inputs:{value:[1,"value"],linear:[1,"linear"],transitionOptions:[1,"transitionOptions"]},outputs:{value:"valueChange"},features:[De([Ft]),M],ngContentSelectors:R,decls:1,vars:0,template:function(n,i){n&1&&(U(),V(0))},dependencies:[O,j],encapsulation:2,changeDetection:0})}return t})(),Mt=(()=>{class t{static \u0275fac=function(n){return new(n||t)};static \u0275mod=Ee({type:t});static \u0275inj=Oe({imports:[L,ye,Fe,te,Se,z,be,j,j]})}return t})()});function xi(t,p){if(t&1){let e=y();a(0,"dvl-button",5),d(1,"translate"),S("onClick",function(){m(e);let i=c(2);return g(i.navigateToTwoFactorSettings())}),s()}t&2&&Q("label",f(1,1,"lblConfigure"))}function Ti(t,p){if(t&1&&(a(0,"div",1)(1,"div",2)(2,"span",3),C(3),d(4,"translate"),s(),l(5,xi,2,3,"dvl-button",4),s()()),t&2){let e=c();r(3),w(f(4,2,"ConnectionLogMessage_2FANotConfigured")),r(2),u(e.canConfigureMFA()?5:-1)}}function Mi(t,p){if(t&1){let e=y();a(0,"div",6)(1,"div",10)(2,"h5"),C(3),d(4,"translate"),s(),a(5,"div",11)(6,"dvl-button",12),d(7,"translate"),S("onClick",function(){m(e);let i=c(2);return g(i.reAuthenticate())}),s()()()()}t&2&&(r(3),w(f(4,3,"lblReAuthenticationIsRequired")),r(3),Q("label",f(7,5,"lblReAuthenticate")),_("warning",!0))}function Pi(t,p){if(t&1){let e=y();a(0,"div",6),d(1,"translate"),a(2,"label"),C(3),d(4,"translate"),s(),a(5,"div",15)(6,"input",16),d(7,"translate"),J("ngModelChange",function(i){m(e);let o=c(3);return Y(o.password,i)||(o.password=i),g(i)}),S("keyup.enter",function(){m(e);let i=c(3);return g(i.validatePassword())}),s(),a(8,"dvl-button",17),d(9,"translate"),S("onClick",function(){m(e);let i=c(3);return g(i.validatePassword())}),s()()()}if(t&2){let e=c(3);h("form-error",e.passwordValiationFailed),k("data-error",f(1,10,"lblInvalidPassword")),r(3),w(f(4,12,"lblEnterYourPasswordToContinue")),r(3),Q("placeholder",f(7,14,"lblPassword")),K("ngModel",e.password),_("autoFocus",!0)("disabled",e.passwordValidated),r(2),Q("label",f(9,16,"lblSubmit")),_("disabled",e.passwordValidated)}}function Ii(t,p){if(t&1&&(a(0,"div",13),l(1,Pi,10,18,"div",14),s()),t&2){let e=c(2);r(),u(e.isSingleSignOnUser?-1:1)}}function ki(t,p){t&1&&(a(0,"span",30),C(1),d(2,"translate"),s()),t&2&&(r(),w(f(2,1,"lblDefault")))}function Oi(t,p){if(t&1){let e=y();a(0,"div",26)(1,"div",27),F(2,"i",28),a(3,"h5",29),C(4),d(5,"translate"),s(),l(6,ki,3,3,"span",30),a(7,"div",31)(8,"dvl-button",32),S("onClick",function(i){let o=m(e).$implicit,v=c(4);return g(v.onActionMenuClick(o,i))}),s(),F(9,"p-menu",33,0),s()()()}if(t&2){let e=p.$implicit,n=c(4);h("disabled",n.disableMfaEdit),r(2),re(e.icon),r(2),w(f(5,8,e.name)),r(2),u(e.isDefault?6:-1),r(3),_("popup",!0)("model",n.actionMenuItems)}}function Ai(t,p){if(t&1&&(a(0,"div",20)(1,"h4",23),C(2),d(3,"translate"),s(),a(4,"div",24),ae(5,Oi,11,10,"div",25,Te),s()()),t&2){let e=c(3);r(2),w(f(3,1,"lblConfigured")),r(3),se(e.getConfiguredFactors())}}function Ei(t,p){t&1&&(a(0,"span",30),C(1),d(2,"translate"),s()),t&2&&(r(),w(f(2,1,"lblDefault")))}function Ui(t,p){if(t&1){let e=y();a(0,"div",36)(1,"div",27),F(2,"i",28),a(3,"h5",29),C(4),d(5,"translate"),s(),l(6,Ei,3,3,"span",30),a(7,"div",31)(8,"dvl-button",32),S("onClick",function(i){let o=m(e).$implicit,v=c(4);return g(v.onActionMenuClick(o,i))}),s(),F(9,"p-menu",33,0),s()()()}if(t&2){let e=p.$implicit,n=c(4);h("disabled",n.disableMfaEdit),r(2),re(e.icon),r(2),w(f(5,8,e.name)),r(2),u(e.isDefault?6:-1),r(3),_("popup",!0)("model",n.actionMenuItems)}}function Vi(t,p){if(t&1&&(a(0,"div",34)(1,"h4",23),C(2),d(3,"translate"),s(),a(4,"div",24),ae(5,Ui,11,10,"div",35,Te),s()()),t&2){let e=c(3);h("mt-4",e.getConfiguredFactors().length>0),r(2),w(f(3,3,"lblPreConfigured")),r(3),se(e.getPreConfiguredFactors())}}function Di(t,p){t&1&&(a(0,"div",40)(1,"span"),C(2),d(3,"translate"),s(),F(4,"i",41),s()),t&2&&(r(2),w(f(3,1,"lblConfigure")))}function Bi(t,p){if(t&1){let e=y();a(0,"div",39),S("click",function(){let i=m(e).$implicit,o=c(4);return g(o.onCardClick(i))}),a(1,"div",27),F(2,"i",28),a(3,"h5",29),C(4),d(5,"translate"),s(),l(6,Di,5,3,"div",40),s()()}if(t&2){let e=p.$implicit,n=c(4);h("disabled",n.disableMfaEdit)("clickable",!e.isConfigured&&!n.disableMfaEdit&&n.currentlyConfiguringFactor===null),r(2),re(e.icon),r(2),w(f(5,8,e.name)),r(2),u(!n.disableMfaEdit&&n.currentlyConfiguringFactor===null?6:-1)}}function $i(t,p){if(t&1&&(a(0,"div",37)(1,"h4",23),C(2),d(3,"translate"),s(),a(4,"div",24),ae(5,Bi,7,10,"div",38,Te),s()()),t&2){let e=c(3);h("mt-4",e.getConfiguredFactors().length>0||e.getPreConfiguredFactors().length>0),r(2),w(f(3,3,"lblNotConfigured")),r(3),se(e.getUnconfiguredFactors())}}function ji(t,p){if(t&1){let e=y();a(0,"div",13)(1,"div",6)(2,"dvl-messages",18),J("valueChange",function(i){m(e);let o=c(2);return Y(o.userMsgTwoFactor,i)||(o.userMsgTwoFactor=i),g(i)}),s()(),a(3,"div",19),l(4,Ai,7,3,"div",20)(5,Vi,7,5,"div",21)(6,$i,7,5,"div",22),s()()}if(t&2){let e=c(2);r(2),K("value",e.userMsgTwoFactor),r(2),u(e.getConfiguredFactors().length>0?4:-1),r(),u(e.getPreConfiguredFactors().length>0?5:-1),r(),u(e.getUnconfiguredFactors().length>0?6:-1)}}function zi(t,p){if(t&1&&(a(0,"h4"),C(1),d(2,"translate"),d(3,"translateEnum"),s()),t&2){let e=c(3);r(),Ve(" ",f(2,2,"lblConfiguring")," ",Be(3,4,e.currentlyConfiguringFactor,e.TwoFactorAuthenticationType)," ")}}function Qi(t,p){if(t&1){let e=y();a(0,"div",6)(1,"dvl-checkbox",46),d(2,"translate"),S("onChange",function(i){m(e);let o=c(4);return g(o.preConfiguredOnChange(i))}),J("ngModelChange",function(i){m(e);let o=c(4);return Y(o.twoFactorEntity.twoFactorInfo.isPreConfigured,i)||(o.twoFactorEntity.twoFactorInfo.isPreConfigured=i),g(i)}),s()()}if(t&2){let e=c(4);r(),Q("label",f(2,3,"lblConfigureLaterByUser")),_("disabled",e.currentlyConfiguringFactor===4&&!e.workingCopy.userProfile.cellPhone),K("ngModel",e.twoFactorEntity.twoFactorInfo.isPreConfigured)}}function Ri(t,p){if(t&1){let e=y();a(0,"div",47)(1,"sd-two-factor",48),S("onReloadTwoFactorConfiguration",function(){m(e);let i=c(4);return g(i.reloadTwoFactor())})("onSaveTwoFactor",function(i){m(e);let o=c(4);return g(o.saveTwoFactor(i))})("onUpdateDuoCapability",function(i){m(e);let o=c(4);return g(o.updateDuoCapability(i))}),s()()}if(t&2){let e=c(4);h("disabled",e.isPreConfiguredSaved),r(),_("password",e.password)("twoFactorInfo",e.twoFactorEntity.twoFactorInfo)("twoFactorEntity",e.twoFactorEntity)("username",e.workingCopy.userSecurity.name)("userId",e.workingCopy.userSecurity.id)("cellPhone",e.workingCopy.userProfile.cellPhone)("email",e.workingCopy.userAccount.email)("setupMode",!0)("setupOnInit",!1)}}function Li(t,p){if(t&1&&(a(0,"div",19)(1,"div",44),l(2,Qi,3,5,"div",6),a(3,"div",6),l(4,Ri,2,11,"div",45),s()()()),t&2){let e=c(3);r(2),u(e.hideConfigureLater?-1:2),r(2),u(e.reload2fa?-1:4)}}function Ni(t,p){if(t&1){let e=y();a(0,"div",13)(1,"div",42)(2,"dvl-button",43),d(3,"translate"),S("onClick",function(){m(e);let i=c(2);return g(i.cancelConfiguration())}),s(),l(4,zi,4,7,"h4"),a(5,"dvl-messages",18),J("valueChange",function(i){m(e);let o=c(2);return Y(o.userMsgTwoFactor,i)||(o.userMsgTwoFactor=i),g(i)}),s()(),l(6,Li,5,2,"div",19),s()}if(t&2){let e=c(2);r(2),_("label",f(3,4,"lblBack")),r(2),u(e.currentlyConfiguringFactor!==null?4:-1),r(),K("value",e.userMsgTwoFactor),r(),u(e.currentlyConfiguringFactor!==null&&e.twoFactorEntity?6:-1)}}function Wi(t,p){if(t&1){let e=y();l(0,Mi,8,7,"div",6),a(1,"p-stepper",7),S("valueChange",function(i){m(e);let o=c();return g(o.activeStepIndex=i)}),a(2,"p-step-panels")(3,"p-step-panel",8),l(4,Ii,2,1,"ng-template",9),s(),a(5,"p-step-panel",8),l(6,ji,7,4,"ng-template",9),s(),a(7,"p-step-panel",8),l(8,Ni,7,6,"ng-template",9),s()()()}if(t&2){let e=c();u(e.showReAuthButton?0:-1),r(),_("value",e.activeStepIndex),r(2),_("value",0),r(2),_("value",1),r(2),_("value",2)}}var Fi,Te,Yn,qi=Me(()=>{pe();qt();Yt();At();We();Et();Dt();Gt();Ht();Zt();Wt();Rt();ni();dt();Nt();ii();ti();xe();oi();_t();Kt();si();Lt();pe();ei();ri();Qt();ci();pi();Jt();Xt();ai();Ut();dt();We();Ye();xe();_t();Fi=["actionMenu"],Te=(t,p)=>p.type;Yn=(()=>{class t extends qe{constructor(e,n,i,o,v,Pt,It,kt,Ot){super(),this.authService=e,this.accountService=n,this.translatorService=i,this.configurationService=o,this.emailConfigurationService=v,this.serverInformationService=Pt,this.currentUserService=It,this.twoFactorUiService=kt,this.router=Ot,this.onAuthTypeChange=new q,this.onConfiguratedByUserChange=new q,this.onPasswordValidated=new q,this.workingCopyChange=new q,this.twoFactorAuthenticationType=P,this.twoFactorEntity=null,this.twoFactorInfo=null,this.resetTwoFactorConfiguaration=!1,this.userMsgTwoFactor=[],this.reload2fa=!1,this.emailEnabled=!1,this.password="",this.passwordValidated=!1,this.passwordValiationFailed=!1,this.isCurrentUser=!1,this.currentEditingFactorId=null,this.currentSecondFactors=[],this.currentlyConfiguringFactor=null,this.isPreConfiguredSaved=!1,this.showReAuthButton=!1,this.isSingleSignOnUser=!1,this.activeStepIndex=0,this.authorizedTwoFactor=[],this.actionMenuItems=[],this.TwoFactorAuthenticationType=P}get twoFactorType(){return this._twoFactorType}set twoFactorType(e){e!==this._twoFactorType&&(this._twoFactorType=e,this.onAuthTypeChange.emit(e))}get is2FATwilioActivated(){return!!this.serverInformationService.serverInformation&&this.serverInformationService.serverInformation.is2FATwilioActivated}get disableMfaEdit(){return this.isCurrentUser&&(this.isSingleSignOnUser?this.showReAuthButton:!this.passwordValidated)}ngOnInit(){this.isCurrentUser=this.currentUserService.currentUserId==this.workingCopy.userSecurity.id,this.getAuthorizedAuthenticationTypes(),this.initEmailConfiguration(),this.checkIfSingleSignOnUser(),this.initializeStepperState()}ngOnDestroy(){this.sessionTimerSubscription&&this.sessionTimerSubscription.unsubscribe()}initializeStepperState(){this.isCurrentUser&&(!this.isSingleSignOnUser||this.showReAuthButton)?this.activeStepIndex=0:this.activeStepIndex=1}canProceedToStep(e){if(!this.isCurrentUser)return e>=1;switch(e){case 0:return!0;case 1:return this.isSingleSignOnUser?!this.showReAuthButton:this.passwordValidated;case 2:return(this.isSingleSignOnUser?!this.showReAuthButton:this.passwordValidated)&&this.currentlyConfiguringFactor!==null;default:return!1}}nextStep(){this.activeStepIndex<2&&this.canProceedToStep(this.activeStepIndex+1)&&this.activeStepIndex++}previousStep(){this.activeStepIndex>0&&this.activeStepIndex--}goToStep(e){this.canProceedToStep(e)&&(this.activeStepIndex=e)}saveTwoFactor(e){this.currentEditingFactorId&&(e.id=this.currentEditingFactorId),e.id||(e.id=we.newGuid());let n=this.workingCopy.userAccount.twoFactorInfoArray.findIndex(i=>i.id===e.id);this.workingCopy.userAccount.twoFactorInfoArray.length===0&&(e.isDefault=!0),n>=0?this.workingCopy.userAccount.twoFactorInfoArray[n]=e:this.workingCopy.userAccount.twoFactorInfoArray.push(e),this.workingCopyChange.emit(this.workingCopy),this.currentlyConfiguringFactor=null,this.twoFactorEntity=null,this.updateSecondFactorBoxes(),this.userMsgTwoFactor.push({severity:"info",detail:this.translatorService.translate("OperationCompletedSuccessfully")}),this.goToStep(1)}reloadTwoFactor(){this.reload2fa=!0,this.currentlyConfiguringFactor&&this.getUserTwoFactorEntity(this.currentlyConfiguringFactor).pipe(N(this.destroyed$)).subscribe(()=>this.reload2fa=!1)}preConfiguredOnChange(e){if(this.currentlyConfiguringFactor===P.Email&&this.workingCopy.userAccount.email===""){this.setFormPristine();return}if(e){let n=new ee;n.id=this.twoFactorEntity.twoFactorInfo.id||we.newGuid(),n.authenticationType=this.currentlyConfiguringFactor,n.isPreConfigured=!0,this.currentlyConfiguringFactor===P.Email?n.accountName=this.workingCopy.userAccount.email:n.accountName=this.workingCopy.userSecurity.name,this.twoFactorEntity.twoFactorInfo.key&&(n.key=this.twoFactorEntity.twoFactorInfo.key);let i=this.workingCopy.userAccount.twoFactorInfoArray.findIndex(o=>o.id===n.id);this.workingCopy.userAccount.twoFactorInfoArray.length===0&&(n.isDefault=!0),i>=0?this.workingCopy.userAccount.twoFactorInfoArray[i]=n:this.workingCopy.userAccount.twoFactorInfoArray.push(n),this.workingCopyChange.emit(this.workingCopy),this.twoFactorEntity.twoFactorInfo.id=n.id,this.twoFactorEntity.twoFactorInfo.isPreConfigured=!0,this.isPreConfiguredSaved=!0,this.updateSecondFactorBoxes(),this.userMsgTwoFactor.push({severity:"info",detail:this.translatorService.translate("OperationCompletedSuccessfully")})}else this.isPreConfiguredSaved=!1,this.onConfiguratedByUserChange.emit(e)}updateDuoCapability(e){let n=this.workingCopy.userAccount.twoFactorInfoArray.findIndex(i=>i.authenticationType===P.Duo);if(n>=0){this.workingCopy.userAccount.twoFactorInfoArray[n].duoAutoCapability=e;let i=this.workingCopy.userAccount.twoFactorInfoArray[n],o=new ee;o.accountName=i.accountName,o.duoAutoCapability=e,o.authenticationType=i.authenticationType,o.isConfigured=i.isConfigured,o.alternateTwoFactorType=i.alternateTwoFactorType,o.duoSelectedCapability=i.duoSelectedCapability,o.key=i.key,o.duoSelectedDeviceID=i.duoSelectedDeviceID,o.isPreConfigured=i.isPreConfigured||i.isPreConfiguredByUser,o.validationCode=i.validationCode,o.phone=i.phone,this.saveTwoFactor(o)}}validatePassword(){let e=new at;e.password=this.password,this.accountService.validatePassword(e).pipe(N(this.destroyed$)).subscribe(n=>{n?(this.passwordValidated=!0,this.passwordValiationFailed=!1,this.onPasswordValidated.emit(this.password),this.nextStep()):(this.passwordValidated=!1,this.passwordValiationFailed=!0)})}initEmailConfiguration(){this.emailConfigurationService.getEmailConfiguration().pipe(N(this.destroyed$)).subscribe(e=>this.emailEnabled=e.emailConfigurationType!==it.None)}checkIfSingleSignOnUser(){this.isSingleSignOnUser=this.workingCopy.userSecurity.authenticationType!==ve.Builtin&&this.workingCopy.userSecurity.authenticationType!==ve.Domain,this.isSingleSignOnUser&&this.isCurrentUser&&this.startSessionTimer()}startSessionTimer(){this.sessionStartTime=new Date(this.currentUserService.accessSnapshot.sessionCreationDate),this.sessionTimerSubscription=Ie(1e3).subscribe(()=>{let n=Math.floor((new Date().getTime()-this.sessionStartTime.getTime())/1e3);this.showReAuthButton=n>300,this.showReAuthButton&&this.isSingleSignOnUser&&this.isCurrentUser&&this.goToStep(0)})}reAuthenticate(){window.sessionStorage.setItem("promptMFAConfig","true"),this.authService.promptExternalLogin(this.currentUserService.currentUsername).subscribe(e=>{this.authService.logout(!0),window.location.href=e.location})}configureSecondFactor(e,n){this.userMsgTwoFactor=[],this.currentlyConfiguringFactor=e,this.isPreConfiguredSaved=!1,this.currentEditingFactorId=n||null,this.getUserTwoFactorEntity(e).pipe(N(this.destroyed$)).subscribe(()=>{if(e===P.Email&&this.workingCopy.userAccount.email==="")this.setFormPristine();else if(e===P.Radius){let i=new ee;i.authenticationType=P.Radius,i.accountName=this.workingCopy.userSecurity.name,i.isConfigured=!0,n&&(i.id=n),this.saveTwoFactor(i)}this.goToStep(2)})}onCardClick(e){!e.isConfigured&&!this.disableMfaEdit&&this.currentlyConfiguringFactor===null&&this.configureSecondFactor(e.type)}removeSecondFactor(e){let n=this.workingCopy.userAccount.twoFactorInfoArray.findIndex(i=>i.id===e);if(n>=0){if(!this.workingCopy.userAccount.twoFactorInfoArray.splice(n,1)[0]){this.userMsgTwoFactor.push({severity:"error",detail:this.translatorService.translate("FailedToRemoveFactor")});return}this.workingCopyChange.emit(this.workingCopy),this.updateSecondFactorBoxes(),this.userMsgTwoFactor.push({severity:"info",detail:this.translatorService.translate("FactorRemovedSuccessfully")}),this.currentForm&&(this.currentForm.form.markAsDirty(),this.currentForm.form.patchValue({}))}}cancelConfiguration(){this.currentlyConfiguringFactor=null,this.twoFactorEntity=null,this.userMsgTwoFactor=[],this.isPreConfiguredSaved=!1,this.previousStep()}getConfiguredFactorInfo(e){return this.workingCopy.userAccount.twoFactorInfoArray.find(n=>n.authenticationType===e&&(n.isConfigured||n.isPreConfigured))}getConfiguredFactors(){return this.currentSecondFactors.filter(e=>e.isConfigured&&!e.isPreConfigured)}getPreConfiguredFactors(){return this.currentSecondFactors.filter(e=>e.isPreConfigured)}getUnconfiguredFactors(){return this.currentSecondFactors.filter(e=>!e.isConfigured)}hasSecondFactors(){return this.getConfiguredFactors().length>0||this.getPreConfiguredFactors().length>0||this.getUnconfiguredFactors().length>0}canConfigureMFA(){return this.currentUserService.isAdministrator}onActionMenuClick(e,n){this.initActionMenuItems(e),this.actionMenu.toggle(n)}navigateToTwoFactorSettings(){this.router.navigate(["/administration","server-settings","two-factor"])}updateSecondFactorBoxes(){this.currentSecondFactors=this.authorizedTwoFactor.map(e=>{let n=this.getConfiguredFactorInfo(e);return{id:n?.id,type:e,name:this.twoFactorUiService.getFactorDisplayName(e),icon:this.twoFactorUiService.getFactorIcon(e),isConfigured:!!n,isDefault:n?.isDefault??!1,isPreConfigured:n?.isPreConfigured??!1}})}getUserTwoFactorEntity(e){return new Pe(n=>{this.onConfiguratedByUserChange.emit(!1),this.configurationService.getUserTwoFactorEntity(this.workingCopy.userSecurity.id,e).subscribe(i=>{i||(i=new nt),i.twoFactorInfo||(i.twoFactorInfo=new ee),i.twoFactorInfo.accountName=this.workingCopy.userAccount.email,i.twoFactorInfo.key=i.secretKey,this.twoFactorEntity=i,this.twoFactorType=e,this.onConfiguratedByUserChange.emit(this.twoFactorEntity.twoFactorInfo.isPreConfigured),n.next(!0),n.complete()})})}getAuthorizedAuthenticationTypes(){let e=[];this.serverInformationService.publicServerInformation.availabelTwoFactorAuthenticationTypes&&(e=e.concat(this.serverInformationService.publicServerInformation.availabelTwoFactorAuthenticationTypes)),this.authorizedTwoFactor=e,this.updateSecondFactorBoxes()}setFormPristine(){this.currentForm.form.markAsPristine(),this.currentForm.form.updateValueAndValidity()}initActionMenuItems(e){this.actionMenuItems=[],e.isDefault||this.actionMenuItems.push({label:this.translatorService.translate("lblSetAsDefault"),icon:"di dvl-icon-check-circle",command:()=>this.setDefaultFactor(e.id)}),this.actionMenuItems.push({label:this.translatorService.translate("lblReconfigureMFA"),icon:"di dvl-icon-edit",command:()=>this.configureSecondFactor(e.type,e.id)}),(this.currentSecondFactors.filter(n=>n.isConfigured).length>1||!(this.serverInformationService.publicServerInformation.disableSelfMFADelete&&!this.currentUserService.isAdministrator))&&this.actionMenuItems.push({label:this.translatorService.translate("lblRemove2FA"),icon:"di dvl-icon-delete",command:()=>this.removeSecondFactor(e.id)})}setDefaultFactor(e){this.workingCopy.userAccount.twoFactorInfoArray.forEach(n=>{n.isDefault=n.id===e}),this.workingCopyChange.emit(this.workingCopy),this.updateSecondFactorBoxes(),this.userMsgTwoFactor.push({severity:"info",detail:this.translatorService.translate("DefaultFactorSetSuccessfully")}),this.currentForm&&(this.currentForm.form.markAsDirty(),this.currentForm.form.patchValue({}))}static{this.\u0275fac=function(n){return new(n||t)(x(st),x(Ct),x(Xe),x(yt),x(St),x(ot),x(rt),x(vt),x(je))}}static{this.\u0275cmp=T({type:t,selectors:[["dvls-user-two-factor-configuration"]],viewQuery:function(n,i){if(n&1&&Ue(Fi,5),n&2){let o;D(o=B())&&(i.actionMenu=o.first)}},inputs:{workingCopy:"workingCopy",currentForm:"currentForm",hideConfigureLater:"hideConfigureLater"},outputs:{onAuthTypeChange:"onAuthTypeChange",onConfiguratedByUserChange:"onConfiguratedByUserChange",onPasswordValidated:"onPasswordValidated",workingCopyChange:"workingCopyChange"},features:[M],decls:2,vars:1,consts:[["actionMenu",""],[1,"p-message","p-message-warn"],[1,"flex","items-center","w-full","gap-4"],[1,"flex-1"],["variant","primary",3,"label"],["variant","primary",3,"onClick","label"],[1,"col-span-12","form-group"],[3,"valueChange","value"],[3,"value"],["pTemplate","content"],[1,"session-info-container","p-card","p-3","mb-3"],[1,"reauth-button"],["variant","primary","size","sm","icon","pi pi-refresh",3,"onClick","warning","label"],[1,"grid","gap-4"],[1,"col-span-12","form-group",3,"form-error"],[1,"flex","flex-col","gap-4"],["type","password","sdAutoFocusInput","","pInputText","","autocomplete","off",1,"w-full",3,"ngModelChange","keyup.enter","ngModel","autoFocus","disabled","placeholder"],["variant","primary",3,"onClick","disabled","label"],["messageStyleClass","no-margin",3,"valueChange","value"],[1,"col-span-12"],[1,"configured-factors-region"],[1,"preconfigured-factors-region",3,"mt-4"],[1,"unconfigured-factors-region",3,"mt-4"],[1,"region-title"],[1,"second-factors-grid"],[1,"second-factor-card","p-card","configured",3,"disabled"],[1,"second-factor-card","p-card","configured"],[1,"factor-header"],[1,"factor-icon"],[1,"factor-name"],[1,"default-label"],[1,"factor-actions"],["variant","icon","icon","pi pi-ellipsis-v",3,"onClick"],["appendTo","body",3,"popup","model"],[1,"preconfigured-factors-region"],[1,"second-factor-card","p-card","preconfigured",3,"disabled"],[1,"second-factor-card","p-card","preconfigured"],[1,"unconfigured-factors-region"],[1,"second-factor-card","p-card",3,"disabled","clickable"],[1,"second-factor-card","p-card",3,"click"],[1,"configure-hint"],[1,"pi","pi-arrow-right"],[1,"col-span-12","form-group","flex","items-center","configuration-title"],["variant","secondary","icon","pi pi-chevron-left",3,"onClick","label"],[1,"p-card","p-3"],[1,"two-factor-form-wrapper",3,"disabled"],["binary","true","name","preconfiguredTwoFa",3,"onChange","ngModelChange","label","disabled","ngModel"],[1,"two-factor-form-wrapper"],[3,"onReloadTwoFactorConfiguration","onSaveTwoFactor","onUpdateDuoCapability","password","twoFactorInfo","twoFactorEntity","username","userId","cellPhone","email","setupMode","setupOnInit"]],template:function(n,i){n&1&&l(0,Ti,6,4,"div",1)(1,Wi,9,5),n&2&&u(i.hasSecondFactors()?1:0)},dependencies:[Ze,ft,lt,pt,Ne,Qe,Re,Le,tt,ut,Z,ct,ht,Mt,L,Fe,te,gt,mt,wt,et],styles:['[_nghost-%COMP%]     .homeBox{margin:0}.row-button[_ngcontent-%COMP%]{display:flex;align-items:flex-end}.row-button[_ngcontent-%COMP%]   button[_ngcontent-%COMP%]{border-color:#cdcdcd;padding:3px 7px}.session-info-container[_ngcontent-%COMP%]{display:flex;flex-direction:row;border:1px solid var(--border-base-100);border-radius:6px;align-items:center;justify-content:center}.session-info-container[_ngcontent-%COMP%]   h5[_ngcontent-%COMP%]{font-weight:600}.session-info-container[_ngcontent-%COMP%]   .reauth-button[_ngcontent-%COMP%]{margin-left:auto}.second-factors-grid[_ngcontent-%COMP%]{display:flex;flex-direction:column;gap:1rem;margin-top:1rem}.second-factor-card[_ngcontent-%COMP%]{position:relative;transition:all .3s ease;border:2px solid transparent;border-radius:.5rem;display:flex;padding:1rem 1.5rem;min-width:0}.second-factor-card.configured[_ngcontent-%COMP%]{border-color:var(--blue-primary)}.second-factor-card.configured[_ngcontent-%COMP%]   .factor-header[_ngcontent-%COMP%]   .factor-icon[_ngcontent-%COMP%]{color:var(--blue-primary)}.second-factor-card.preconfigured[_ngcontent-%COMP%]{border-color:#7fbcec}.second-factor-card.preconfigured[_ngcontent-%COMP%]   .factor-header[_ngcontent-%COMP%]   .factor-icon[_ngcontent-%COMP%]{color:#7fbcec}.second-factor-card.preconfigured[_ngcontent-%COMP%]   .factor-actions[_ngcontent-%COMP%]     .p-button{color:#7fbcec}.second-factor-card[_ngcontent-%COMP%]   .default-label[_ngcontent-%COMP%]{background-color:var(--blue-500);color:var(--text-color-primary);border-radius:.375rem;padding:.25rem .5rem;font-size:.875rem;flex-shrink:0;white-space:nowrap}.second-factor-card.disabled[_ngcontent-%COMP%]{opacity:.6;pointer-events:none}.second-factor-card.clickable[_ngcontent-%COMP%]{cursor:pointer;border-color:var(--grey-300)}.second-factor-card.clickable[_ngcontent-%COMP%]:hover{border-color:var(--blue-primary);background-color:var(--blue-300);box-shadow:0 4px 12px #007bff26}.second-factor-card.clickable[_ngcontent-%COMP%]:hover   .factor-icon[_ngcontent-%COMP%], .second-factor-card.clickable[_ngcontent-%COMP%]:hover   .configure-hint[_ngcontent-%COMP%]{color:var(--blue-primary)}.second-factor-card.clickable[_ngcontent-%COMP%]:hover   .configure-hint[_ngcontent-%COMP%]   i[_ngcontent-%COMP%]{transform:scale(1.1)}.second-factor-card[_ngcontent-%COMP%]:hover:not(.disabled):not(.clickable){box-shadow:0 4px 12px #0000001a}.factor-header[_ngcontent-%COMP%]{display:flex;align-items:center;width:100%;gap:.75rem;min-height:2.5rem}.factor-header[_ngcontent-%COMP%]   .factor-icon[_ngcontent-%COMP%]{font-size:1.5rem;color:var(--grey-600);flex-shrink:0}.factor-header[_ngcontent-%COMP%]   .factor-name[_ngcontent-%COMP%]{margin:0;font-size:1.1rem;font-weight:600;flex:0 1 auto;min-width:0;word-break:break-word;overflow-wrap:break-word}.factor-header[_ngcontent-%COMP%]   .configure-hint[_ngcontent-%COMP%]{margin-left:auto}.factor-actions[_ngcontent-%COMP%]{display:flex;gap:.5rem;align-items:center;margin-left:auto;flex-shrink:0}.factor-actions[_ngcontent-%COMP%]     .p-button{border:0;border-radius:1rem;font-size:1rem}.configure-hint[_ngcontent-%COMP%]{display:flex;align-items:center;justify-content:center;gap:.5rem;color:var(--grey-600);font-size:.8rem;padding:.25rem .5rem;border-radius:.375rem;background-color:var(--grey-100);border:1px dashed var(--grey-300);transition:all .3s ease;text-align:center;white-space:nowrap}.configure-hint[_ngcontent-%COMP%]   i[_ngcontent-%COMP%]{font-size:.8rem;transition:transform .3s ease}.configure-hint[_ngcontent-%COMP%]   span[_ngcontent-%COMP%]{font-weight:500}.factor-warning[_ngcontent-%COMP%]     .p-messages{margin:0}.factor-warning[_ngcontent-%COMP%]     .p-messages .p-message{padding:.5rem;font-size:.8rem}.configuration-panel[_ngcontent-%COMP%]{margin-top:2rem}.configuration-panel[_ngcontent-%COMP%]   .p-card[_ngcontent-%COMP%]{border-left:4px solid var(--blue-primary)}.configuration-panel[_ngcontent-%COMP%]   h4[_ngcontent-%COMP%]{color:var(--blue-primary);margin-bottom:1rem;display:flex;align-items:center;gap:.5rem}.configuration-panel[_ngcontent-%COMP%]   h4[_ngcontent-%COMP%]:before{content:"";width:8px;height:8px;background-color:var(--blue-primary);border-radius:50%;animation:_ngcontent-%COMP%_pulse 2s infinite}@keyframes _ngcontent-%COMP%_pulse{0%{opacity:1}50%{opacity:.5}to{opacity:1}}.session-info-container[_ngcontent-%COMP%]{border-left:4px solid var(--yellow-primary)}.session-info-container[_ngcontent-%COMP%]   .reauth-button[_ngcontent-%COMP%]{margin-top:1rem}.configuration-title[_ngcontent-%COMP%]{gap:8px}.configuration-title[_ngcontent-%COMP%]     .p-button{background:transparent;border:none;padding:0}  .p-stepper-nav{display:none}.two-factor-form-wrapper[_ngcontent-%COMP%]{position:relative}.two-factor-form-wrapper.disabled[_ngcontent-%COMP%]{pointer-events:none;opacity:.5;-webkit-user-select:none;user-select:none}.two-factor-form-wrapper.disabled[_ngcontent-%COMP%]:after{content:"";position:absolute;inset:0;background-color:#8080801a;border-radius:.5rem;cursor:not-allowed}@media (max-width: 768px){.factor-actions[_ngcontent-%COMP%]     .p-button{min-width:auto;flex:none}}']})}}return t})()});export{Yn as a,qi as b};
