import{Sg as o,Tg as p}from"./chunk-GDJMZZ4C.js";import{Rd as h,h as r,ma as u,sa as a}from"./chunk-4HFZZ7HS.js";import{e as i}from"./chunk-SZAW6ZBW.js";var b,C=i(()=>{u();p();h();b=(()=>{class e{constructor(){this.hasOtpSubject=new r(!1),this.currentConnectionSubject=new r(null),this.currentConnectionRequest=this.currentConnectionSubject.asObservable()}get hasOTP(){return this.hasOtpSubject.value}getCurrentConnection(){return this.currentConnectionRequest}getCurrentConnectionSnapshot(){return this.currentConnectionSubject.getValue()}setCurrentConnection(t,n=!1){let s=this.getCurrentConnectionSnapshot(),c=!n&&s?.connection&&t&&t.id===s?.connection.id;t&&this.setHasOTP(t),this.currentConnectionSubject.next({connection:t,isAnUpdate:c})}setHasOTP(t){let n=t.data.otpKeyItem?.hasSensitiveData||!!t.data.otpConnectionID||!!t.data.otpPrivateVaultSearchString||t.data.otpSourceMode===o.CurrentSession||t.data.otpSourceMode===o.Inherited;this.hasOtpSubject.next(n)}static{this.\u0275fac=function(n){return new(n||e)}}static{this.\u0275prov=a({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});export{b as a,C as b};
