import{e}from"./chunk-SZAW6ZBW.js";var r,s=e(()=>{r=class{constructor(){this.emailPattern=/^[a-zA-Z0-9.!#$%&’*+/=?^_`{|}~-]+@[a-zA-Z0-9-]+(?:\.[a-zA-Z0-9-]+)+$/}validate(t){return this.emailPattern.test(this.extractEmail(t))}extractEmail(t){let a=t.lastIndexOf("<"),n=t.lastIndexOf(">");return a>=0&&n>=a?t.substring(a+1,n-1):t}}});export{r as a,s as b};
