import{i as e,j as o}from"./chunk-2JML2BCE.js";import{e as n}from"./chunk-SZAW6ZBW.js";var t=n(()=>{o();e.setLanguage(["fr","fr-FR"],{Markdown:"Markdown",WYSIWYG:"WYSIWYG",Write:"\xC9crire",Preview:"Aper\xE7u",Headings:"En-t\xEAtes",Paragraph:"Paragraphe",Bold:"Gras",Italic:"Italique",Strike:"Barr\xE9",Code:"Code en ligne",Line:"Ligne",Blockquote:"Citation","Unordered list":"Liste non-ordonn\xE9e","Ordered list":"Liste ordonn\xE9e",Task:"T\xE2che",Indent:"Retrait",Outdent:"Sortir","Insert link":"Ins\xE9rer un lien","Insert CodeBlock":"Ins\xE9rer un bloc de code","Insert table":"Ins\xE9rer un tableau","Insert image":"Ins\xE9rer une image",Heading:"En-t\xEAte","Image URL":"URL de l'image","Select image file":"S\xE9lectionnez un fichier image","Choose a file":"Choisissez un fichier","No file":"Pas de fichier",Description:"Description",OK:"OK",More:"de plus",Cancel:"Annuler",File:"Fichier",URL:"URL","Link text":"Texte du lien","Add row to up":"Ajouter une ligne vers le haut","Add row to down":"Ajouter une ligne vers le bas","Add column to left":"Ajouter une colonne \xE0 gauche","Add column to right":"Ajouter une colonne \xE0 droite","Remove row":"Supprimer une ligne","Remove column":"Supprimer une colonne","Align column to left":"Aligner \xE0 gauche","Align column to center":"Aligner au centre","Align column to right":"Aligner \xE0 droite","Remove table":"Supprimer le tableau","Would you like to paste as table?":"Voulez-vous coller ce contenu en tant que tableau ?","Text color":"Couleur du texte","Auto scroll enabled":"D\xE9filement automatique activ\xE9","Auto scroll disabled":"D\xE9filement automatique d\xE9sactiv\xE9","Choose language":"Choix de la langue"})});t();
