import{U as l,V as C,Wa as u,Xa as d}from"./chunk-7YUL6BIM.js";import{a as c,b as v}from"./chunk-V3J5YJ3L.js";import{b as n,c as p}from"./chunk-YTY7HHO3.js";import{Rd as P,na as h,sa as o,x as i,xa as a}from"./chunk-4HFZZ7HS.js";import{e as m}from"./chunk-SZAW6ZBW.js";var G,S=m(()=>{h();C();v();P();d();p();G=(()=>{class s{constructor(e,t){this.pamAssetServices=e,this.translatorService=t}getVaultContentSummary(e){return this.pamAssetServices.PamFolders_GetPamVaultContentSummary(e)}getPamFolderCheckoutPoliciesAssignments(){return this.pamAssetServices.PamCheckoutPolicies_GetPamFolderCheckoutPoliciesAssignments().pipe(i(e=>e.result===c.Success?e.data:[]))}getFolderContentSummary(e){return this.pamAssetServices.PamFolders_GetPamFolderContentSummary(e)}buildDeleteFolderMessage(e,t=null){let r="";return t=new l(t),t.folderCount+t.credentialCount>0&&(r+="<p>"+this.translatorService.translate("PamFolderNotEmpty"),r+="<br />",r+="- "+this.translatorService.translate("PamXFolders",t.folderCount)+"<br />",r+="- "+this.translatorService.translate("PamXAccounts",t.credentialCount)+"<br />",r+="- "+this.translatorService.translate("PamXActiveCheckouts",t.checkoutActiveCount)+"<br />",r+="- "+this.translatorService.translate("PamXPendingCheckouts",t.checkoutPendingCount)+"<br />",r+="- "+this.translatorService.translate("PamXGrantedCheckouts",t.checkoutGrantedCount)+"<br /></p>"),r+=this.translatorService.translate("AreYouSureYouWantToDelete",e),r}static{this.\u0275fac=function(t){return new(t||s)(a(u),a(n))}}static{this.\u0275prov=o({token:s,factory:s.\u0275fac,providedIn:"root"})}}return s})()});export{G as a,S as b};
