import{a as O,b as V,c as W,d as B}from"./chunk-7LY47UGN.js";import{Ce as D,De as F,Ee as l,Fe as N,Wj as R,Xj as Q,s as v,t as j,ug as w,vg as T,we as h,wg as z,xe as b,xg as C,ye as m,ze as y}from"./chunk-GDJMZZ4C.js";import{c as g,d as k,g as U,h as M}from"./chunk-V3J5YJ3L.js";import{b as G,c as E}from"./chunk-YTY7HHO3.js";import{Rd as I,ka as i,na as P,sa as S,x as r,xa as d}from"./chunk-4HFZZ7HS.js";import{e as A}from"./chunk-SZAW6ZBW.js";var f,x=A(()=>{k();V();f=class o extends O{toQueryString(){let a=[];return a.push(super.toQueryString()),this.groupId!==null&&this.groupId.toString()&&a.push("groupId="+encodeURIComponent(this.groupId.toString())),this.hideExistingUsers!==null&&this.hideExistingUsers.toString()&&a.push("hideExistingUsers="+encodeURIComponent(this.hideExistingUsers.toString())),a.join("&")}constructor(a=null){super(a),this.clone=()=>new o(this),this.groupId=g.manageDefaultValue(a,"groupId",""),this.hideExistingUsers=g.manageDefaultValue(a,"hideExistingUsers",!1)}}});var oe,H=A(()=>{j();P();T();b();y();N();F();B();C();x();I();E();M();Q();oe=(()=>{class o extends v{constructor(t,e,s){super(t,e),this.assetService=s}getAdDomains(t){return this.assetService.GetAllDomains(t).pipe(i(e=>this.handleGetApiResult(e)),r(e=>this.returnWhenSuccessful(e,()=>e.data.map(s=>new m(s)))))}getAdUser(t,e){return this.assetService.GetAdUser(t,e).pipe(i(s=>this.handleApiResult(s,"UserNotFoundOnDomain")),r(s=>this.returnWhenSuccessful(s,()=>s.data?new w(s.data):null)))}getAdUserProfile(t,e){return this.assetService.GetAdUserProfile(t,e).pipe(i(s=>this.handleApiResult(s,"UserNotFoundOnDomain")),r(s=>this.returnWhenSuccessful(s,()=>s.data?new z(s.data):null)))}getAdUsers(t,e,s){return this.assetService.GetAllDomainUsers(t.join(","),e,s).pipe(i(n=>this.handleGetApiResult(n)),r(n=>this.returnWhenSuccessful(n,()=>n.data?n.data.map(c=>new l(c)):null)))}getAdUserGroup(t,e){return this.assetService.GetAdGroup(t,e).pipe(i(s=>this.handleApiResult(s,"GroupNotFoundOnDomain")),r(s=>this.returnWhenSuccessful(s,()=>s.data)))}getAdUserGroups(t,e){return this.assetService.GetAllDomainGroups(t.join(","),e).pipe(i(s=>this.handleGetApiResult(s)),r(s=>this.returnWhenSuccessful(s,()=>s.data.map(n=>new h(n)))))}getAzureDomains(){return this.assetService.GetAllAzureDomains().pipe(i(t=>this.handleGetApiResult(t)),r(t=>this.returnWhenSuccessful(t,()=>t.data.map(e=>new m(e)))))}getAzureUsers(t){return this.assetService.GetAzureDomainUsers(t,null).pipe(i(e=>this.handleGetApiResult(e)),r(e=>this.returnWhenSuccessful(e,()=>e.data?.items?e.data.items.map(s=>new l(s)):null)))}getPagedAzureUsers(t,e,s,n,c,u){let p=new f;return p.pageSize=e,p.nextPageToken=s||null,p.filter=n,p.sortOrder=c,p.hideExistingUsers=u,this.assetService.GetAzureDomainUsers(t,p)}getAzureUser(t){return this.assetService.GetAzureDomainUser(t).pipe(i(e=>this.handleGetApiResult(e)),r(e=>this.returnWhenSuccessful(e,()=>e.data?new D(e.data):null)))}getAzureUserGroups(){return this.assetService.GetAzureDomainGroups(null).pipe(i(t=>this.handleGetApiResult(t)),r(t=>this.returnWhenSuccessful(t,()=>t.data?.items.map(e=>new h(e)))))}getPagedAzureUserGroups(t,e,s,n,c=!1){let u=new W;return u.pageSize=t,u.nextPageToken=e||null,u.filter=s,u.sortOrder=n,u.hideExistingUsers=c,this.assetService.GetAzureDomainGroups(u)}getAzureGroupMembers(t){return this.assetService.GetAzureGroupsMember(t).pipe(i(e=>this.handleGetApiResult(e)),r(e=>this.returnWhenSuccessful(e,()=>e.data.map(s=>new l(s)))))}getAzureUserMemberOf(t){return this.assetService.GetAzureUserMemberOf(t).pipe(i(e=>this.handleGetApiResult(e)),r(e=>this.returnWhenSuccessful(e,()=>e.data.map(s=>new h(s)))))}getOktaDomains(){return this.assetService.GetOktaDomain().pipe(i(t=>this.handleGetApiResult(t)),r(t=>this.returnWhenSuccessful(t,()=>t.data.map(e=>new m(e)))))}getOktaUserGroups(){return this.assetService.GetOktaGroups(null).pipe(i(t=>this.handleGetApiResult(t)),r(t=>this.returnWhenSuccessful(t,()=>t.data.map(e=>new h(e)))))}getOktaUsers(t){return this.assetService.GetAllOktaUsers(t).pipe(i(e=>this.handleGetApiResult(e)),r(e=>this.returnWhenSuccessful(e,()=>e.data?e.data.map(s=>new l(s)):null)))}getPingOneDomains(){return this.assetService.GetPingOneDomain().pipe(i(t=>this.handleGetApiResult(t)),r(t=>this.returnWhenSuccessful(t,()=>t.data.map(e=>new m(e)))))}getPingOneUsers(t,e){return this.assetService.GetAllPingOneUsers(t,e).pipe(i(s=>this.handleGetApiResult(s)),r(s=>this.returnWhenSuccessful(s,()=>s.data?s.data.map(n=>new l(n)):null)))}getPingOneUserGroups(){return this.assetService.GetPingOneGroups(null).pipe(i(t=>this.handleGetApiResult(t)),r(t=>this.returnWhenSuccessful(t,()=>t.data.map(e=>new h(e)))))}static{this.\u0275fac=function(e){return new(e||o)(d(G),d(U),d(R))}}static{this.\u0275prov=S({token:o,factory:o.\u0275fac,providedIn:"root"})}}return o})()});export{oe as a,H as b};
