import{a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v}from"./chunk-BNMZLN3G.js";import"./chunk-XRGTMJIH.js";import"./chunk-UOI75ONS.js";import"./chunk-4HFZZ7HS.js";import"./chunk-SZAW6ZBW.js";v();export{g as AnimationDriver,f as NoopAnimationDriver,r as \u0275Animation,n as \u0275AnimationEngine,t as \u0275AnimationRenderer,u as \u0275AnimationRendererFactory,h as \u0275AnimationStyleNormalizer,s as \u0275BaseAnimationRenderer,i as \u0275NoopAnimationStyleNormalizer,p as \u0275WebAnimationsDriver,o as \u0275WebAnimationsPlayer,m as \u0275WebAnimationsStyleNormalizer,l as \u0275allowPreviousPlayerStylesMerge,k as \u0275camelCaseToDashCase,d as \u0275containsElement,q as \u0275createEngine,a as \u0275getParentElement,e as \u0275invokeQuery,j as \u0275normalizeKeyframes,b as \u0275validateStyleProperty,c as \u0275validateWebAnimatableStyleProperty};
