import{c as f,d as S}from"./chunk-UAMVFZP3.js";import"./chunk-Z77K36CL.js";import"./chunk-U636PDRQ.js";import"./chunk-43QQ6LNX.js";import"./chunk-QRHDOUB7.js";import"./chunk-6ABP6KIV.js";import"./chunk-PQQRTQYF.js";import"./chunk-YKKPJHZ2.js";import"./chunk-KNAULZU4.js";import"./chunk-A7OV5QSE.js";import{c as l,d as D}from"./chunk-SKBHE3FA.js";import"./chunk-CU6UMDSY.js";import"./chunk-EZHNF6LB.js";import"./chunk-2DPI5O4Z.js";import"./chunk-SN34CLEC.js";import"./chunk-ZENOWJOG.js";import"./chunk-5ZECTPCZ.js";import"./chunk-DZLHJ5XE.js";import"./chunk-KMAXH6NO.js";import"./chunk-2JML2BCE.js";import"./chunk-MBRDXADW.js";import"./chunk-A6CDAFAT.js";import"./chunk-KA2SPUWT.js";import"./chunk-KLB4BWDP.js";import"./chunk-YINSOKTV.js";import"./chunk-7VLSZZAB.js";import"./chunk-TKXVPDQE.js";import"./chunk-QCYFTXYO.js";import"./chunk-DWPKFEIH.js";import"./chunk-VKHMVPMU.js";import"./chunk-FVJGYA76.js";import"./chunk-R65ANNQU.js";import"./chunk-QRVWWFBT.js";import"./chunk-UAPDCKDX.js";import"./chunk-QUUSYLZK.js";import"./chunk-MRFU7ZH5.js";import"./chunk-S6WI4OAL.js";import"./chunk-FESI7YJY.js";import"./chunk-5JRUNHF6.js";import"./chunk-4WJQGABW.js";import"./chunk-MAQNKK2A.js";import"./chunk-XMICNJPL.js";import"./chunk-MAVIT7QG.js";import"./chunk-6Q7YU3GA.js";import"./chunk-7YUL6BIM.js";import"./chunk-A3Y77RII.js";import"./chunk-5DATZV72.js";import"./chunk-44HOQHWU.js";import"./chunk-RXTWCCZH.js";import"./chunk-XPEI6QAS.js";import"./chunk-HH37EZMD.js";import"./chunk-6PNFVHSA.js";import"./chunk-QST7UV2H.js";import"./chunk-45NZ5NLX.js";import"./chunk-LDWXTTVD.js";import"./chunk-LPEQMDNZ.js";import"./chunk-WWZYDBMP.js";import"./chunk-3OCEUHZW.js";import"./chunk-QUK73RJT.js";import"./chunk-WSU244SW.js";import"./chunk-R2ULAEP2.js";import"./chunk-2FOKRXOS.js";import"./chunk-UGQXGBKF.js";import"./chunk-TCUK52QT.js";import"./chunk-JU5LF7AN.js";import"./chunk-AFLM5QFV.js";import"./chunk-DCQJQBJ7.js";import"./chunk-VYRWAFSM.js";import"./chunk-UTF7W4HP.js";import"./chunk-OJRDBZWF.js";import"./chunk-LFKM2JDJ.js";import"./chunk-6G52AAKU.js";import"./chunk-QQZCALHS.js";import"./chunk-EXP5MDOK.js";import"./chunk-M55KBHC5.js";import"./chunk-JE2X3YFA.js";import{e as c,g as v,p as s,v as C}from"./chunk-HW6LKGMV.js";import"./chunk-XRGTMJIH.js";import"./chunk-ZNCTAJJP.js";import"./chunk-THDAWOZM.js";import"./chunk-FMPRHYG5.js";import"./chunk-GDJMZZ4C.js";import"./chunk-V3J5YJ3L.js";import{a as p,b as h}from"./chunk-UIVS6623.js";import"./chunk-2XIOYYFS.js";import"./chunk-HYPRVELK.js";import{b as a,c as g}from"./chunk-YTY7HHO3.js";import"./chunk-UOI75ONS.js";import{Fb as i,Qb as n,Rd as u,Ub as r,yc as m}from"./chunk-4HFZZ7HS.js";import{e as d}from"./chunk-SZAW6ZBW.js";var W,T=d(()=>{h();C();D();S();u();v();g();W=(()=>{class t extends p{constructor(o,e){super(),this.titleService=o,this.translatorService=e}static{this.\u0275fac=function(e){return new(e||t)(i(c),i(a))}}static{this.\u0275cmp=n({type:t,selectors:[["ng-component"]],features:[r],decls:3,vars:0,template:function(e,b){e&1&&m(0,"router-outlet")(1,"dps-connection-comment")(2,"dvls-ticket-selector")},dependencies:[s,l,f],encapsulation:2})}}return t})()});T();export{W as WebClientComponent};
