import{e as c}from"./chunk-SZAW6ZBW.js";var i,a=c(()=>{i=function(o){return o[o.image=0]="image",o[o.icon=1]="icon",o[o.booleanIcon=2]="booleanIcon",o[o.action=3]="action",o[o.function=4]="function",o}(i||{})});export{i as a,a as b};
