import{a as p,b as f}from"./chunk-24LNMKH3.js";import"./chunk-EXSN6QWV.js";import"./chunk-JE2X3YFA.js";import{p as l,v as c}from"./chunk-HW6LKGMV.js";import"./chunk-GDJMZZ4C.js";import"./chunk-V3J5YJ3L.js";import"./chunk-UIVS6623.js";import"./chunk-2XIOYYFS.js";import"./chunk-HYPRVELK.js";import"./chunk-YTY7HHO3.js";import"./chunk-UOI75ONS.js";import{Qb as n,Rd as a,Ub as o,wc as i,xc as r,yc as s}from"./chunk-4HFZZ7HS.js";import{e as u}from"./chunk-SZAW6ZBW.js";var M,d=u(()=>{f();c();a();M=(()=>{class t extends p{constructor(){super()}getComponentTitle(){return"lblConfiguration"}static{this.\u0275fac=function(e){return new(e||t)}}static{this.\u0275cmp=n({type:t,selectors:[["ng-component"]],features:[o],decls:2,vars:0,consts:[[1,"full-height","full-width"]],template:function(e,g){e&1&&(i(0,"div",0),s(1,"router-outlet"),r())},dependencies:[l],styles:["[_nghost-%COMP%]   ul[_ngcontent-%COMP%]{list-style-type:none;padding:0}[_nghost-%COMP%]   .page-content[_ngcontent-%COMP%]{width:82%;display:flex}"]})}}return t})()});d();export{M as ConfigurationComponent};
