import{a as R,b as rt}from"./chunk-7LH6B767.js";import"./chunk-JRDY34ZJ.js";import"./chunk-ZZVK6M6N.js";import{a as G,b as it}from"./chunk-GKRJ4DYR.js";import"./chunk-CD2FEHIR.js";import"./chunk-A7OV5QSE.js";import"./chunk-SN34CLEC.js";import"./chunk-ZENOWJOG.js";import"./chunk-SADKIAAT.js";import"./chunk-Z5U6ZLPW.js";import"./chunk-2VAKOPKD.js";import{a as q,b as et}from"./chunk-24LNMKH3.js";import"./chunk-U3VV32SL.js";import"./chunk-5VIETITT.js";import"./chunk-4ESJQPEA.js";import"./chunk-KA2SPUWT.js";import"./chunk-KLB4BWDP.js";import"./chunk-YINSOKTV.js";import"./chunk-OW2F6GOM.js";import"./chunk-LDPS3LK4.js";import{a as L,b as z,c as H}from"./chunk-RDKNUQF3.js";import{a as $,b as tt}from"./chunk-RIBYI2VQ.js";import"./chunk-FVJGYA76.js";import"./chunk-MRFU7ZH5.js";import"./chunk-4WJQGABW.js";import"./chunk-MAQNKK2A.js";import"./chunk-XMICNJPL.js";import"./chunk-MAVIT7QG.js";import"./chunk-6Q7YU3GA.js";import"./chunk-7YUL6BIM.js";import"./chunk-A3Y77RII.js";import"./chunk-5DATZV72.js";import"./chunk-3RHWAFOM.js";import"./chunk-EXSN6QWV.js";import"./chunk-QUK73RJT.js";import"./chunk-R2ULAEP2.js";import"./chunk-ZSHAELBT.js";import"./chunk-2FOKRXOS.js";import"./chunk-UGQXGBKF.js";import{C as M,E,g as k,i as I,n as A}from"./chunk-TCUK52QT.js";import{a as B,b as Z}from"./chunk-JU5LF7AN.js";import"./chunk-AFLM5QFV.js";import"./chunk-DCQJQBJ7.js";import{xa as j,ya as Y}from"./chunk-VYRWAFSM.js";import{a as Q,b as X}from"./chunk-LFKM2JDJ.js";import"./chunk-QQZCALHS.js";import"./chunk-M55KBHC5.js";import"./chunk-JE2X3YFA.js";import"./chunk-HW6LKGMV.js";import"./chunk-XRGTMJIH.js";import"./chunk-FMPRHYG5.js";import{Wa as O,Xa as x}from"./chunk-GDJMZZ4C.js";import"./chunk-V3J5YJ3L.js";import"./chunk-UIVS6623.js";import"./chunk-2XIOYYFS.js";import"./chunk-HYPRVELK.js";import{b as N,c as K}from"./chunk-YTY7HHO3.js";import"./chunk-UOI75ONS.js";import{Ab as l,Ac as C,Cc as b,Fb as p,Gc as F,J as a,Ja as c,Ka as g,Kc as y,Qb as f,Rc as P,Rd as J,Sc as D,Tc as w,Ub as h,Xc as _,ia as m,jc as o,na as W,ud as T,vd as V,wc as s,x as u,xc as d,yc as v,zc as S}from"./chunk-4HFZZ7HS.js";import{e as U}from"./chunk-SZAW6ZBW.js";var nt,yt,ot=U(()=>{W();E();et();x();X();it();Y();H();rt();Z();J();K();tt();H();E();nt=["currentForm"],yt=(()=>{class n extends q{constructor(t,e){super(),this.translatorService=t,this.pamSettingsService=e,this.titleHeader=[{title:this.translatorService.translate("lblAdministration"),link:"/administration/"},{title:this.translatorService.translate("lblPrivilegedAccessManagement"),link:"/administration/pam/"},{title:this.translatorService.translate("lblDefaultSettings")}],this.settings=new O,this.isFormLoaded=!1}get isDirty(){return this.currentForm?this.isFormLoaded&&this.currentForm.dirty:!1}getComponentTitle(){return"lblDefaultSettings"}ngOnInit(){super.ngOnInit()}ngAfterViewInit(){this.getSettings()}save(){return this.onSubmit().subscribe(()=>this.setFormPristine())}cancelChange(){return!1}rejectChange(){return!0}validateChange(){return this.onSubmit()}onSubmit(){return this.pamSettingsService.setSettings(this.settings).pipe(m(this.destroyed$),a(t=>!!t),u(t=>!!t))}getSettings(){this.pamSettingsService.getSettings(!0).pipe(m(this.destroyed$),a(t=>!!t)).subscribe(t=>{this.settings=t,this.setFormPristine(),this.isFormLoaded=!0})}setFormPristine(){setTimeout(()=>{this.currentForm?.form?.markAsPristine(),this.currentForm?.form?.updateValueAndValidity()})}static{this.\u0275fac=function(e){return new(e||n)(p(N),p($))}}static{this.\u0275cmp=f({type:n,selectors:[["dvls-pam-settings-dashboard"]],viewQuery:function(e,i){if(e&1&&P(nt,5),e&2){let r;D(r=w())&&(i.currentForm=r.first)}},features:[h],decls:8,vars:8,consts:[["currentForm","ngForm"],[3,"titlePage"],["header-right",""],["variant","icon","showDelay","500","tooltipPosition","bottom","icon","di dvl-icon-save",3,"onClick","dvlTooltip","disabled"],["autocomplete","off","novalidate","",1,"full-height"],[3,"settings","form","mdColSize"]],template:function(e,i){if(e&1){let r=b();s(0,"dvl-page-wrapper-v2",1),S(1,2),s(2,"dvl-button",3),T(3,"translate"),F("onClick",function(){return c(r),g(i.save())}),d(),C(),s(4,"form",4,0)(6,"p-fieldset"),v(7,"dps-pam-settings",5),d()()()}if(e&2){let r=_(5);o("titlePage",i.titleHeader),l(2),y("dvlTooltip",V(3,6,"lblSave")),o("disabled",!i.isDirty),l(5),o("settings",i.settings)("form",r)("mdColSize",12)}},dependencies:[Q,G,B,j,z,L,M,A,k,I,R],encapsulation:2})}}return n})()});ot();export{yt as PamSettingsDashboardComponent};
