import{a as o,b as p}from"./chunk-SUSYLW52.js";import{a as m,c as s}from"./chunk-VHKJVCTZ.js";import{a as i,b as n}from"./chunk-CXJOJS5Y.js";import"./chunk-ILBO5ZGN.js";import"./chunk-ZSHAELBT.js";import"./chunk-MWQIPWDG.js";import"./chunk-UTF7W4HP.js";import"./chunk-OJRDBZWF.js";import"./chunk-6G52AAKU.js";import"./chunk-QQZCALHS.js";import"./chunk-EXP5MDOK.js";import"./chunk-M55KBHC5.js";import"./chunk-JE2X3YFA.js";import"./chunk-HW6LKGMV.js";import{Yb as t,Zb as e}from"./chunk-GDJMZZ4C.js";import"./chunk-V3J5YJ3L.js";import"./chunk-UIVS6623.js";import"./chunk-2XIOYYFS.js";import"./chunk-HYPRVELK.js";import"./chunk-YTY7HHO3.js";import"./chunk-UOI75ONS.js";import"./chunk-4HFZZ7HS.js";import{e as a}from"./chunk-SZAW6ZBW.js";var A,d=a(()=>{n();p();s();e();A=[{path:"",children:[{path:"",canActivate:[i,o,m],data:{permissions:[t.PamSettingsJitTemplatesView]},loadComponent:()=>import("./chunk-SOELGHVG.js").then(r=>r.SshJitTemplatesDashboardComponent)}]}]});d();export{A as SSH_JIT_TEMPLATES_ROUTES};
