import{i as a,j as o}from"./chunk-2JML2BCE.js";import{e}from"./chunk-SZAW6ZBW.js";var i=e(()=>{o();a.setLanguage(["gl","gl-ES"],{Markdown:"Markdown",WYSIWYG:"WYSIWYG",Write:"Escribir",Preview:"Vista previa",Headings:"Encabezados",Paragraph:"Par\xE1grafo",Bold:"Negri\xF1a",Italic:"Cursiva",Strike:"Riscado",Code:"C\xF3digo",Line:"Li\xF1a",Blockquote:"Cita","Unordered list":"Lista desordenada","Ordered list":"Lista ordenada",Task:"Tarefa",Indent:"Sangr\xEDa",Outdent:"Anular sangr\xEDa","Insert link":"Inserir enlace","Insert CodeBlock":"Inserir bloque de c\xF3digo","Insert table":"Inserir t\xE1boa","Insert image":"Inserir imaxe",Heading:"Encabezado","Image URL":"URL da imaxe","Select image file":"Seleccionar arquivo da imaxe","Choose a file":"Escoge un archivo","No file":"Ning\xFAn archivo",Description:"Descrici\xF3n",OK:"Aceptar",More:"M\xE1is",Cancel:"Cancelar",File:"Arquivo",URL:"URL","Link text":"Texto do enlace","Add row to up":"Engade fila para arriba","Add row to down":"Engade fila para abaixo","Add column to left":"Engade columna \xE1 esquerda","Add column to right":"Engade columna \xE1 dereita","Remove row":"Eliminar fila","Remove column":"Eliminar columna","Align column to left":"Ali\xF1ar \xE1 esquerda","Align column to center":"Centrar","Align column to right":"Ali\xF1ar \xE1 dereita","Remove table":"Eliminar t\xE1boa","Would you like to paste as table?":"Desexa pegar como t\xE1boa?","Text color":"Cor do texto","Auto scroll enabled":"Desprazamento autom\xE1tico habilitado","Auto scroll disabled":"Desprazamento autom\xE1tico deshabilitado","Choose language":"Elixir idioma"})});i();
