import{Qb as n,Rd as i}from"./chunk-4HFZZ7HS.js";import{e as r}from"./chunk-SZAW6ZBW.js";var c,a=r(()=>{i();c=(()=>{class e{static{this.\u0275fac=function(t){return new(t||e)}}static{this.\u0275cmp=n({type:e,selectors:[["ng-component"]],decls:0,vars:0,template:function(t,m){},encapsulation:2})}}return e})()});a();export{c as PamTreeContainerComponent};
