import{a as b,b as D}from"./chunk-LFKM2JDJ.js";import{a as I,b as F}from"./chunk-UIVS6623.js";import{K as x,m as C}from"./chunk-UOI75ONS.js";import{Ab as a,Na as u,Qb as f,Rd as y,Ub as h,Zc as s,_c as l,jc as g,ld as v,sa as m,ud as p,vd as c,wc as o,xc as r}from"./chunk-4HFZZ7HS.js";import{e as d}from"./chunk-SZAW6ZBW.js";var S,E=d(()=>{y();S=(()=>{class e{copy(n){let t=document.createElement("textarea");t.style.position="absolute",t.style.left="10000px",document.body.appendChild(t),t.value=n,t.select(),document.execCommand("copy"),document.body.removeChild(t)}static{this.\u0275fac=function(t){return new(t||e)}}static{this.\u0275prov=m({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});var j,P,M=d(()=>{F();x();D();y();j=e=>({"data-label-highlight":e}),P=(()=>{class e extends I{constructor(){super(...arguments),this.highlight=!1,this.divider=!1,this.translate=!1}static{this.\u0275fac=(()=>{let n;return function(i){return(n||(n=u(e)))(i||e)}})()}static{this.\u0275cmp=f({type:e,selectors:[["dvls-diagnostic-information"]],inputs:{label:"label",value:"value",highlight:"highlight",divider:"divider",translate:"translate"},features:[h],decls:9,vars:9,consts:[[1,"grid","grid-cols-12","gap-4"],[1,"col-span-12","md:col-span-5"],[3,"ngClass"],[1,"col-span-12","md:col-span-7"]],template:function(t,i){t&1&&(o(0,"div",0)(1,"div",1)(2,"label",2),s(3),p(4,"translate"),r()(),o(5,"div",3)(6,"span"),s(7),p(8,"translate"),r()()()),t&2&&(a(2),g("ngClass",v(7,j,i.highlight)),a(),l(c(4,3,i.label)),a(4),l(i.translate?c(8,5,i.value):i.value))},dependencies:[C,b],encapsulation:2})}}return e})()});export{P as a,M as b,S as c,E as d};
