import{c as e,d as r}from"./chunk-V3J5YJ3L.js";import{e as a}from"./chunk-SZAW6ZBW.js";var o,i=a(()=>{r();o=class t{toQueryString(){let s=[];if(this.changePasswordID!==null&&this.changePasswordID.toString()&&s.push("changePasswordID="+encodeURIComponent(this.changePasswordID.toString())),this.newPassword!==null&&this.newPassword.toString()&&s.push("newPassword="+encodeURIComponent(this.newPassword.toString())),this.oldPassword!==null&&this.oldPassword.toString()&&s.push("oldPassword="+encodeURIComponent(this.oldPassword.toString())),this.safeNewPassword!==null&&this.safeNewPassword.toString()&&s.push("safeNewPassword="+encodeURIComponent(this.safeNewPassword.toString())),this.safeOldPassword!==null&&this.safeOldPassword.toString()&&s.push("safeOldPassword="+encodeURIComponent(this.safeOldPassword.toString())),this.unknownProperties)for(let n=0;n<this.unknownProperties.length;n++)this.unknownProperties[n]!==null&&s.push("unknownProperties="+encodeURIComponent(this.unknownProperties[n].toString()));return s.join("&")}constructor(s=null){this.clone=()=>new t(this),this.changePasswordID=e.manageDefaultValue(s,"changePasswordID",null),this.newPassword=e.manageDefaultValue(s,"newPassword",""),this.oldPassword=e.manageDefaultValue(s,"oldPassword",""),this.safeNewPassword=e.manageDefaultValue(s,"safeNewPassword",""),this.safeOldPassword=e.manageDefaultValue(s,"safeOldPassword",""),this.unknownProperties=e.manageDefaultArrayValue(s,"unknownProperties",null)}}});export{o as a,i as b};
