import{wa as a,ya as m}from"./chunk-VYRWAFSM.js";import{Fb as s,Gc as p,Rd as f,Sb as d,Ya as r}from"./chunk-4HFZZ7HS.js";import{e as c}from"./chunk-SZAW6ZBW.js";var I,h=c(()=>{f();m();I=(()=>{class i{constructor(e,t){this.tooltip=e,this.elementRef=t,this.elementIsContainedInADiv=!1}setTooltipState(){let e=this.elementRef.nativeElement,t=e.parentElement,n=window.getComputedStyle(t),l=t.clientWidth-parseInt(n.paddingLeft,10)-parseInt(n.paddingRight,10),o=this.elementIsContainedInADiv?e.scrollWidth:e.offsetWidth;e&&t&&o>l?this.tooltip._tooltipOptions.disabled=!1:this.tooltip._tooltipOptions.disabled=!0}static{this.\u0275fac=function(t){return new(t||i)(s(a),s(r))}}static{this.\u0275dir=d({type:i,selectors:[["","pTooltip","","dvlsTextOverflowTooltipDirective",""]],hostBindings:function(t,n){t&1&&p("mouseenter",function(o){return n.setTooltipState(o)})},inputs:{elementIsContainedInADiv:"elementIsContainedInADiv"}})}}return i})()});export{I as a,h as b};
