import{Rd as c,g as t,ma as a,sa as i}from"./chunk-4HFZZ7HS.js";import{e as s}from"./chunk-SZAW6ZBW.js";var b,l=s(()=>{a();c();b=(()=>{class e{constructor(){this.visibleRequestSource=new t,this.dialogErrorSource=new t,this.visibleRequest=this.visibleRequestSource.asObservable(),this.dialogError=this.dialogErrorSource.asObservable()}toggleDialog(){this.visibleRequestSource.next()}sendValue(r){this.dialogErrorSource.next(r),this.toggleDialog()}static{this.\u0275fac=function(o){return new(o||e)}}static{this.\u0275prov=i({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});export{b as a,l as b};
