import{Rd as s,g as o,ma as a,sa as r}from"./chunk-4HFZZ7HS.js";import{e as c}from"./chunk-SZAW6ZBW.js";var l,n=c(()=>{a();s();l=(()=>{class t{constructor(){this.showCertificateDialogSource=new o,this.showCertificateDialogObservable=this.showCertificateDialogSource.asObservable()}show(i,e){this.showCertificateDialogSource.next({certificate:i,acceptFunction:e})}static{this.\u0275fac=function(e){return new(e||t)}}static{this.\u0275prov=r({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});export{l as a,n as b};
