import{a as v,b as S}from"./chunk-BJZVALMW.js";import{a,b as m}from"./chunk-DY655T2D.js";import{Rd as n,g as p,ma as u,sa as i,xa as s}from"./chunk-4HFZZ7HS.js";import{a as o,e as c}from"./chunk-SZAW6ZBW.js";var l,f=c(()=>{u();n();l=(()=>{class e{constructor(){this.viewSubject=new p,this.viewRequest=this.viewSubject.asObservable()}view(t){this.viewSubject.next(t)}static{this.\u0275fac=function(r){return new(r||e)}}static{this.\u0275prov=i({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});var I,h=c(()=>{m();S();n();f();I=(()=>{class e extends a{constructor(t){super(),this.providerTypeSelectorService=t}create(t){t.type?super.create(o({data:t.type,options:{providerTemplate:t.providerTemplate}},t)):this.providerTypeSelectorService.view({acceptProvider:(r,d)=>super.create(o({data:r,options:{providerTemplate:d}},t)),selectorType:v.Provider})}static{this.\u0275fac=function(r){return new(r||e)(s(l))}}static{this.\u0275prov=i({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});export{l as a,f as b,I as c,h as d};
