import{e as p}from"./chunk-SZAW6ZBW.js";var e,C=p(()=>{e=function(d){return d[d.Created=0]="Created",d[d.Deleted=1]="Deleted",d[d.Executed=2]="Executed",d[d.Modified=3]="Modified",d[d.Viewed=4]="Viewed",d[d.PasswordViewed=5]="PasswordViewed",d[d.PasswordChanged=6]="PasswordChanged",d[d.Copied=17]="Copied",d[d.Opened=18]="Opened",d[d.Exported=19]="Exported",d[d.Synchronized=20]="Synchronized",d[d.ResetPasswordScheduleAdded=21]="ResetPasswordScheduleAdded",d[d.ResetPasswordScheduleDeleted=22]="ResetPasswordScheduleDeleted",d[d.InvalidValue=23]="InvalidValue",d[d.DevolutionsGatewayDown=24]="DevolutionsGatewayDown",d[d.Closed=25]="Closed",d[d.Expired=14]="Expired",d[d.Expiring=38]="Expiring",d}(e||{});(function(d){function l(){return"NotificationEventActionEntry"}d.getClass=l;function u(w){let s="";return typeof d.getClass=="function"&&(s=d.getClass()),s+"_"+d[w]}d.getEnumKey=u})(e||(e={}))});export{e as a,C as b};
